//AstroCalc.cpp
//
//This is the interface between the main program and the Swiss Ephemeris.
//To use another astrology library, edit this file and Translation.cpp/.h,
//It also simplifies the use of the Swiss Ephemeris considerably, by
//encapsulating all the temporary variables needed when calling.
//
//Copyright (C) 2003 mikesag
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include "AstroConst.h"
#include "Translation.h"

#include "AstroCalc.h"

#include "sweph/swephexp.h"
#include "sweph/swemisc.h"

#include "wx/datetime.h"

#define MINS_IN_HOUR 60.0
#define SECS_IN_HOUR 3600.0

#define MINS_IN_DEG  MINS_IN_HOUR
#define SECS_IN_DEG  SECS_IN_HOUR

static int node_type = USE_TRUE;
static int apog_type = USE_TRUE;

static int  err_place;  // meaning varies depending on function
static bool was_err;
static char err_msg[AS_MAXCH];

bool ac_error_occured()
{
	return was_err;
}

int ac_find_error()
{
	return err_place;
}

char *ac_error_msg()
{
	return err_msg;
}

void set_node_type(int n)
{
	node_type = n;
}

int  get_node_type()
{
	return (node_type == USE_MEAN) ? USE_MEAN : USE_TRUE;
}

void set_apog_type(int n)
{
	apog_type = n;
}

int  get_apog_type()
{
	return (apog_type == USE_MEAN) ? USE_MEAN : USE_TRUE;
}

void init_astro_library(char *argv0)
{
	make_ephemeris_path(0, argv0);
	set_node_type(USE_TRUE);
	set_apog_type(USE_MEAN);
}

void close_astro_library()
{
	swe_close();
}

void split_longitute(double longitude, trSignPos *pos, int round_min)
{
#ifndef _DEBUG  /* We _want_ it to crash in debug mode! */
	if(pos == NULL) return;  /* Prevent crashes on null pointers */
#endif
	pos->sign    = IntToSign((int)(longitude / 30) + 1);
	pos->degrees = (int)(fmod(longitude, 30.0));
	pos->minutes = (int)floor((fmod(longitude, 30.0) - pos->degrees)*60.0);
	pos->seconds = (((fmod(longitude, 30.0) - pos->degrees)*60.0) - pos->minutes)*60.0;
	if(round_min)
	{
		if(pos->seconds > 30.0)
			pos->minutes++;
		if(pos->minutes == 60)
		{
			pos->minutes = 0;
			pos->degrees++;
		}
		if(pos->degrees == 30)
		{
			pos->degrees = 0;
			pos->sign++;
		}
		if(pos->sign == (PISCES + 1))
			pos->sign = ARIES;
	}
}

void split_dms(double x, trDMS *dms)
{
	int is_neg;
	is_neg = 0;
	if(x < 0.0)
	{
		x = -x;
		is_neg = 1;
	}
	dms->deg = (int)(floor(x));
	dms->min = (int)(floor((x - dms->deg)*60.0));
	dms->sec = (((x - dms->deg)*60.0) - dms->min)*60.0;
	if(is_neg)
	{
		if(dms->deg != 0)
			dms->deg = -dms->deg;
		else if(dms->min != 0)
			dms->min = -dms->min;
		else
			dms->sec = -dms->sec;
	}
}

double merge_dms(trDMS *dms)
{
#ifndef _DEBUG
	if(dms == NULL) return 0.0;
#endif
	if(dms->deg > 0)
		return ((double)dms->deg+dms->min/MINS_IN_DEG+dms->sec/SECS_IN_DEG);
	else if(dms->deg < 0)
		return -((double)(-dms->deg) + dms->min/MINS_IN_DEG +
			dms->sec/SECS_IN_DEG);
	else /* dms->deg == 0 */
	{
		if(dms->min > 0)
			return ((double)dms->min/MINS_IN_DEG + dms->sec/SECS_IN_DEG);
		else if(dms->min < 0)
			return -((double)(-dms->min)/MINS_IN_DEG + dms->sec/SECS_IN_DEG);
		else /* dms->min == 0 */
			return dms->sec/SECS_IN_DEG;
	}
}

void midpoint_dms(trDMS *dms1, trDMS *dms2, trDMS *midpt)
{
	double c1, c2, c3;
	c1 = merge_dms(dms1);
	c2 = merge_dms(dms2);
	c3 = (c1 + c2)/2.0;
	split_dms(c3, midpt);
}

void midpoint_tdp(trTimeDatePlace *tdp1, trTimeDatePlace *tdp2, trTimeDatePlace *midtdp)
{
	wxDateTime wxdt(wxDateTime::Now());
	double jd1, jd2, jd3;

	jd1 = to_juldate(tdp1);
	jd2 = to_juldate(tdp2);
	jd3 = (jd1 + jd2)/2.0;
	/* from_juldate(jd3, 0, FALSE, &(chartdata[CHART_DAVISON])); */
	from_juldate(jd3,
		(tdp1->timezone + tdp2->timezone)/2,
		FALSE, midtdp);
	wxdt.Set(midtdp->date.day,
		(wxDateTime::Month)(wxDateTime::Jan + midtdp->date.month - 1),
		midtdp->date.year);
	if(wxdt.IsDST())
		make_dst(midtdp);
	midpoint_dms(&(tdp1->latitude),
		&(tdp2->latitude), &(midtdp->latitude));
	midpoint_dms(&(tdp1->longitude),
		&(tdp2->longitude), &(midtdp->longitude));
}

void midpoint_planets(trPlanet *pl1, trPlanet *pl2, trChartInfo *midchart)
{
	int i, j;

	for(i = 0; i < NTOTOBJECTS; i++)
	{
		midchart->planet[i].longitude =
			swe_deg_midp(pl1[i].longitude, pl2[i].longitude);

		split_longitute(midchart->planet[i].longitude,
			&(midchart->planet[i].sgn), FALSE);

		midchart->planet[i].long_speed =
			(pl1[i].long_speed +
			pl2[i].long_speed)/2.0;
		
		midchart->planet[i].latitude =
			(pl1[i].latitude +
			pl2[i].latitude)/2.0;

		midchart->planet[i].lat_speed =
			(pl1[i].lat_speed +
			pl2[i].lat_speed)/2.0;

		midchart->planet[i].dist_speed =
			(pl1[i].dist_speed +
			pl2[i].dist_speed)/2.0;
		
		for(j = 1; j <= 12; j++)
		{
			if(midchart->house[j].cusp > midchart->house[j+1].cusp)
			{
				// Special case: when the house crosses 0 Aries
				if((midchart->planet[i].longitude > midchart->house[j].cusp) ||
					(midchart->planet[i].longitude < midchart->house[j+1].cusp))
				{
					midchart->planet[i].in_house = j;
					break;
				}
				
			}
			// Otherwise, search for house position normally
			else if((midchart->planet[i].longitude > midchart->house[j].cusp) &&
				(midchart->planet[i].longitude < midchart->house[j+1].cusp))
			{
				midchart->planet[i].in_house = j;
				break;
			}
		}
	}
}

void progress_tdp(trTimeDatePlace *tdp_orig, trTimeDatePlace *tdp_now,
				  trTimeDatePlace *tdp_progressed)
{
	/* Something about 59.08...? */

	double jd_orig, jd_now, jd_delta, jd_progressed;

	jd_orig = to_juldate(tdp_orig);
	jd_now  = to_juldate(tdp_now);
	
	jd_delta = jd_now - jd_orig;
	jd_progressed = jd_delta/JD_DAYS_IN_YEAR + jd_orig;

	from_juldate(jd_progressed, tdp_orig->timezone, tdp_orig->is_dst,
	             tdp_progressed);

	tdp_progressed->is_progressed = 1;
	tdp_progressed->latitude  = tdp_orig->latitude;
	tdp_progressed->longitude = tdp_orig->longitude;
	tdp_progressed->is_dst = tdp_orig->is_dst;
}

void negate_dms(trDMS *dms)
{
	if(dms->deg != 0)
		dms->deg = -dms->deg;
	else if(dms->min != 0)
		dms->min = -dms->min;
	else
		dms->sec = -dms->sec;
}

int is_negative_dms(const trDMS *dms)
{
	if(dms->deg != 0)
		return (dms->deg < 0);
	else if(dms->min != 0)
		return (dms->min < 0);
	else
		return (dms->sec < 0);
}

static int days_in_month(int month, int year)
{
	int leap_year;
	
	leap_year = ((year % 4 == 0) && (year % 400 != 0)) ? 1 : 0;

	switch(month)
	{
	case 1:  return 31;
	case 2:  return 28 + leap_year;
	case 3:  return 31;
	case 4:  return 30;
	case 5:  return 31;
	case 6:  return 30;
	case 7:  return 31;
	case 8:  return 31;
	case 9:  return 30;
	case 10: return 31;
	case 11: return 30;
	case 12: return 31;
	}

	return 15; /* something obviously wrong for an error value */
}

void correct_for_dst(trTimeDatePlace *tdp) /* edits in-place */
{
	if(!tdp->is_dst) return;

	tdp->time.hours -= 1;
	if(tdp->time.hours < 0)
	{
		tdp->time.hours += 24;
		tdp->date.day   -= 1;
		if(tdp->date.day < 1)
		{
			tdp->date.month -= 1;
			if(tdp->date.month < 1)
			{
				tdp->date.month = 12;
				tdp->date.year -= 1;
			}
			tdp->date.day = days_in_month(tdp->date.month, tdp->date.year);
		}
	}

	tdp->is_dst = 0;
}

void make_dst(trTimeDatePlace *tdp) /* edits in-place */
{
	if(tdp->is_dst) return;

	tdp->time.hours += 1;
	if(tdp->time.hours >= 24)
	{
		tdp->time.hours -= 24;
		tdp->date.day   += 1;
		if(tdp->date.day > days_in_month(tdp->date.month, tdp->date.year))
		{
			tdp->date.month += 1;
			if(tdp->date.month > 12)
			{
				tdp->date.month = 1;
				tdp->date.year += 1;
			}
		}
	}

	tdp->is_dst = 1;
}

double calc_juldate(int y, int mon, int d, int h, int min, double sec,
					int timezone, int is_dst)
{
	double base_jd;
	base_jd = swe_julday(y, mon, d, h + ((double)min/MINS_IN_HOUR) +
		sec/SECS_IN_HOUR, SE_GREG_CAL);
	return (base_jd - ((timezone + (is_dst ? 1 : 0))/24.0));
}

void convert_juldate(double juldate, int timezone, int is_dst,
					 int *y, int *mon, int *d, int *h, int *min, double *sec)
{
	double jut, minf;
	juldate += (timezone + (is_dst ? 1 : 0))/24.0;
	swe_revjul(juldate, SE_GREG_CAL, y, mon, d, &jut);
	*h   = (int)(floor(jut));
	*min = (int)(floor((jut - *h)*60.0));
	minf = (jut - *h)*60.0;
	*sec = (int)(floor((minf - *min)*60.0));
}

double to_juldate(trTimeDatePlace *tdp)
{
	return calc_juldate(tdp->date.year, tdp->date.month, tdp->date.day,
		tdp->time.hours, tdp->time.min, tdp->time.sec, tdp->timezone, tdp->is_dst);
}

void from_juldate(double juldate, int timezone, int is_dst,
				  trTimeDatePlace *tdp)
{
	tdp->timezone = timezone;
	tdp->is_dst   = is_dst;
	convert_juldate(juldate, timezone, is_dst, &(tdp->date.year),
		&(tdp->date.month), &(tdp->date.day), &(tdp->time.hours),
		&(tdp->time.min), &(tdp->time.sec));
}







void astro_calc(trTimeDatePlace *tdp, int hsys, trChartInfo *pinf)
{
#ifndef _DEBUG
	if((tdp == NULL) || (pinf == NULL)) return;
#endif

	get_houses(tdp, hsys, pinf->house, &(pinf->armc), &(pinf->sidereal_time));
	get_planets_no_houses(tdp, pinf->planet);
	fill_planets_in_houses(pinf, hsys);
}

void astro_calc_progressed(trTimeDatePlace *tdp1, trTimeDatePlace *tdp2,
						   trTimeDatePlace *tdppr, int hsys, trChartInfo *pinf)
{
#ifndef _DEBUG
	if((tdp1 == NULL) || (tdp2 == NULL) || (tdppr == NULL) || (pinf == NULL))
		return;
#endif

	int i, ii;
	trHousePos h1[HOUSEARRSIZE];
	double armc, armc1, sidt, sidt1, deltat, eps;

	if(hsys != NULL_HOUSES)
	{
		get_houses(tdp1, hsys, h1, &armc1, &sidt1);
		
		eps = get_ecliptic(tdppr);
		
		deltat = to_juldate(tdp2) - to_juldate(tdp1);
		
		armc = armc1 + (deltat*360.0)/(SQ(JD_DAYS_IN_YEAR));
		while(armc > 360.0)
			armc -= 360.0;
		while(armc < 0.0)
			armc += 360.0;
		sidt = armc/15.0;
		pinf->armc = armc;
		pinf->sidereal_time = sidt;
		get_houses_ex(tdp1, hsys, pinf->house, 1, &armc, &sidt,
			merge_dms(&(tdp1->latitude)), eps);
	}
	else /* hsys == NULL */
	{
		for(i = 1; i <= 13; i++)
		{
			ii = (i != 13) ? i : 1;
			pinf->house[i].cusp = 30.0*(i - 1);
			pinf->house[i].hsystem = hsys;
			split_longitute(pinf->house[i].cusp, &(pinf->house[i].sgn), FALSE);
		}
	}

	get_planets_no_houses(tdppr, pinf->planet);
	fill_planets_in_houses(pinf, hsys);
}

void astro_calc_composite(trTimeDatePlace *tdp1, trTimeDatePlace *tdp2,
						  double armc1, double armc2,
						  int hsys, trChartInfo *pinf)
{
#ifndef _DEBUG
	if((tdp1 == NULL) || (tdp2 == NULL) || (pinf == NULL)) return;
#endif

	trPlanet pl1[NTOTOBJECTS], pl2[NTOTOBJECTS];
	trHousePos h1[HOUSEARRSIZE], h2[HOUSEARRSIZE];
	double armc, tarmc1, tarmc2, sidt, sidt1, sidt2;
	double eps, eps1, eps2, lat;

	get_houses(tdp1, hsys, h1, &tarmc1, &sidt1);
	get_houses(tdp2, hsys, h2, &tarmc2, &sidt2);
	get_planets_no_houses(tdp1, pl1);
	get_planets_no_houses(tdp2, pl2);
	eps1 = get_ecliptic(tdp1);
	eps2 = get_ecliptic(tdp2);

	pinf->armc = armc = swe_deg_midp(armc1, armc2);
	pinf->sidereal_time = sidt = (sidt1 + sidt2)/2.0;
	eps = (eps1 + eps2)/2.0;
	lat = (merge_dms(&(tdp1->latitude)) +
	       merge_dms(&(tdp2->latitude)))/2.0;
	get_houses_ex(tdp1, hsys, pinf->house, 1, &armc, &sidt, lat, eps);

	midpoint_planets(pl1, pl2, pinf);

	fill_planets_in_houses(pinf, hsys);
}

void get_planets_no_houses(trTimeDatePlace *tdp, trPlanet pl[NTOTOBJECTS])
{
#ifndef _DEBUG
	if((tdp == NULL) || (pl == NULL)) return;
#endif

	trTimeDatePlace *my_tdp = tdp;

	int i, se_i, se_ret;
	double ftim, jul_ut;
	static double pldata[6];

	strcpy(err_msg, "");
	was_err = false;

	if(tdp->is_dst)
	{
		my_tdp = new trTimeDatePlace;
		*my_tdp = *tdp;
		correct_for_dst(my_tdp);
	}
	ftim = my_tdp->time.hours + ((double)(my_tdp->time.min)/MINS_IN_HOUR) +
		((double)(tdp->time.sec)/SECS_IN_HOUR);
	jul_ut = swe_julday(my_tdp->date.year, my_tdp->date.month, my_tdp->date.day,
		ftim, SE_GREG_CAL) - (my_tdp->timezone/24.0);

	for(i = 0; i < NTOTOBJECTS; i++)
	{
		se_i = ToSwissEph(SUN+i);
		if((i + SUN) == NNODE)
		{
			se_i = ToSwissEph((get_node_type() == USE_MEAN) ?
				MEAN_NODE : TRUE_NODE);
		}
		else if((i + SUN) == LILITH)
		{
			se_i = ToSwissEph((get_apog_type() == USE_MEAN) ?
				MEAN_APOG : OSCU_APOG);
		}
		se_ret = swe_calc_ut(jul_ut, se_i, SEFLG_SPEED, pldata, err_msg);
		if(se_ret < 0)
		{
			was_err = true;
			err_place = SUN + i;
			/*if(tdp->is_dst) delete my_tdp;*/
			/*return;*/
		}
		pl[i].longitude  = pldata[0];
		pl[i].latitude   = pldata[1];
		pl[i].dist       = pldata[2];
		pl[i].long_speed = pldata[3];
		pl[i].lat_speed  = pldata[4];
		pl[i].dist_speed = pldata[5];
		split_longitute(pl[i].longitude, &(pl[i].sgn), FALSE);
	}

	if(tdp->is_dst) delete my_tdp;
}

void get_houses(trTimeDatePlace *tdp, int hsys, trHousePos house[HOUSEARRSIZE],
				double *armc, double *sidereal_time)
{
	get_houses_ex(tdp, hsys, house, 0, armc, sidereal_time, 0.0, 0.0);
}

void get_houses_ex(trTimeDatePlace *tdp, int hsys, trHousePos house[HOUSEARRSIZE],
                   int by_armc, double *armc, double *sidereal_time,
				   double latitude, double eps)
{
#ifndef _DEBUG
	if((tdp == NULL) || (house == NULL)) return;
#endif

	trTimeDatePlace *my_tdp = tdp;

	int i, ii, se_h, se_ret;
	double ftim, jul_ut, sidt;
	static double se_house[13], se_ascmc[10];
	double lat, lon;

#ifndef _DEBUG
	if(by_armc && (armc == NULL))
	{
		was_err = true;
		strcpy(err_msg, "ARMC required but not supplied.");
		err_place = 0; /* there is no indication where the error occured here */
		return;
	}
#endif

	if(!IS_HSYSTEM(hsys))
	{
		was_err = true;
		err_place = hsys;
		strcpy(err_msg, "Invalid house system!");
		return;
	}

	if(hsys == NULL_HOUSES)
	{
		for(i = 1; i <= 13; i++)
		{
			ii = (i != 13) ? i : 1;
			house[i].cusp = 30.0*(i - 1);
			house[i].hsystem = hsys;
			split_longitute(house[i].cusp, &(house[i].sgn), FALSE);
		}
		return;
	}

	strcpy(err_msg, "");
	was_err = false;

	if(tdp->is_dst)
	{
		my_tdp = new trTimeDatePlace;
		*my_tdp = *tdp;
		correct_for_dst(my_tdp);
	}
	ftim = my_tdp->time.hours + ((double)(my_tdp->time.min)/MINS_IN_HOUR) +
		((double)(tdp->time.sec)/SECS_IN_HOUR);
	jul_ut = swe_julday(my_tdp->date.year, my_tdp->date.month, my_tdp->date.day,
		ftim, SE_GREG_CAL) - (my_tdp->timezone/24.0);	

	se_h = ToSwissEph(hsys);
	lat = merge_dms(&(my_tdp->latitude));
	lon = merge_dms(&(my_tdp->longitude));

	sidt = swe_sidtime(jul_ut) + fabs(lon)/15.0;
	if(sidt >= 24) sidt -= 24;
	if(sidt < 0)   sidt += 24;
#ifndef _DEBUG
	if(sidereal_time != NULL)  /* only do the NULL-check in non-debug mode */
#endif
		*sidereal_time = sidt;

	if(!by_armc)
	{
		se_ret = swe_houses(jul_ut, lat, lon, se_h, se_house, se_ascmc);
		if(se_ret < 0)
		{
			/* Try Porphyry houses in case the problem was too great a latitude */
			se_ret = swe_houses(jul_ut, lat, lon, ToSwissEph(PORPHYRY), se_house, se_ascmc);
			if(se_ret < 0)
			{
				was_err = true;
				strcpy(err_msg, "Error calculating houses");
				err_place = 0; /* there is no indication where the error occured here */
				if(tdp->is_dst) delete my_tdp;
				return;
			}
		}
	}
	else /* by_armc */
	{
		se_ret = swe_houses_armc(*armc, latitude, eps, se_h, se_house, se_ascmc);
		if(se_ret < 0)
		{
			/* Try Porphyry houses in case the problem was too great a latitude */
			se_ret = swe_houses_armc(*armc, latitude, eps, ToSwissEph(PORPHYRY),
					se_house, se_ascmc);
			if(se_ret < 0)
			{
				was_err = true;
				strcpy(err_msg, "Error calculating houses");
				err_place = 0; /* there is no indication where the error occured here */
				if(tdp->is_dst) delete my_tdp;
				return;
			}
		}
	}
	
	for(i = 1; i <= 13; i++)
	{
		ii = (i != 13) ? i : 1;
		house[i].cusp = se_house[ii];
		house[i].hsystem = hsys;
		split_longitute(house[i].cusp, &(house[i].sgn), FALSE);
	}

#ifndef _DEBUG
	if(armc != NULL)  /* only do the NULL-check in non-debug mode */
#endif
		*armc = se_ascmc[SE_ARMC];

	if(tdp->is_dst) delete my_tdp;
}

void get_planet(int npl, trTimeDatePlace *tdp, trPlanet *pl)
{
#ifndef _DEBUG
	if((tdp == NULL) || (pl == NULL)) return;
#endif

	trTimeDatePlace *my_tdp = tdp;

	strcpy(err_msg, "");

	if(!IS_PLANETARY_BODY(npl))
	{
		was_err = true;
		err_place = npl;
		strcpy(err_msg, "Invalid planet/object passed in!");
		return;
	}

	int se_n, se_ret;
	double ftim, jul_ut;
	double pldata[6];

	was_err = false;
	strcpy(err_msg, "");

	if(tdp->is_dst)
	{
		my_tdp = new trTimeDatePlace;
		*my_tdp = *tdp;
		correct_for_dst(my_tdp);
	}
	ftim = my_tdp->time.hours + ((double)(my_tdp->time.min)/MINS_IN_HOUR) +
		((double)(tdp->time.sec)/SECS_IN_HOUR);
	jul_ut = swe_julday(my_tdp->date.year, my_tdp->date.month, my_tdp->date.day,
		ftim, SE_GREG_CAL) - (my_tdp->timezone/24.0);
	
	se_n = ToSwissEph(npl);
	if(npl == NNODE)
	{
		se_n = ToSwissEph((get_node_type() == USE_MEAN) ?
			MEAN_NODE : TRUE_NODE);
	}
	else if(npl == LILITH)
	{
		se_n = ToSwissEph((get_apog_type() == USE_MEAN) ?
			MEAN_APOG : OSCU_APOG);
	}
	se_ret = swe_calc_ut(jul_ut, se_n, SEFLG_SPEED, pldata, err_msg);
	if(se_ret < 0)
	{
		was_err = true;
		err_place = npl;
		if(tdp->is_dst) delete my_tdp;
		return;
	}
	pl->longitude  = pldata[0];
	pl->latitude   = pldata[1];
	pl->dist       = pldata[2];
	pl->long_speed = pldata[3];
	pl->lat_speed  = pldata[4];
	pl->dist_speed = pldata[5];
	split_longitute(pl->longitude, &(pl->sgn), FALSE);

	if(tdp->is_dst) delete my_tdp;
}

double get_ecliptic(trTimeDatePlace *tdp)
{
#ifndef _DEBUG
	if(tdp == NULL) return 0.0;
#endif

	trTimeDatePlace *my_tdp = tdp;

	int se_n, se_ret;
	double ftim, jul_ut;
	double pldata[6];

	was_err = false;
	strcpy(err_msg, "");

	if(tdp->is_dst)
	{
		my_tdp = new trTimeDatePlace;
		*my_tdp = *tdp;
		correct_for_dst(my_tdp);
	}
	ftim = my_tdp->time.hours + ((double)(my_tdp->time.min)/MINS_IN_HOUR) +
		((double)(tdp->time.sec)/SECS_IN_HOUR);
	jul_ut = swe_julday(my_tdp->date.year, my_tdp->date.month, my_tdp->date.day,
		ftim, SE_GREG_CAL) - (my_tdp->timezone/24.0);
	
	se_n = ToSwissEph(ECLIPTIC_OBLIQUITY);
	se_ret = swe_calc_ut(jul_ut, se_n, 0, pldata, err_msg);
	if(se_ret < 0)
	{
		was_err = true;
		err_place = ECLIPTIC_OBLIQUITY;
		if(tdp->is_dst) delete my_tdp;
		return 0.0;
	}

	if(tdp->is_dst) delete my_tdp;

	return pldata[0];
}

int get_which_house(int npl, trChartInfo *pinf, trHousePos house[HOUSEARRSIZE])
{
	int i;

	for(i = 1; i <= 12; i++)
	{
		if(house[i].cusp > house[i+1].cusp)
		{
			/* Special case: when the house crosses 0 Aries */
			if((pinf->planet[npl].longitude > house[i].cusp) ||
				(pinf->planet[npl].longitude < house[i+1].cusp))
			{
				return i;
			}
			
		}
		/* Otherwise, search for house position normally */
		else if((pinf->planet[npl].longitude > house[i].cusp) &&
			(pinf->planet[npl].longitude < house[i+1].cusp))
		{
			return i;
		}
	}

	return 0;
}

void fill_planets_in_houses(trChartInfo *pinf, int hsys)
{
	int i, j;

	/* Fill in the extra information not filled in by the above functions*/
	for(i = 0; i < NTOTOBJECTS; i++)
	{
		pinf->planet[i].hsystem = hsys;
		for(j = 1; j <= 12; j++)
		{
			if(pinf->house[j].cusp > pinf->house[j+1].cusp)
			{
				/* Special case: when the house crosses 0 Aries */
				if((pinf->planet[i].longitude > pinf->house[j].cusp) ||
				   (pinf->planet[i].longitude < pinf->house[j+1].cusp))
				{
					pinf->planet[i].in_house = j;
					break;
				}

			}
			/* Otherwise, search for house position normally */
			else if((pinf->planet[i].longitude > pinf->house[j].cusp) &&
			        (pinf->planet[i].longitude < pinf->house[j+1].cusp))
			{
				pinf->planet[i].in_house = j;
				break;
			}
		} /* end for j */
	} /* end for i */
}






double ac_get_planet_longitude(int npl, trTimeDatePlace *tdp)
{
#ifndef _DEBUG
	if(tdp == NULL) return 0.0;
#endif
	trPlanet pl;
	get_planet(npl, tdp, &pl);
	if(!ac_error_occured())
		return pl.longitude;
	else
		return 0.0;
}

double ac_get_house_pos(int nhouse, trTimeDatePlace *tdp, int hsys)
{
#ifndef _DEBUG
	if((tdp == NULL) || (!IS_HOUSE(nhouse))) return 0.0;
#endif
	trHousePos myhouse[HOUSEARRSIZE];

	double armc, sidt;

	get_houses(tdp, hsys, myhouse, &armc, &sidt);
	if(!ac_error_occured())
		return myhouse[nhouse].cusp;
	else
		return 0.0;
}
