//AstroConst.h: These are the astrological constants used throughout the program,
//with the exception of the routines that directly call the astrology library
//(Swiss Ephemeris) or the glyph-drawing routines (Astrolog's Turtle Drawer)
//
//Copyright (C) 2003 mikesag
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#ifndef __ASTROCONST_H__
#define __ASTROCONST_H__

// The Signs
#define SIGNS     500

// Some sections are marked specially for easy text processing
// This is so some tedious switch() statements can be written programattically

//-- BEGIN SIGNS --//
#define ARIES     500
#define TAURUS    501
#define GEMINI    502
#define CANCER    503
#define LEO       504
#define VIRGO     505
#define LIBRA     506
#define SCORPIO   507
#define SAGITTARIUS 508
#define SAG       SAGITTARIUS
#define CAPRICORN 509
#define AQUARIUS  510
#define PISCES    511
//-- END SIGNS --//

#define FIRST_SIGN ARIES
#define IS_SIGN(n) (((n) >= ARIES) && ((n) <= PISCES))


// The Houses
#define HOUSES    750

//-- BEGIN HOUSES --//
#define HOUSE1    750
#define HOUSE2    751
#define HOUSE3    752
#define HOUSE4    753
#define HOUSE5    754
#define HOUSE6    755
#define HOUSE7    756
#define HOUSE8    757
#define HOUSE9    758
#define HOUSE10   759
#define HOUSE11   760
#define HOUSE12   761
//-- END HOUSES --//

#define IS_HOUSE(n) (((n) >= HOUSE1) && ((n) <= HOUSE12))

#define FIRST_HOUSE HOUSE1
#define ASCENDANT HOUSE1
#define ASC       HOUSE1

#define NADIR     HOUSE4

#define DESCENDANT HOUSE7

#define MIDHEAVEN HOUSE10

// House systems
//-- BEGIN HOUSE_SYSTEMS --//
#define PLACIDUS      780
#define KOCH          781
#define EQUAL         782
#define EQUAL_HOUSES  EQUAL
#define NULL_HOUSES   783
#define CAMPANUS      784
#define HORIZON       785
#define AZIMUT_HOUSES HORIZON
#define PORPHYRY      786
#define REGIOMONTANUS 787
#define VEHLOW        788
#define MERIDIAN      789
#define ALCABITIUS    790
#define TOPOCENTRIC   791
#define POLICH        TOPOCENTRIC
#define MORINUS       792
#define GAUQUELIN     793  /* Not implemented yet */
//-- END HOUSE_SYSTEMS --//

#define FIRST_HSYSTEM PLACIDUS
#define NHSYSTEMS (MORINUS - PLACIDUS + 1)

#define IS_HSYSTEM(n) (((n) >= PLACIDUS) && ((n) <= MORINUS))

// The Aspects
#define ASPECTS_BLOCK_START 1000

//-- BEGIN ASPECTS --//
#define CONJUNCTION       1000
#define OPPOSITION        1001
#define SQUARE            1002
#define TRINE             1003
#define SEXTILE           1004
#define INCONJUNCT        1005
#define SEMISEXTILE       1006
#define SEMISQUARE        1007
#define SESQUIQUADRATURE  1008
#define QUINTILE          1009
#define BIQUINTILE        1010
#define SEMIQUINTILE      1011
#define SEPTILE           1012
#define NOVILE            1013
#define BINOVILE          1014
#define BISEPTILE         1015
#define TRISEPTILE        1016
#define QUATRONOVILE      1017
//-- END ASPECTS --//
#define PARALLEL          1018
#define CONTRAPARALLEL    1019
#define DISPOSITOR_ASPECT 1020

#define FIRST_ASPECT CONJUNCTION
#define NASPECTS (QUATRONOVILE - CONJUNCTION + 1)
#define IS_ASPECT(n) (((n) >= CONJUNCTION) && ((n) <= QUATRONOVILE))

// The Planets
#define PLANETS  10000 /* Anything greater than 10000 is something in space */

// the range of SUN to PLUTO is assumed to be unbroken
//-- BEGIN PLANETS --//
#define SUN      10000
#define MOON     10001
#define MERCURY  10002
#define VENUS    10003
#define MARS     10004
#define JUPITER  10005
#define SATURN   10006
#define URANUS   10007
#define NEPTUNE  10008
#define PLUTO    10009
//-- END PLANETS --//

#define FIRST_PLANET SUN
#define FIRST_OBJECT FIRST_PLANET
#define NPLANETS (PLUTO - SUN + 1)
#define IS_PLANET(n) (((n) >= SUN) && ((n) <= PLUTO))

// Next, come some of the more common minor celestrial objects
//-- BEGIN OBJECTS --//
#define NNODE    10010
#define LILITH   10011

#define CHIRON   10012

#define CERES    10013
#define PALLAS   10014
#define JUNO     10015
#define VESTA    10016
#define PHOLUS   10017

// Uranians
#define U_CUPIDO     10018
#define U_HADES      10019
#define U_ZEUS       10020
#define U_KRONOS     10021
#define U_APOLLON    10022
#define U_ADMETOS    10023
#define U_VULKANUS   10024
#define U_POSEIDON   10025
#define U_PROSERPINA 10026

#define FIRST_NON_PLANET NNODE
#define NOBJECTS    (U_POSEIDON - NNODE + 1)
#define NTOTOBJECTS (U_POSEIDON - SUN + 1)

// Other heavenly bodies

#define VERTEX      10150
#define SNODE       10151 /* Same as Lilith? */

#define PFORTUNE    10152
#define EPOINT      10153
#define EASTPOINT   EPOINT
#define EAST_POINT  EPOINT

#define EARTH       10154

#define ECLIPTIC_OBLIQUITY 10155

#define MEAN_NODE   10170
#define TRUE_NODE   10171 /* Same as N. Node, by default */
#define MEAN_APOG   10172 /* Same as Lilith by default */
#define OSCU_APOG   10173
#define MEAN_APOGEE MEAN_APOG
#define OSCU_APOGEE OSCU_APOG

//-- END OBJECTS --//

#define IS_OTHER_BODY(n) (((n) == CHIRON) || ((n) == LILITH) || ((n) == EARTH) || \
						  (((n) >= CERES) && ((n) <= VESTA)) || \
						  (((n) >= NNODE) && ((n) <= EPOINT)) || \
						  (((n) >= MEAN_NODE) && ((n) <= OSCU_APOG)) || \
						  (((n) >= U_CUPIDO) && ((n) <= U_PROSERPINA)))

#define IS_PLANETARY_BODY(n) ((IS_PLANET(n)) || (IS_OTHER_BODY(n)))






// Miscellaneous Constants


// Attributes, specifically for use in colouring
#define colFIRE     250
#define colEARTH    251
#define colAIR      252
#define colWATER    253

#define colCHIRON   260  /* Chiron & friends; Ceres, Pallas, Juno, Vesta */
#define colOTHER    261
#define colURANIAN  262
#define colERROR    263  /* Good for visually spotting errors */

#define colCARDINAL 270
#define colFIXED    271
#define colMUTABLE  272


// Aspect Configurations
#define STELLIUM    300
#define GRAND_TRINE 301
#define T_SQUARE    302
#define KITE_CONF   303
#define GRAND_CROSS 304
#define YOD_CONF    305
#define MYSTIC_RECT 306
#define BOOMERANG   307
#define GR_QUINTILE 308
#define GR_SEXTILE  309

#define FIRST_ASPCONF STELLIUM
#define N_ASPCONFS (GR_SEXTILE - STELLIUM + 1)

// Number of planets, followed by the number of aspects listed
// after it, followed by the aspects themselves.
// Note that for every planet, all C(n,2) = n!/((n - 2)! 2!)
// = n*(n - 1)/2 = sum(i=[1,n],i) aspects that occur must be
// listed to prevent conjunctions from taking away from them

#define STELLIUM_A    {3, 3, CONJUNCTION, CONJUNCTION, CONJUNCTION}
#define GRAND_TRINE_A {3, 3, TRINE, TRINE, TRINE}
#define T_SQUARE_A    {3, 3, SQUARE, SQUARE, OPPOSITION}
#define YOD_CONF_A    {3, 3, INCONJUNCT, INCONJUNCT, SEXTILE}
#define KITE_CONF_A   {4, 6, TRINE, TRINE, TRINE, SEXTILE, SEXTILE, OPPOSITION}
#define GRAND_CROSS_A {4, 6, SQUARE, SQUARE, SQUARE, SQUARE, OPPOSITION, OPPOSITION}
#define MYSTIC_RECT_A {4, 6, TRINE, TRINE, SEXTILE, SEXTILE, OPPOSITION, OPPOSITION}
#define BOOMERANG_A   {4, 6, INCONJUNCT, INCONJUNCT, OPPOSITION, SEXTILE, \
SEMISEXTILE, SEMISEXTILE}
#define GR_QUINTILE_A {5, 10, QUINTILE, QUINTILE, QUINTILE, QUINTILE, QUINTILE, \
QUINTILE, QUINTILE, QUINTILE, QUINTILE, QUINTILE}
#define GR_SEXTILE_A  {6, 15, SEXTILE, SEXTILE, SEXTILE, SEXTILE, SEXTILE, SEXTILE, \
TRINE, TRINE, TRINE, TRINE, TRINE, TRINE, OPPOSITION, OPPOSITION, OPPOSITION}


// Months (Western calendar)
#define JANUARY     331
#define FEBRUARY    332
#define MARCH       333
#define APRIL       334
#define MAY         335
#define JUNE        336
#define JULY        337
#define AUGUST      338
#define SEPTEMBER   339
#define OCTOBER     340
#define NOVEMBER    341
#define DECEMBER    342

#define FIRST_MONTH JANUARY



#endif
