//InputDialogs.cpp: Dialogs to enter in the time/date/place (TDP) of the
//                  charts and which planets show (called "restrictions",
//                  after Astrolog)
//
//Copyright (C) 2003 mikesag
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include "wx/wxprec.h"

#ifndef WX_PRECOMP
	#include "wx/wx.h"
#endif

#include "wx/string.h"
#include "wx/sizer.h"
#include "wx/dialog.h"
#include "wx/notebook.h"
#include "wx/statbox.h"
#include "wx/stattext.h"
#include "wx/valtext.h"
#include "wx/dynarray.h"
#include "wx/msgdlg.h"

#include <math.h>

#include "Translation.h"
#include "InputDialogs.h"

// ----------------------------------------------------------------------------
// time/date/place input dialog box
// ----------------------------------------------------------------------------

IMPLEMENT_CLASS(CInputDateTimePlaceDlg, wxDialog)
BEGIN_EVENT_TABLE(CInputDateTimePlaceDlg, wxDialog)
	EVT_SHOW(CInputDateTimePlaceDlg::OnShow)
	EVT_BUTTON(wxID_OK, CInputDateTimePlaceDlg::OnOK)
	EVT_BUTTON(wxID_CANCEL, CInputDateTimePlaceDlg::OnCancel)
	EVT_BUTTON(ID_GUESS, CInputDateTimePlaceDlg::OnGuessDST)
END_EVENT_TABLE()

CInputDateTimePlaceDlg::CInputDateTimePlaceDlg(wxWindow *parent, wxWindowID id,
							const wxString &title, bool is_small, long style) :
                        wxDialog(parent, id, title, wxDefaultPosition, wxDefaultSize, style)
{
	wxSizer *topsizer  = new wxBoxSizer(wxVERTICAL);
	wxSizer *horzsizer = new wxBoxSizer(wxHORIZONTAL);
	wxSizer *calsizer  = new wxBoxSizer(wxHORIZONTAL);
	wxSizer *vertsizer = new wxBoxSizer(wxVERTICAL);

	cal = new wxCalendarCtrl(this, -1,
	                             wxDateTime::Now(),
                                 wxDefaultPosition,
                                 wxDefaultSize,
                                 wxCAL_SUNDAY_FIRST |
                                 wxCAL_SHOW_SURROUNDING_WEEKS |
                                 wxRAISED_BORDER);
	if(!is_small)
		topsizer->Add(cal, 0, wxALIGN_CENTER | wxALL, 10);
	else
		calsizer->Add(cal);

    wxFlexGridSizer *datasizer = new wxFlexGridSizer( 2 );
    datasizer->AddGrowableCol( 1 );

	wxStaticText *namelabel = new wxStaticText(this, -1, "Name:",
		wxDefaultPosition, wxDefaultSize, wxST_NO_AUTORESIZE | wxALIGN_RIGHT);
	nametext = new wxTextCtrl(this, -1, "");
	nametext->SetMaxLength(60);
	datasizer->Add(namelabel, 1, wxALL | wxALIGN_RIGHT, 5);
	datasizer->Add(nametext,  1, wxALL, 5);
	
	wxStaticText *citylabel = new wxStaticText(this, -1, "City:",
		wxDefaultPosition, wxDefaultSize, wxST_NO_AUTORESIZE | wxALIGN_RIGHT);
	citytext = new wxTextCtrl(this, -1, "");
	citytext->SetMaxLength(60);
	datasizer->Add(citylabel, 1, wxALL | wxALIGN_RIGHT, 5);
	datasizer->Add(citytext,  1, wxALL, 5);

	wxStaticText *timelabel = new wxStaticText(this, -1, "Time:",
		wxDefaultPosition, wxDefaultSize, wxST_NO_AUTORESIZE | wxALIGN_RIGHT);
	timetext = new wxTextCtrl(this, -1, "");
	timetext->SetMaxLength(15);
	datasizer->Add(timelabel, 1, wxALL | wxALIGN_RIGHT, 5);
	datasizer->Add(timetext,  1, wxALL, 5);

	wxStaticText *latlabel = new wxStaticText(this, -1, "Latitude:",
		wxDefaultPosition, wxDefaultSize, wxST_NO_AUTORESIZE | wxALIGN_RIGHT);
	lattext = new wxTextCtrl(this, -1, "");
	lattext->SetMaxLength(15);
	datasizer->Add(latlabel, 1, wxALL | wxALIGN_RIGHT, 5);
	datasizer->Add(lattext,  1, wxALL, 5);

	wxStaticText *lonlabel = new wxStaticText(this, -1, "Longitude:",
		wxDefaultPosition, wxDefaultSize, wxST_NO_AUTORESIZE | wxALIGN_RIGHT);
	lontext = new wxTextCtrl(this, -1, "");
	lontext->SetMaxLength(15);
	datasizer->Add(lonlabel, 1, wxALL | wxALIGN_RIGHT, 5);
	datasizer->Add(lontext,  1, wxALL, 5);

	wxStaticText *tzlabel = new wxStaticText(this, -1, "Time Zone:",
		wxDefaultPosition, wxDefaultSize, wxST_NO_AUTORESIZE | wxALIGN_RIGHT);
	tztext = new wxTextCtrl(this, -1, "", wxDefaultPosition, wxDefaultSize); //,
		//0, wxTextValidator(wxFILTER_NUMERIC, &m_tz));
	tztext->SetMaxLength(3);

	datasizer->Add(tzlabel, 1, wxALL | wxALIGN_RIGHT, 5);
	datasizer->Add(tztext,  1, wxALL, 5);

	if(!is_small)
		topsizer->Add(datasizer);
	else
		vertsizer->Add(datasizer);

	wxSizer *dstsizer  = new wxBoxSizer(wxHORIZONTAL);
	dst_check = new wxCheckBox(this, -1, "Daylight Savings Time");
	dstsizer->Add(dst_check, 0, wxALIGN_CENTER);
	wxButton *guessbtn = new wxButton(this, ID_GUESS, "Guess",
		wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT);
	dstsizer->Add(guessbtn, 0, wxALIGN_CENTER);
	if(!is_small)
		topsizer->Add(dstsizer, 0, wxALL | wxALIGN_CENTER, 10);
	else
		vertsizer->Add(dstsizer, 0, wxALL | wxALIGN_CENTER, 10);

	progressed_check = new wxCheckBox(this, -1, "Progressed to Transit Date/Time");
	if(!is_small)
		topsizer->Add(progressed_check, 0, wxBOTTOM | wxALIGN_CENTER, 10);
	else
		vertsizer->Add(progressed_check, 0, wxBOTTOM | wxALIGN_CENTER, 10);

	if(is_small)
	{
		horzsizer->Add(vertsizer, 0, wxALL, 10);
		horzsizer->Add(calsizer, 0, wxALL | wxALIGN_CENTER, 10);
		topsizer->Add(horzsizer);
	}

	wxSizer *btnsizer = new wxBoxSizer(wxHORIZONTAL);

	wxButton *okbtn = new wxButton(this, wxID_OK, "OK");
	wxButton *cancelbtn = new wxButton(this, wxID_CANCEL, "Cancel");
	btnsizer->Add(okbtn, 0, wxALL, 10);
	btnsizer->Add(cancelbtn, 0, wxALL, 10);

	topsizer->Add(btnsizer, 0, wxALIGN_CENTER);

	SetAutoLayout(TRUE);
	SetSizer(topsizer);
	
	topsizer->Fit(this);
	topsizer->SetSizeHints(this);

	SetDefaultItem(okbtn);
}

CInputDateTimePlaceDlg::~CInputDateTimePlaceDlg()
{
}

void CInputDateTimePlaceDlg::OnShow(wxShowEvent &event)
{
	static bool first_time = true;
	if(first_time)
	{
		first_time = false;
		Center();
	}
}

void CInputDateTimePlaceDlg::OnGuessDST(wxCommandEvent &event)
{
	wxDateTime wxdt;
	wxdt = cal->GetDate();
	dst_check->SetValue(wxdt.IsDST() ? true : false);
}




// ----------------------------------------------------------------------------
// restrictions dialog box
// ----------------------------------------------------------------------------

#define R_BTN_BASE 70

IMPLEMENT_CLASS(CInputRestrictionsDlg, wxDialog)
BEGIN_EVENT_TABLE(CInputRestrictionsDlg, wxDialog)
	EVT_SHOW(CInputRestrictionsDlg::OnShow)
	EVT_BUTTON(wxID_OK, CInputRestrictionsDlg::OnOK)
	EVT_BUTTON(wxID_CANCEL, CInputRestrictionsDlg::OnCancel)
	EVT_BUTTON(-1, CInputRestrictionsDlg::OnButton)
END_EVENT_TABLE()

CInputRestrictionsDlg::CInputRestrictionsDlg(wxWindow *parent, wxWindowID id,
							const wxString &title, bool is_small, long style) :
         wxDialog(parent, id, title, wxDefaultPosition, wxDefaultSize, style)
{
	int i, j;

	wxSizer *topsizer = new wxBoxSizer(wxVERTICAL);

	wxSizer *allres_sizer = new wxBoxSizer(wxHORIZONTAL);

	wxStaticBox *plbox = new wxStaticBox(this, -1, "Planets");
	wxStaticBoxSizer *plsizer = new wxStaticBoxSizer(plbox, wxVERTICAL);

	for(i = 0; i < NPLANETS; i++)
	{
		r_check[i] = new wxCheckBox(this, BASE_R_CB_ID+i,
			trGetName(FIRST_PLANET+i));
		plsizer->Add(r_check[i], 0, wxALL, 2);
	}

	plsizer->Add(new wxButton(this, R_BTN_BASE+0, "All",
		wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT), 0, wxALL | wxALIGN_CENTER, 2);
	plsizer->Add(new wxButton(this, R_BTN_BASE+1, "None",
		wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT), 0, wxALL | wxALIGN_CENTER, 2);
	plsizer->Add(new wxButton(this, R_BTN_BASE+2, "Toggle",
		wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT), 0, wxALL | wxALIGN_CENTER, 2);

	allres_sizer->Add(plsizer, 0, wxALL, 10);

	wxStaticBox *mibox = new wxStaticBox(this, -1, "Minor Objects");
	wxStaticBoxSizer *misizer = new wxStaticBoxSizer(mibox, wxVERTICAL);
	wxSizer *mihorzsizer = new wxBoxSizer(wxHORIZONTAL);
	wxSizer *mivertsizer = new wxBoxSizer(wxVERTICAL);

	for(; i < NTOTOBJECTS; i++)
	{
		r_check[i] = new wxCheckBox(this, BASE_R_CB_ID+i,
			trGetName(FIRST_PLANET+i));
		if(is_small && (i == (NPLANETS + NTOTOBJECTS)/2))
		{
			mihorzsizer->Add(mivertsizer);
			mivertsizer = new wxBoxSizer(wxVERTICAL);
		}
		mivertsizer->Add(r_check[i], 0, wxALL, 2);
	}

	mihorzsizer->Add(mivertsizer);
	misizer->Add(mihorzsizer);

	misizer->Add(new wxButton(this, R_BTN_BASE+10, "All",
		wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT), 0, wxALL | wxALIGN_CENTER, 2);
	misizer->Add(new wxButton(this, R_BTN_BASE+11, "None",
		wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT), 0, wxALL | wxALIGN_CENTER, 2);
	misizer->Add(new wxButton(this, R_BTN_BASE+12, "Toggle",
		wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT), 0, wxALL | wxALIGN_CENTER, 2);

	allres_sizer->Add(misizer, 0, wxALL, 10);
	
	wxStaticBox *houbox = new wxStaticBox(this, -1, "House Cusps");
	wxStaticBoxSizer *housizer = new wxStaticBoxSizer(houbox, wxVERTICAL);

	for(j = 0; j < 12; i++, j++)
	{
		r_check[i] = new wxCheckBox(this, BASE_R_CB_ID+i,
			trGetName(FIRST_HOUSE+j));
		housizer->Add(r_check[i], 0, wxALL, 2);
	}

	housizer->Add(new wxButton(this, R_BTN_BASE+20, "All",
		wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT), 0, wxALL | wxALIGN_CENTER, 2);
	housizer->Add(new wxButton(this, R_BTN_BASE+21, "None",
		wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT), 0, wxALL | wxALIGN_CENTER, 2);
	housizer->Add(new wxButton(this, R_BTN_BASE+22, "Toggle",
		wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT), 0, wxALL | wxALIGN_CENTER, 2);

	allres_sizer->Add(housizer, 0, wxALL, 10);
	
	topsizer->Add(allres_sizer);

	wxSizer *btnsizer = new wxBoxSizer(wxHORIZONTAL);

	wxButton *okbtn = new wxButton(this, wxID_OK, "OK");
	wxButton *cancelbtn = new wxButton(this, wxID_CANCEL, "Cancel");
	btnsizer->Add(okbtn, 0, wxALL, 10);
	btnsizer->Add(cancelbtn, 0, wxALL, 10);

	topsizer->Add(btnsizer, 0, wxALIGN_CENTER);

	SetAutoLayout(TRUE);
	SetSizer(topsizer);
	
	topsizer->Fit(this);
	topsizer->SetSizeHints(this);

	SetDefaultItem(okbtn);
}

CInputRestrictionsDlg::~CInputRestrictionsDlg()
{
}

void CInputRestrictionsDlg::OnShow(wxShowEvent &event)
{
	static bool first_time = true;
	if(first_time)
	{
		first_time = false;
		Center();
	}
}

void CInputRestrictionsDlg::OnButton(wxCommandEvent &event)
{
	int i, imin, imax;
	int n_event = event.GetId() - R_BTN_BASE;

	if(n_event / 10 == 0)
	{
		imin = 0;
		imax = NPLANETS;
	}
	else if(n_event / 10 == 1)
	{
		imin = NPLANETS;
		imax = NTOTOBJECTS;
	}
	else if(n_event / 10 == 2)
	{
		imin = NTOTOBJECTS;
		imax = NTOTOBJECTS+12;
	}
	else
		return;

	for(i = imin; i < imax; i++)
	{
		if(r_check[i]->IsEnabled())
		{
			if(n_event % 10 == 0)
				r_check[i]->SetValue(TRUE);
			else if(n_event % 10 == 1)
				r_check[i]->SetValue(FALSE);
			else if(n_event % 10 == 2)
				r_check[i]->SetValue(!r_check[i]->GetValue());
		}
	}
}




// ----------------------------------------------------------------------------
// aspect restrictions dialog box
// ----------------------------------------------------------------------------

IMPLEMENT_CLASS(CInputAspectRestrictionsDlg, wxDialog)
BEGIN_EVENT_TABLE(CInputAspectRestrictionsDlg, wxDialog)
	EVT_SHOW(CInputAspectRestrictionsDlg::OnShow)
	EVT_BUTTON(wxID_OK, CInputAspectRestrictionsDlg::OnOK)
	EVT_BUTTON(wxID_CANCEL, CInputAspectRestrictionsDlg::OnCancel)
	EVT_BUTTON(-1, CInputAspectRestrictionsDlg::OnButton)
END_EVENT_TABLE()

CInputAspectRestrictionsDlg::CInputAspectRestrictionsDlg(wxWindow *parent,
	wxWindowID id, const wxString &title, bool is_small, long style) :
	wxDialog(parent, id, title, wxDefaultPosition, wxDefaultSize, style)
{
	wxStaticText *stattext;
	wxString str;
	wxFont fnt;
	int i, j;

	wxSizer *h_sizer;

	wxSizer *topsizer = new wxBoxSizer(wxVERTICAL);

	wxSizer *allres_sizer = new wxBoxSizer(wxHORIZONTAL);

	wxSizer *as_sizer = new wxFlexGridSizer(3, 0, 10);

	for(i = 0; i < NASPECTS; i++)
	{
		if(is_small && (i == NASPECTS/2))
		{
			allres_sizer->Add(as_sizer, 0, wxALL, 10);
			
			as_sizer = new wxFlexGridSizer(3, 0, 10);
		}

		if((i == 0) || (is_small && (i == NASPECTS/2)))
		{
			for(j = 0; j < 3; j++)
			{
				if     (j == 0) str = "Aspect";
				else if(j == 1) str = "Definition";
				else if(j == 2) str = "Orb";
				
				stattext = new wxStaticText(this, -1, str);
				fnt = stattext->GetFont();
				fnt.SetWeight(wxBOLD);
				fnt.SetPointSize(fnt.GetPointSize() + 2);
				stattext->SetFont(fnt);
				as_sizer->Add(stattext, -1, wxALIGN_CENTER | wxBOTTOM, 10);
			}
		}

		r_check[i] = new wxCheckBox(this, BASE_R_CB_ID+i,
			trGetName(FIRST_ASPECT+i));
		str.Printf("%.2f", AspectDef(FIRST_ASPECT+i));
		stattext = new wxStaticText(this, -1, str,
			wxDefaultPosition, wxDefaultSize,
			wxST_NO_AUTORESIZE | wxALIGN_RIGHT);
		orb_text[i] = new wxTextCtrl(this, BASE_R_ORB_ID+i, "",
			wxDefaultPosition, wxSize(35, -1), 0,
			wxTextValidator(wxFILTER_NUMERIC, &(orb_str[i])));

		as_sizer->Add(r_check[i]);
		as_sizer->Add(stattext, 0, wxALIGN_RIGHT);
		as_sizer->Add(orb_text[i], 0, wxALIGN_CENTER);
	}

	as_sizer->Add(new wxButton(this, R_BTN_BASE+0, "All",
		wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT), 0, wxALL | wxALIGN_CENTER, 2);
	as_sizer->Add(1, 1);
	h_sizer = new wxBoxSizer(wxHORIZONTAL);
	h_sizer->Add(new wxButton(this, R_BTN_BASE+3, "+1",
		wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT), 0, wxALL | wxALIGN_CENTER, 2);
	h_sizer->Add(new wxButton(this, R_BTN_BASE+4, "-1",
		wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT), 0, wxALL | wxALIGN_CENTER, 2);
	as_sizer->Add(h_sizer, 0, wxALIGN_CENTER);

	as_sizer->Add(new wxButton(this, R_BTN_BASE+1, "None",
		wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT), 0, wxALL | wxALIGN_CENTER, 2);
	as_sizer->Add(1, 1);
	h_sizer = new wxBoxSizer(wxHORIZONTAL);
	h_sizer->Add(new wxButton(this, R_BTN_BASE+5, "*2",
		wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT), 0, wxALL | wxALIGN_CENTER, 2);
	h_sizer->Add(new wxButton(this, R_BTN_BASE+6, "/2",
		wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT), 0, wxALL | wxALIGN_CENTER, 2);
	as_sizer->Add(h_sizer, 0, wxALIGN_CENTER);

	as_sizer->Add(new wxButton(this, R_BTN_BASE+2, "Toggle",
		wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT), 0, wxALL | wxALIGN_CENTER, 2);
	
	/*
	as_sizer->Add(1, 1);
	as_sizer->Add(new wxButton(this, R_BTN_BASE+7, "Load Defaults",
		wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT), 0, wxALL | wxALIGN_CENTER, 2);

	as_sizer->Add(1, 1);
	as_sizer->Add(1, 1);
	as_sizer->Add(new wxButton(this, R_BTN_BASE+8, "Save Defaults",
		wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT), 0, wxALL | wxALIGN_CENTER, 2);
	*/

	allres_sizer->Add(as_sizer, 0, wxALL, 10);
	
	topsizer->Add(allres_sizer);

	wxSizer *btnsizer = new wxBoxSizer(wxHORIZONTAL);

	wxButton *okbtn = new wxButton(this, wxID_OK, "OK");
	wxButton *cancelbtn = new wxButton(this, wxID_CANCEL, "Cancel");
	btnsizer->Add(okbtn, 0, wxALL, 10);
	btnsizer->Add(cancelbtn, 0, wxALL, 10);

	topsizer->Add(btnsizer, 0, wxALIGN_CENTER);

	SetAutoLayout(TRUE);
	SetSizer(topsizer);
	
	topsizer->Fit(this);
	topsizer->SetSizeHints(this);

	SetDefaultItem(okbtn);
}

CInputAspectRestrictionsDlg::~CInputAspectRestrictionsDlg()
{
}

void CInputAspectRestrictionsDlg::OnShow(wxShowEvent &event)
{
	static bool first_time = true;
	if(first_time)
	{
		first_time = false;
		Center();
	}
}

void CInputAspectRestrictionsDlg::OnButton(wxCommandEvent &event)
{
	int i;
	wxString str;
	double val = 0.0;
	int n_event = event.GetId() - R_BTN_BASE;

	if(n_event == 7)
	{
		/*TODO: Load from configuration file if possible; if not,
		  then load global defaults instead (as in the loop below)*/
	}
	if(n_event == 8)
	{
		/*TODO: Save as defaults*/
		wxMessageBox("This will save the current orbs and "
			"restrictions as the default.");
		return;
	}

	for(i = 0; i < NASPECTS; i++)
	{
		if((n_event >= 3) && (n_event <= 7))
		{
			str = orb_text[i]->GetValue();
			str.ToDouble(&val);
		}
		switch(n_event)
		{
		case 0: r_check[i]->SetValue(TRUE);  break;
		case 1: r_check[i]->SetValue(FALSE); break;
		case 2: r_check[i]->SetValue(!r_check[i]->GetValue()); break;
		case 3: val += 1.0; break;
		case 4: val -= 1.0; break;
		case 5: val *= 2.0; break;
		case 6: val /= 2.0; break;
		case 7: val  = AspectOrbDefaults(FIRST_ASPECT+i); break;
		}
		if((n_event >= 3) && (n_event <= 7))
		{
			str.Printf("%.1f", val);
			orb_text[i]->SetValue(str);
		}
	}
}



// ----------------------------------------------------------------------------
// aspect search dialog box
// ----------------------------------------------------------------------------

#define FA_LB_BASE 50

IMPLEMENT_CLASS(CFindAspectsDlg, wxDialog)
BEGIN_EVENT_TABLE(CFindAspectsDlg, wxDialog)
	EVT_SHOW(CFindAspectsDlg::OnShow)
	EVT_BUTTON(wxID_OK, CFindAspectsDlg::OnOK)
	EVT_BUTTON(wxID_CANCEL, CFindAspectsDlg::OnCancel)
	EVT_BUTTON(-1, CFindAspectsDlg::OnButton)
	EVT_LISTBOX(-1, CFindAspectsDlg::OnListBoxSel)
END_EVENT_TABLE()

CFindAspectsDlg::CFindAspectsDlg(wxWindow *parent,
	wxWindowID id, const wxString &title, bool is_small, long style) :
	wxDialog(parent, id, title, wxDefaultPosition, wxDefaultSize, style)
{
	const int lbdiff = 50, lbheight = 200, smalldiff = 50;
	wxStaticText *stattext;
	wxSizer *topsizer = new wxBoxSizer(wxVERTICAL);

	wxSizer *listsizer = new wxBoxSizer(wxHORIZONTAL);

	wxSizer *v_sizer;

	planet1 = new wxListBox(this, FA_LB_BASE + 0, wxDefaultPosition,
		wxSize(100, lbheight - (is_small ? smalldiff : 0)),
		0, NULL, wxLB_MULTIPLE);
	aspect  = new wxListBox(this, FA_LB_BASE + 1, wxDefaultPosition,
		wxSize(100, lbheight - lbdiff - (is_small ? smalldiff : 0)),
		0, NULL, wxLB_MULTIPLE);
	planet2 = new wxListBox(this, FA_LB_BASE + 2, wxDefaultPosition,
		wxSize(100, lbheight - (is_small ? smalldiff : 0)),
		0, NULL, wxLB_MULTIPLE);

	v_sizer = new wxBoxSizer(wxVERTICAL);
	v_sizer->Add(planet1, 0, wxBOTTOM, 5);
	v_sizer->Add(all0 = new wxButton(this, R_BTN_BASE+0, "All",
		wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT), 0, wxALL | wxALIGN_CENTER, 2);
	v_sizer->Add(none0 = new wxButton(this, R_BTN_BASE+1, "None",
		wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT), 0, wxALL | wxALIGN_CENTER, 2);
	v_sizer->Add(toggle0 = new wxButton(this, R_BTN_BASE+2, "Toggle",
		wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT), 0, wxALL | wxALIGN_CENTER, 2);
	listsizer->Add(v_sizer, 0, wxALL, 10);

	v_sizer = new wxBoxSizer(wxVERTICAL);
	v_sizer->Add(1, lbdiff/2);
	v_sizer->Add(aspect);
	v_sizer->Add(1, lbdiff/2 + 5);
	v_sizer->Add(all1 = new wxButton(this, R_BTN_BASE+10, "All",
		wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT), 0, wxALL | wxALIGN_CENTER, 2);
	v_sizer->Add(none1 = new wxButton(this, R_BTN_BASE+11, "None",
		wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT), 0, wxALL | wxALIGN_CENTER, 2);
	v_sizer->Add(toggle1 = new wxButton(this, R_BTN_BASE+12, "Toggle",
		wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT), 0, wxALL | wxALIGN_CENTER, 2);
	listsizer->Add(v_sizer, 0, wxALL, 10);
	
	v_sizer = new wxBoxSizer(wxVERTICAL);
	v_sizer->Add(planet2, 0, wxBOTTOM, 5);
	v_sizer->Add(all2 = new wxButton(this, R_BTN_BASE+20, "All",
		wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT), 0, wxALL | wxALIGN_CENTER, 2);
	v_sizer->Add(none2 = new wxButton(this, R_BTN_BASE+21, "None",
		wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT), 0, wxALL | wxALIGN_CENTER, 2);
	v_sizer->Add(toggle2 = new wxButton(this, R_BTN_BASE+22, "Toggle",
		wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT), 0, wxALL | wxALIGN_CENTER, 2);
	listsizer->Add(v_sizer, 0, wxALL, 10);

	topsizer->Add(listsizer, 0, wxALIGN_CENTER | wxALL, 10);

	wxSizer *orbsizer = new wxBoxSizer(wxHORIZONTAL);

	stattext = new wxStaticText(this, -1, "With orbs from ");
	orbsizer->Add(stattext, 0, wxALIGN_CENTER);
	min_orb_text = new wxTextCtrl(this, BASE_R_ORB_ID, "",
		wxDefaultPosition, wxSize(50, -1), 0,
		wxTextValidator(wxFILTER_NUMERIC, &min_orb_str));
	orbsizer->Add(min_orb_text);
	stattext = new wxStaticText(this, -1, " to ");
	orbsizer->Add(stattext, 0, wxALIGN_CENTER);
	max_orb_text = new wxTextCtrl(this, BASE_R_ORB_ID + 1, "",
		wxDefaultPosition, wxSize(50, -1), 0,
		wxTextValidator(wxFILTER_NUMERIC, &max_orb_str));
	orbsizer->Add(max_orb_text);
	stattext = new wxStaticText(this, -1, " degrees");
	orbsizer->Add(stattext, 0, wxALIGN_CENTER);

	topsizer->Add(orbsizer, 0, wxALIGN_CENTER | wxBOTTOM, 10);

	wxSizer *sortbysizer = new wxBoxSizer(wxHORIZONTAL);

	stattext = new wxStaticText(this, -1, "Sort by:");
	sortbysizer->Add(stattext, 0, wxALIGN_CENTER | wxRIGHT, 10);

	wxSizer *rbsizer = new wxBoxSizer(wxVERTICAL);
	sortorb = new wxRadioButton(this, -1, "Orb");
	sortpl1 = new wxRadioButton(this, -1, "First Planet");
	sortpl2 = new wxRadioButton(this, -1, "Second Planet");
	sortorb->SetValue(TRUE);
	rbsizer->Add(sortorb);
	rbsizer->Add(sortpl1);
	rbsizer->Add(sortpl2);
	sortbysizer->Add(rbsizer);

	needboth = new wxCheckBox(this, -1, "Only Aspects Involving Both Planets");
	needboth->SetValue(FALSE);

	if(!is_small)
	{
		topsizer->Add(sortbysizer, 0, wxALIGN_CENTER | wxBOTTOM, 10);
	}
	else
	{
		orbsizer->Add(20, 0);
		orbsizer->Add(sortbysizer, 0, 0, 10);
	}
	topsizer->Add(needboth, 0, wxALIGN_CENTER | wxBOTTOM, 10);


	wxSizer *btnsizer = new wxBoxSizer(wxHORIZONTAL);

	wxButton *okbtn = new wxButton(this, wxID_OK, "OK");
	wxButton *cancelbtn = new wxButton(this, wxID_CANCEL, "Cancel");
	btnsizer->Add(okbtn, 0, wxALL, 10);
	btnsizer->Add(cancelbtn, 0, wxALL, 10);

	topsizer->Add(btnsizer, 0, wxALIGN_CENTER);
	
	
	SetAutoLayout(TRUE);
	SetSizer(topsizer);
	
	topsizer->Fit(this);
	topsizer->SetSizeHints(this);

	SetDefaultItem(okbtn);
}

CFindAspectsDlg::~CFindAspectsDlg()
{
}

void CFindAspectsDlg::OnShow(wxShowEvent &event)
{
	static bool first_time = true;
	if(first_time)
	{
		first_time = false;
		Center();
	}
}

void CFindAspectsDlg::OnButton(wxCommandEvent &event)
{
	int i, j, imax, n_sels, found;
	int n_event = event.GetId() - R_BTN_BASE;
	static int in_recursion = FALSE;
	wxArrayInt sels;
	wxListBox *lb;

	if(n_event / 10 == 0)      { imax = planet1->GetCount(); lb = planet1; }
	else if(n_event / 10 == 1) { imax = aspect->GetCount();  lb = aspect; }
	else if(n_event / 10 == 2) { imax = planet2->GetCount(); lb = planet2; }
	else return;

	n_sels = lb->GetSelections(sels);
	for(i = 0; i < imax; i++)
	{
		if(n_event % 10 == 0)
			lb->SetSelection(i, TRUE);
		else if(n_event % 10 == 1)
			lb->SetSelection(i, FALSE);
		else if(n_event % 10 == 2)
		{
			found = 0;
			for(j = 0; j < n_sels; j++)
				if(sels[j] == i)
					found = 1;
			lb->SetSelection(i, !found);
		}
	}

	if(is_single_chart && !in_recursion && ((n_event / 10 == 0) || (n_event / 10 == 2)))
	{
		in_recursion = TRUE;
		if(n_event / 10 == 0)
		{
			event.SetId(R_BTN_BASE + 20 + (n_event % 10));
			all2->Command(event);
			event.SetId(R_BTN_BASE +  0 + (n_event % 10));
		}
		else if(n_event / 10 == 2)
		{
			event.SetId(R_BTN_BASE +  0 + (n_event % 10));
			all0->Command(event);
			event.SetId(R_BTN_BASE + 20 + (n_event % 10));
		}
		in_recursion = FALSE;
	}
}

void CFindAspectsDlg::OnListBoxSel(wxCommandEvent &event)
{
	wxListBox *lb1, *lb2;
	wxArrayInt sels;
	bool found;
	int n_sels, i, j;
	int n_event = event.GetId() - FA_LB_BASE;

	if(is_single_chart)
	{
		if     (n_event == 0) { lb1 = planet1; lb2 = planet2; }
		else if(n_event == 2) { lb1 = planet2; lb2 = planet1; }
		else                  { return; }

		n_sels = lb1->GetSelections(sels);
		for(i = 0; i < lb2->GetCount(); i++)
		{
			found = 0;
			for(j = 0; j < n_sels; j++)
				if(sels[j] == i)
					found = 1;

			lb2->SetSelection(i, found);
		}
	}
}




// ----------------------------------------------------------------------------
// calculation settings dialog box
// ----------------------------------------------------------------------------

IMPLEMENT_CLASS(CCalculationSettingsDlg, wxDialog)
BEGIN_EVENT_TABLE(CCalculationSettingsDlg, wxDialog)
	EVT_SHOW(CCalculationSettingsDlg::OnShow)
	EVT_BUTTON(wxID_OK, CCalculationSettingsDlg::OnOK)
	EVT_BUTTON(wxID_CANCEL, CCalculationSettingsDlg::OnCancel)
END_EVENT_TABLE()

CCalculationSettingsDlg::CCalculationSettingsDlg(wxWindow *parent,
	wxWindowID id, const wxString &title, bool is_small, long style) :
	wxDialog(parent, id, title, wxDefaultPosition, wxDefaultSize, style)
{
	int i;
	wxString hsysstrlst[NHSYSTEMS];
	wxString nnodestrlst[2] = {"Mean Node", "True Node"};
	wxString lilithstrlst[2] = {"Mean Apogee", "True Apogee"};

	wxSizer *topsizer    = new wxBoxSizer(wxVERTICAL);
	wxSizer *mainsizer   = new wxBoxSizer(wxHORIZONTAL);
	wxSizer *minorsizer  = new wxBoxSizer(wxVERTICAL);
	wxSizer *btnsizer    = new wxBoxSizer(wxHORIZONTAL);

	for(i = 0; i < NHSYSTEMS; i++)
		hsysstrlst[i] = wxString("&") + trGetName(FIRST_HSYSTEM + i);
	housesys = new wxRadioBox(this, -1, "House System", wxDefaultPosition,
		wxDefaultSize, NHSYSTEMS, hsysstrlst, 1, wxRA_SPECIFY_COLS);

	mainsizer->Add(housesys, 0, wxALL, 10);

	nnodetype = new wxRadioBox(this, -1, "Moon's North Node", wxDefaultPosition,
		wxDefaultSize, 2, nnodestrlst, 1, wxRA_SPECIFY_COLS);
	lilithtype = new wxRadioBox(this, -1, "Moon's Apogee/Lilith", wxDefaultPosition,
		wxDefaultSize, 2, lilithstrlst, 1, wxRA_SPECIFY_COLS);

	minorsizer->Add(nnodetype, 0, wxALL, 5);
	minorsizer->Add(lilithtype, 0, wxALL, 5);
	mainsizer->Add(minorsizer, 0, wxALL, 5);

	topsizer->Add(mainsizer);

	wxButton *okbtn = new wxButton(this, wxID_OK, "OK");
	wxButton *cancelbtn = new wxButton(this, wxID_CANCEL, "Cancel");
	
	btnsizer->Add(okbtn, 0, wxALL, 10);
	btnsizer->Add(cancelbtn, 0, wxALL, 10);

	topsizer->Add(btnsizer, 0, wxALIGN_CENTER);

	SetAutoLayout(TRUE);
	SetSizer(topsizer);
	
	topsizer->Fit(this);
	topsizer->SetSizeHints(this);

	SetDefaultItem(okbtn);
}

CCalculationSettingsDlg::~CCalculationSettingsDlg()
{
}

void CCalculationSettingsDlg::OnShow(wxShowEvent &event)
{
	static bool first_time = true;
	if(first_time)
	{
		first_time = false;
		Center();
	}
}




// ----------------------------------------------------------------------------
// time motion settings dialog box
// ----------------------------------------------------------------------------

IMPLEMENT_CLASS(CTimeMotionDlg, wxDialog)
BEGIN_EVENT_TABLE(CTimeMotionDlg, wxDialog)
	EVT_SHOW(CTimeMotionDlg::OnShow)
	EVT_BUTTON(wxID_OK, CTimeMotionDlg::OnOK)
	EVT_BUTTON(wxID_CANCEL, CTimeMotionDlg::OnCancel)
	EVT_CHECKBOX(ID_SEP_TIMERS, CTimeMotionDlg::OnSepTimersChange)
	EVT_CHECKBOX(ID_SMOOTH_MOVEMENT, CTimeMotionDlg::OnSmoothMovementChange)
	EVT_CHECKBOX(ID_PAUSE_TOGGLE, CTimeMotionDlg::OnPauseChange)
END_EVENT_TABLE()

CTimeMotionDlg::CTimeMotionDlg(wxWindow *parent,
	wxWindowID id, const wxString &title, bool is_small, long style) :
	wxDialog(parent, id, title, wxDefaultPosition, wxDefaultSize, style)
{
	const wxString NullList[] = {""};
	const int NStepMethod = 2;
	const wxString StepMethodList[] = {
		"Time Consistent", "Calendar Consistent"};
	const wxString DefaultChartUnit = "Hours";
	const wxString DefaultRealUnit  = "Seconds";

	wxBoxSizer *topsizer = new wxBoxSizer(wxVERTICAL);
	wxBoxSizer *btnsizer = new wxBoxSizer(wxHORIZONTAL);
	wxBoxSizer *fsizer, *msizer, *psizer;
	wxBoxSizer *prsizer, *mtsizer, *ptsizer;
	wxStaticText *movement_time_label, *per_real_time_label;
	wxString check_box_label, panel_title;
	int i;

	notebook = new wxNotebook(this, -1);
#if !(wxCHECK_VERSION(2,6,0))
	wxNotebookSizer *nbsizer = new wxNotebookSizer(notebook);
#endif

	gen_panel = new wxPanel(notebook, -1);
	notebook->AddPage(gen_panel, "General");
	wxBoxSizer *gen_panelsizer = new wxBoxSizer(wxVERTICAL);

	// Overall options
	gen_panelsizer->Add(1, 10);
	sep_timers = new wxCheckBox(gen_panel, ID_SEP_TIMERS,
		_("Separate Chart Speeds (see other tabs ------>)"));
	gen_panelsizer->Add(sep_timers, 0, wxALL, 5);
	smooth_movement = new wxCheckBox(gen_panel, ID_SMOOTH_MOVEMENT,
		_("Smooth Movement"));
	gen_panelsizer->Add(smooth_movement, 0, wxALL, 5);
	pause_toggle = new wxCheckBox(gen_panel, ID_PAUSE_TOGGLE,
		_("Pause at Each Step"));

	// Smooth-Movement Specific Options
	gen_panelsizer->Add(pause_toggle, 0, wxALL, 5);

	wxStaticText *pause_time_label =
		new wxStaticText(gen_panel, -1, _("Pause Duration (sec):"));
	pause_time_text = new wxTextCtrl(gen_panel, -1, "");
	pause_time_text->SetMaxLength(15);
	psizer = new wxBoxSizer(wxHORIZONTAL);
	psizer->Add(pause_time_label, 0, wxALL | wxALIGN_CENTER, 5);
	psizer->Add(pause_time_text,  0, wxALL | wxALIGN_CENTER, 5);
	gen_panelsizer->Add(psizer);

	wxStaticText *frame_rate_label =
		new wxStaticText(gen_panel, -1, _("Frame Rate (fps):"));
	//frame_rate_text = new wxTextCtrl(gen_panel, -1, "");
	frame_rate_slider = new wxSlider(gen_panel, -1, 1, 1, 100,
		wxDefaultPosition, wxDefaultSize, wxSL_HORIZONTAL |	wxSL_LABELS);
	//frame_rate_text->SetMaxLength(15);
	fsizer = new wxBoxSizer(wxHORIZONTAL);
	fsizer->Add(frame_rate_label, 0, wxALL, 5);
	fsizer->Add(frame_rate_slider,  0, wxALL, 5);
	gen_panelsizer->Add(fsizer);
	//wxStaticText *frame_rate_caveat =
	//	new wxStaticText(gen_panel, -1,
	//	_("Note: Maximum frame rate is 100 frames per sec (fps)"));
	//gen_panelsizer->Add(frame_rate_caveat, 0, wxLEFT, 5);

	// Non-Smooth-Movement Specific Option
	step_method = new wxRadioBox(gen_panel, -1, "Step Method",
		wxDefaultPosition, wxDefaultSize, NStepMethod, StepMethodList,
		1, wxRA_SPECIFY_COLS);
	gen_panelsizer->Add(step_method, 0, wxALL, 5);

	// Set initial states of the overall options
	sep_timers->SetValue(false);
	smooth_movement->SetValue(true);
	pause_toggle->SetValue(false);
	pause_time_text->SetValue("2.5");
#ifdef __WXGTK__
	// wxGTK cannot handle the high frame rates
	frame_rate_slider->SetValue(20);
#else
	//frame_rate_text->SetValue("72");
	frame_rate_slider->SetValue(72);
#endif

	/////////////////////////////////////////
	// TEMP -- delete when implemented     //
	pause_toggle->Hide();
	pause_time_label->Hide();
	pause_time_text->Hide();
	gen_panelsizer->Remove(pause_toggle);
	gen_panelsizer->Remove(psizer);
	// END TEMP                            //
	/////////////////////////////////////////

	// Select which charts to animate
	wxStaticBox *wcsb = new wxStaticBox(gen_panel, -1, "Which Chart(s)");
	wxStaticBoxSizer *which_charts = new wxStaticBoxSizer(wcsb, wxVERTICAL);
	for(i = 0; i < TM_NCHARTS; i++)
	{
		switch(i)
		{
		case 0: check_box_label = _("Native"); break;
		case 1: check_box_label = _("Partner"); break;
		case 2: check_box_label = _("Transit"); break;
		default: check_box_label = "ERROR";
		}

	    which_chart[i] = new wxCheckBox(gen_panel, -1, check_box_label);
		which_charts->Add(which_chart[i], 0, wxALL, 2);
		which_chart[i]->SetValue(true);
	}
	which_charts->Add(150, 1);
	gen_panelsizer->Add(which_charts, 0, wxALL, 5);

	gen_panelsizer->Add(1, 20);

	// Movement speed for the charts
	movement_time_label =
		new wxStaticText(gen_panel, -1, _("Chart Movement Time..."));
	gen_panelsizer->Add(movement_time_label, 0, wxALL, 5);
	movement_time_text = new wxTextCtrl(gen_panel, -1, "");
	movement_time_text->SetMaxLength(15);
	movement_time_text->SetValue("1");
	movement_time_unit = new wxComboBox(gen_panel, -1, "",
		wxDefaultPosition, wxDefaultSize, 0, NullList, wxCB_READONLY);
	msizer = new wxBoxSizer(wxHORIZONTAL);
	msizer->Add(movement_time_text, 0, wxALL | wxALIGN_CENTER, 5);
	msizer->Add(movement_time_unit, 0, wxALL | wxALIGN_CENTER, 5);
	gen_panelsizer->Add(msizer);

	per_real_time_label =
		new wxStaticText(gen_panel, -1, _("...Per Real Time"));
	gen_panelsizer->Add(per_real_time_label, 0, wxALL, 5);
	per_real_time_text = new wxTextCtrl(gen_panel, -1, "");
	per_real_time_text->SetMaxLength(15);
	per_real_time_text->SetValue("1");
	per_real_time_unit = new wxComboBox(gen_panel, -1, "",
		wxDefaultPosition, wxDefaultSize, 0, NullList, wxCB_READONLY);
	prsizer = new wxBoxSizer(wxHORIZONTAL);
	prsizer->Add(per_real_time_text, 0, wxALL | wxALIGN_CENTER, 5);
	prsizer->Add(per_real_time_unit, 0, wxALL | wxALIGN_CENTER, 5);
	gen_panelsizer->Add(prsizer);

#if (defined(__WXGTK__) && wxCHECK_VERSION(2,6,0))
	// Kludge to make the dialog look right on wxGTK 2.6
	gen_panelsizer->Add(300, 0);
#endif
	
	gen_panel->SetAutoLayout( TRUE );
	gen_panel->SetSizer( gen_panelsizer );

    // The individual chart tabs
	for(i = 0; i < TM_NCHARTS; i++)
	{
		switch(i)
		{
		case 0: panel_title = "Native"; break;
		case 1: panel_title = "Partner"; break;
		case 2: panel_title = "Transit"; break;
		default: panel_title = "ERROR";
		}

	    wxPanel *panel = chart_panel[i] = new wxPanel(notebook, -1);
		notebook->AddPage(panel, panel_title);

	    wxBoxSizer *panelsizer = new wxBoxSizer( wxVERTICAL );

		panelsizer->Add(1, 10);

		movement_time_label =
			new wxStaticText(panel, -1, _("Chart Movement Time..."));
		panelsizer->Add(movement_time_label, 0, wxALL, 5);
		move_time_text[i] = new wxTextCtrl(panel, -1, "");
		move_time_text[i]->SetMaxLength(15);
		move_time_text[i]->SetValue("1");
		move_time_unit[i] = new wxComboBox(panel, -1, "",
			wxDefaultPosition, wxDefaultSize, 0, NullList, wxCB_READONLY);
		mtsizer = new wxBoxSizer(wxHORIZONTAL);
		mtsizer->Add(move_time_text[i], 0, wxALL | wxALIGN_CENTER, 5);
		mtsizer->Add(move_time_unit[i], 0, wxALL | wxALIGN_CENTER, 5);
		panelsizer->Add(mtsizer);
		
		per_real_time_label =
			new wxStaticText(panel, -1, _("...Per Real Time"));
		panelsizer->Add(per_real_time_label, 0, wxALL, 5);
		per_time_text[i] = new wxTextCtrl(panel, -1, "");
		per_time_text[i]->SetMaxLength(15);
		per_time_text[i]->SetValue("1");
		per_time_unit[i] = new wxComboBox(panel, -1, "",
			wxDefaultPosition, wxDefaultSize, 0, NullList, wxCB_READONLY);
		ptsizer = new wxBoxSizer(wxHORIZONTAL);
		ptsizer->Add(per_time_text[i], 0, wxALL | wxALIGN_CENTER, 5);
		ptsizer->Add(per_time_unit[i], 0, wxALL | wxALIGN_CENTER, 5);
		panelsizer->Add(ptsizer);
		
		panel->SetAutoLayout( TRUE );
		panel->SetSizer( panelsizer );
	}

	// Finish up the notebook sizer


#if !(wxCHECK_VERSION(2,6,0))
	nbsizer->Fit(this);
	nbsizer->SetSizeHints(this);
	topsizer->Add(nbsizer);
#else
	topsizer->Add(notebook, 0, wxALL, 0);
#endif

	// The onmipresent "ok" and "cancel" buttons
	wxButton *okbtn = new wxButton(this, wxID_OK, "&OK");
	wxButton *cancelbtn = new wxButton(this, wxID_CANCEL, "&Cancel");
	
	btnsizer->Add(okbtn, 0, wxALL, 10);
	btnsizer->Add(cancelbtn, 0, wxALL, 10);

	topsizer->Add(btnsizer, 0, wxALIGN_CENTER);

	// Put it all together

	SetAutoLayout(TRUE);
	SetSizer(topsizer);
	
	topsizer->Fit(this);
	topsizer->SetSizeHints(this);

	SetDefaultItem(okbtn);
	UpdateDialog();
	UpdateSpans(step_method->GetSelection() == 0);
	movement_time_unit->SetValue(DefaultChartUnit);
	per_real_time_unit->SetValue(DefaultRealUnit);
	for(i = 0; i < TM_NCHARTS; i++)
	{
		move_time_unit[i]->SetValue(DefaultChartUnit);
		per_time_unit[i]->SetValue(DefaultRealUnit);
	}
}

CTimeMotionDlg::~CTimeMotionDlg()
{
}

void CTimeMotionDlg::OnShow(wxShowEvent &event)
{
	static bool first_time = true;
	if(first_time)
	{
		first_time = false;
		Center();
	}
}

// Altering options can change the rest of the interface; said
// change is centralized in UpdateDialog(), and is simply called.
void CTimeMotionDlg::OnSepTimersChange(wxCommandEvent &event)
{
	UpdateDialog();
}

void CTimeMotionDlg::OnSmoothMovementChange(wxCommandEvent &event)
{
	UpdateDialog();
}

void CTimeMotionDlg::OnPauseChange(wxCommandEvent &event)
{
	UpdateDialog();
}

// CTimeMotionDlg::UpdateDialog(): Updates the state of the dialog,
// specifically the enabled-ness of certain controls based on the
// state of other controls so you can't change something that is
// not being used at the current time, which might confuse certain
// users.
void CTimeMotionDlg::UpdateDialog()
{
	static bool do_smooth_movement_last = false, first_time = true;
	bool use_sep_timers, do_smooth_movement, do_pause;
	wxString old_real_unit;
	int i, j;

	const int NUnits = 7, NAbbrevUnits = 2;
	const wxString UnitsList[] = {"Seconds", "Minutes", "Hours", "Days",
		"Weeks", "Months", "Years"};
	const wxString AbbrevUnitsList[] = {"Seconds", "Minutes"};
	const wxString DefaultRealUnit  = "Seconds";

	// If sep_timers (Separate Timers) is checked, activate the
	// other tabs and deactivate the chart-time-per-unit-time
	// in the general tab, and vise-versa if it's unchecked.
	// Also, change the check box's label to help people find
	// where the individual chart movement settings are.
	use_sep_timers = sep_timers->GetValue();
	for(i = 0; i < TM_NCHARTS; i++)
		chart_panel[i]->Enable(use_sep_timers);
	movement_time_text->Enable(!use_sep_timers);
	movement_time_unit->Enable(!use_sep_timers);
	per_real_time_text->Enable(!use_sep_timers);
	per_real_time_unit->Enable(!use_sep_timers);
	if(!use_sep_timers)
		sep_timers->SetLabel(_("Separate Chart Speeds"));
	else
		sep_timers->SetLabel(_("Separate Chart Speeds (see other tabs ----->)"));

	// Only allow additional pauses and large per-unit time units
	// if smooth movement is enabled; the former as it wouldn't make
	// sense to have two pauses in a row, and the latter as no sane
	// person would want to wait days or years for a movement in the
	// chart.  (If they do, well, it's open-source.)
	if(first_time)
	{
		first_time = false;
		for(j = 0; j < NUnits; j++)
			movement_time_unit->Append(UnitsList[j]);
		for(i = 0; i < TM_NCHARTS; i++)
			for(j = 0; j < NUnits; j++)
				move_time_unit[i]->Append(UnitsList[j]);
	}
	do_smooth_movement = smooth_movement->GetValue();
	if(do_smooth_movement != do_smooth_movement_last)
	{
		do_smooth_movement_last = do_smooth_movement;

		// Enable and disable other controls
		pause_toggle->Enable(do_smooth_movement);
		frame_rate_slider->Enable(do_smooth_movement);
		step_method->Enable(!do_smooth_movement);

		// Update the per-unit combo-boxes
		old_real_unit = per_real_time_unit->GetValue();
		per_real_time_unit->Clear();
		if(do_smooth_movement)
		{
			for(j = 0; j < NUnits; j++)
				per_real_time_unit->Append(UnitsList[j]);
			if(old_real_unit != "")
				per_real_time_unit->SetValue(old_real_unit);
			else
				per_real_time_unit->SetValue(DefaultRealUnit);
		}
		else
		{
			for(j = 0; j < NAbbrevUnits; j++)
				per_real_time_unit->Append(AbbrevUnitsList[j]);
			for(i = 0; (i < NAbbrevUnits) && (old_real_unit != ""); i++)
			{
				if(AbbrevUnitsList[i] == old_real_unit)
				{
					per_real_time_unit->SetValue(old_real_unit);
					break;
				}
			}
			if(per_real_time_unit->GetValue() == "")
			{
				per_real_time_unit->SetValue(DefaultRealUnit);
			}
		}

		for(i = 0; i < TM_NCHARTS; i++)
		{
			old_real_unit = per_time_unit[i]->GetValue();
			per_time_unit[i]->Clear();
			if(do_smooth_movement)
			{
				for(j = 0; j < NUnits; j++)
					per_time_unit[i]->Append(UnitsList[j]);
				if(old_real_unit != "")
					per_time_unit[i]->SetValue(old_real_unit);
				else
					per_time_unit[i]->SetValue(DefaultRealUnit);
			}
			else
			{
				for(j = 0; j < NAbbrevUnits; j++)
					per_time_unit[i]->Append(AbbrevUnitsList[j]);
				for(j = 0; (j < NAbbrevUnits) && (old_real_unit != ""); j++)
				{
					if(AbbrevUnitsList[j] == old_real_unit)
					{
						per_time_unit[i]->SetValue(old_real_unit);
						break;
					}
				}
				if(per_time_unit[i]->GetValue() == "")
				{
					per_time_unit[i]->SetValue(DefaultRealUnit);
				}
			}
		}
	}

	do_pause = pause_toggle->GetValue();
	pause_time_text->Enable(do_smooth_movement && do_pause);
}

bool CTimeMotionDlg::IsTimeSpan(bool sep_rates, int nchart)
{
	if(sep_rates)
		return using_time_span[nchart];

	return using_time_span[TM_NCHARTS];
}

bool CTimeMotionDlg::IsDateSpan(bool sep_rates, int nchart)
{
	if(sep_rates)
		return using_date_span[nchart];

	return using_date_span[TM_NCHARTS];
}

wxTimeSpan CTimeMotionDlg::GetTimeSpan(bool sep_rates, int nchart)
{
	if(sep_rates)
		return timespan[nchart];

	return timespan[TM_NCHARTS];
}

wxDateSpan CTimeMotionDlg::GetDateSpan(bool sep_rates, int nchart)
{
	if(sep_rates)
		return datespan[nchart];

	return datespan[TM_NCHARTS];
}

void CTimeMotionDlg::UpdateSpans(bool translate_to_days)
{
	wxTextCtrl *textctrl;
	wxComboBox *unitctrl;
	for(int i = 0; i < (TM_NCHARTS + 1); i++)
	{
		if(i == TM_NCHARTS)
		{
			textctrl = movement_time_text;
			unitctrl = movement_time_unit;
		}
		else
		{
			textctrl = move_time_text[i];
			unitctrl = move_time_unit[i];
		}

		StrsToWXTimeSpan(textctrl->GetValue(), unitctrl->GetValue(),
			&(timespan[i]), &(datespan[i]), &(using_time_span[i]),
			&(using_date_span[i]), translate_to_days);
	}
}

CTimeMotionDlg::Span CTimeMotionDlg::DecodeTimeUnit(const wxString &units) const
{
	wxString lcunits = units.Lower();
	if(lcunits.Contains("second")) return Seconds;
	else if(lcunits.Contains("minute")) return Minutes;
	else if(lcunits.Contains("hour")) return Hours;
	else if(lcunits.Contains("day")) return Days;
	else if(lcunits.Contains("week")) return Weeks;
	else if(lcunits.Contains("month")) return Months;
	else if(lcunits.Contains("year")) return Years;

	return InvalidSpan;
}

void CTimeMotionDlg::StrsToWXTimeSpan(const wxString &s_amt,
							const wxString &units,
							wxTimeSpan *wxts, wxDateSpan *wxds,
							bool *using_ts, bool *using_ds,
							bool translate_to_days,
							bool prefer_discrete) const
{
	double amt;
	s_amt.ToDouble(&amt);
	AmtAndUnitToWXTimeSpan(amt, units, wxts, wxds,
		using_ts, using_ds, translate_to_days);
}

void CTimeMotionDlg::AmtAndUnitToWXTimeSpan(double amt,
							const wxString &units,
							wxTimeSpan *wxts, wxDateSpan *wxds,
							bool *using_ts, bool *using_ds,
							bool translate_to_days,
							bool prefer_discrete) const
{
	long intamt;
	*using_ds = *using_ts = false;

	// First handle the zero case
	if(fabs(amt) < 0.0001)
	{
		*wxts = wxTimeSpan::Seconds(0);
		return;
	}

	// Now handle the integer case
	if(prefer_discrete && (fabs(amt - floor(amt + 0.5)) < 0.0001))
	{
		intamt = (int)(floor(amt + 0.5));
		switch(DecodeTimeUnit(units))
		{
		case Seconds: *wxts = wxTimeSpan::Seconds(intamt); break;
		case Minutes: *wxts = wxTimeSpan::Minutes(intamt); break;
		case Hours: *wxts = wxTimeSpan::Hours(intamt); break;
		case Days: *wxds = wxDateSpan::Days((int)intamt); break;
		case Weeks: *wxds = wxDateSpan::Weeks((int)intamt); break;
		case Months: *wxds = wxDateSpan::Months((int)intamt); break;
		case Years: *wxds = wxDateSpan::Years((int)intamt); break;
		default:
			*wxts = wxTimeSpan::Seconds(0);
			return;
		}
		switch(DecodeTimeUnit(units))
		{
		case Days:
		case Weeks:
		case Months:
		case Years:
			*using_ds = true;
			break;
		default:
			*using_ts = true;
		}
		return;
	}

	// Finally, the floating-point case.  Months are assumed to be 30 days
	// when there are fractional months to worry about.
	{
		wxString unit;
		wxTimeSpan *tmpts;
		wxDateSpan *tmpds;
		long msec, sec, min, hr;
		int day, mon, wk, yr;
		msec = sec = min = hr = day = wk = mon = yr = 0;
		intamt = (long)(amt);
		unit = units;
		switch(DecodeTimeUnit(unit))
		{
		// Translating years and months to days makes things more
		// consistent, but may not be as intuitive as treating them
		// separately, even with the quirks of differing days in a
		// particular month or year.
		case Years:
			if(translate_to_days)
			{
				yr = 0;
				day = (int)(amt*APPROX_DAYS_IN_YEAR);
			}
			else
			{
				yr = intamt;
				amt -= intamt;
				day = (int)(amt*APPROX_DAYS_IN_YEAR);
			}
			tmpds = new wxDateSpan(0, 0, 0, day);
			*wxds = *tmpds;
			delete tmpds;
			// Calculate a new amount in hours
			amt = ((amt*APPROX_DAYS_IN_YEAR) - day)*24.0;
			unit = "Hours";
			*using_ds = true;
			break;
		case Months:
			if(translate_to_days)
			{
				mon = 0;
				day = (int)(amt*APPROX_DAYS_IN_MONTH);
			}
			else
			{
				mon = intamt;
				amt -= intamt;
				day = (int)(amt*APPROX_DAYS_IN_MONTH);
			}
			tmpds = new wxDateSpan(0, 0, 0, day);
			*wxds = *tmpds;
			delete tmpds;
			amt = ((amt*APPROX_DAYS_IN_MONTH) - day)*24.0;
			unit = "Hours";
			*using_ds = true;
			break;
		case Weeks:
			// For weeks, we don't have to worry about loss of
			// precision as a week is defined as exactly seven
			// days, no ifs, ands, or buts.
			wk = intamt;
			amt -= intamt;
			day = (int)(amt*DAYS_IN_WEEK);
			tmpds = new wxDateSpan(0, 0, 0, day);
			*wxds = *tmpds;
			delete tmpds;
			amt = ((amt*DAYS_IN_WEEK) - day)*24.0;
			unit = "Hours";
			*using_ds = true;
			break;
		case Days:
			// The simplest case is when we already are dealing
			// with days.
			day = intamt;
			amt -= intamt;
			tmpds = new wxDateSpan(0, 0, 0, day);
			amt = ((amt*DAYS_IN_WEEK) - day)*24.0;
			unit = "Hours";
			tmpds = new wxDateSpan(0, 0, 0, day);
			*wxds = *tmpds;
			delete tmpds;
			*using_ds = true;
			break;
		}
		switch(DecodeTimeUnit(unit))
		{
		// Dealing with times is much more straightforward compared
		// to dates.
		case Hours:
			hr = intamt;
			amt -= intamt;
			amt *= 60.0;
			intamt = (long)(amt);
		case Minutes:
			min = intamt;
			amt -= intamt;
			amt *= 60.0;
			intamt = (long)(amt);
		case Seconds:
			// At this rate, all we have left are seconds, so extract the
			// milliseconds and we have wx's highest precision.
			sec = intamt;
			amt -= intamt;
			amt *= 1000.0;
			msec = (long)(amt);
			// Put all we have into the wxTimeSpan!
			tmpts = new wxTimeSpan(hr, min, sec, msec);
			*wxts = *tmpts;
			delete tmpts;
			*using_ts = true;
			break;
		default:
			*wxts = wxTimeSpan::Seconds(0);
			return;
		}
	}
}
