//InputDialogs.h
//
//Copyright (C) 2003 mikesag
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#ifndef __INPUT_DIALOGS_H__
#define __INPUT_DIALOGS_H__

#include "wx/datetime.h"
#include "wx/textctrl.h"
#include "wx/checkbox.h"
#include "wx/listbox.h"
#include "wx/radiobut.h"
#include "wx/radiobox.h"
#include "wx/button.h"
#include "wx/calctrl.h"

#include "AstroCalc.h"

#define ID_GUESS 101

class CInputDateTimePlaceDlg : public wxDialog
{
public:
	CInputDateTimePlaceDlg(wxWindow *parent, wxWindowID id, const wxString &title,
		bool is_small, long style = wxDEFAULT_DIALOG_STYLE);
	virtual ~CInputDateTimePlaceDlg();

	void OnShow(wxShowEvent &event);
	void OnGuessDST(wxCommandEvent &event);

	trTimeDatePlace chartdata[NCHARTTYPES];

	wxCalendarCtrl *cal;
	wxTextCtrl *nametext, *citytext, *timetext, *lattext, *lontext, *tztext;
	wxCheckBox *dst_check, *progressed_check;

private:
	DECLARE_CLASS(CInputDateTimePlaceDlg)
    DECLARE_EVENT_TABLE()
};

#define BASE_R_CB_ID  100
#define BASE_R_ORB_ID 200

class CInputRestrictionsDlg : public wxDialog
{
public:
	CInputRestrictionsDlg(wxWindow *parent, wxWindowID id, const wxString &title,
		bool is_small, long style = wxDEFAULT_DIALOG_STYLE);
	virtual ~CInputRestrictionsDlg();

	void OnShow(wxShowEvent &event);
	void OnButton(wxCommandEvent &event);

	wxCheckBox *r_check[NTOTOBJECTS+12];

private:
	DECLARE_CLASS(CInputRestrictionsDlg)
	DECLARE_EVENT_TABLE()
};

class CInputAspectRestrictionsDlg : public wxDialog
{
public:
	CInputAspectRestrictionsDlg(wxWindow *parent, wxWindowID id, const wxString &title,
		bool is_small, long style = wxDEFAULT_DIALOG_STYLE);
	virtual ~CInputAspectRestrictionsDlg();

	void OnShow(wxShowEvent &event);
	void OnButton(wxCommandEvent &event);

	wxCheckBox *r_check[NASPECTS];
	wxTextCtrl *orb_text[NASPECTS];
	wxString orb_str[NASPECTS];

private:
	DECLARE_CLASS(CInputAspectRestrictionsDlg)
	DECLARE_EVENT_TABLE()
};

class CFindAspectsDlg : public wxDialog
{
public:
	CFindAspectsDlg(wxWindow *parent, wxWindowID id, const wxString &title,
		bool is_small, long style = wxDEFAULT_DIALOG_STYLE);
	virtual ~CFindAspectsDlg();

	void OnShow(wxShowEvent &event);
	void OnButton(wxCommandEvent &event);
	void OnListBoxSel(wxCommandEvent &event);

	wxListBox *planet1, *planet2, *aspect;
	wxTextCtrl *min_orb_text, *max_orb_text;
	wxString min_orb_str, max_orb_str;

	wxButton *all0, *all1, *all2;
	wxButton *none0, *none1, *none2;
	wxButton *toggle0, *toggle1, *toggle2;
	wxRadioButton *sortorb, *sortpl1, *sortpl2;	
	wxCheckBox *needboth;

	bool is_single_chart;
	int lbpl1[NTOTOBJECTS + 12], n_pl1;
	int lbpl2[NTOTOBJECTS + 12], n_pl2;
	int lbasps[NASPECTS], n_asps;

private:
	DECLARE_CLASS(CFindAspectsDlg)
	DECLARE_EVENT_TABLE()
};

class CCalculationSettingsDlg : public wxDialog
{
public:
	CCalculationSettingsDlg(wxWindow *parent, wxWindowID id, const wxString &title,
		bool is_small, long style = wxDEFAULT_DIALOG_STYLE);
	virtual ~CCalculationSettingsDlg();

	void OnShow(wxShowEvent &event);

	wxRadioBox *housesys, *nnodetype, *lilithtype;
private:
	DECLARE_CLASS(CCalculationSettingsDlg)
	DECLARE_EVENT_TABLE()
};

#define TM_NCHARTS 3  /* Native, Partner, Transit */
#define DAYS_IN_WEEK         7
#define APPROX_DAYS_IN_YEAR  365.242198
#define APPROX_DAYS_IN_MONTH 30.43684983 /* Multiply by 12 to get a year */

class CTimeMotionDlg : public wxDialog
{
public:
	CTimeMotionDlg(wxWindow *parent, wxWindowID id, const wxString &title,
		bool is_small, long style = wxDEFAULT_DIALOG_STYLE);
	virtual ~CTimeMotionDlg();

	void OnShow(wxShowEvent &event);
	void OnSepTimersChange(wxCommandEvent &event);
	void OnSmoothMovementChange(wxCommandEvent &event);
	void OnPauseChange(wxCommandEvent &event);

	void UpdateDialog();
	void UpdateSpans(bool translate_to_days = true);
	bool IsTimeSpan(bool sep_rates, int nchart);
	bool IsDateSpan(bool sep_rates, int nchart);
	wxTimeSpan GetTimeSpan(bool sep_rates, int nchart);
	wxDateSpan GetDateSpan(bool sep_rates, int nchart);

	enum Span { InvalidSpan = 0,
		Seconds = 100, Minutes, Hours, Days, Weeks, Months, Years };

	CTimeMotionDlg::Span DecodeTimeUnit(const wxString &units) const;
	void StrsToWXTimeSpan(const wxString &s_amt, const wxString &units,
							wxTimeSpan *wxts, wxDateSpan *wxds,
							bool *using_ts, bool *using_ds,
							bool translate_to_days = true,
							bool prefer_discrete = true) const;
	void AmtAndUnitToWXTimeSpan(double amt, const wxString &units,
							wxTimeSpan *wxts, wxDateSpan *wxds,
							bool *using_ts, bool *using_ds,
							bool translate_to_days = true,
							bool prefer_discrete = true) const;

	wxNotebook *notebook;
	wxPanel *gen_panel, *chart_panel[TM_NCHARTS];
	wxCheckBox *sep_timers, *smooth_movement, *pause_toggle;
	wxCheckBox *which_chart[TM_NCHARTS];
	wxTextCtrl *pause_time_text;//, *frame_rate_text;
	wxSlider *frame_rate_slider;
	wxRadioBox *step_method;
	wxTextCtrl *movement_time_text, *per_real_time_text;
	wxComboBox *movement_time_unit, *per_real_time_unit;
	wxTextCtrl *move_time_text[TM_NCHARTS], *per_time_text[TM_NCHARTS];
	wxComboBox *move_time_unit[TM_NCHARTS], *per_time_unit[TM_NCHARTS];

protected:

	enum {ID_SEP_TIMERS = 100, ID_SMOOTH_MOVEMENT, ID_PAUSE_TOGGLE,
		ID_MOVEMENT_TIME_TEXT, ID_PER_REAL_TIME_TEXT};

	wxTimeSpan timespan[TM_NCHARTS+1];
	wxDateSpan datespan[TM_NCHARTS+1];
	bool using_time_span[TM_NCHARTS+1], using_date_span[TM_NCHARTS+1];
private:
	DECLARE_CLASS(CTimeMotionDlg)
	DECLARE_EVENT_TABLE()
};

#endif
