#include "wx/wxprec.h"

#ifndef WX_PRECOMP
	#include "wx/wx.h"
#endif

#include <math.h>

#include "wx/datetime.h"

#include "Translation.h"
#include "TransitsFrm.h"
#include "TransitsAnim.h"

enum
{
	Anim_Timer,
	Chart1_Timer,
	Chart2_Timer,
	Chart3_Timer
};

// Utility functions

static void TDPtoWX(trTimeDatePlace *tdp, wxDateTime *wxdt)
{
	double secs = tdp->time.sec;
	wxDateTime::wxDateTime_t millisecs = (wxDateTime::wxDateTime_t)
		(fmod(secs - (int)secs, 1000.0)*1000);
	wxdt->Set(tdp->date.day,
		(wxDateTime::Month)(wxDateTime::Jan + tdp->date.month - 1),
		tdp->date.year, tdp->time.hours, tdp->time.min,
		(wxDateTime::wxDateTime_t)tdp->time.sec, millisecs);
}

static void WXtoTDP(wxDateTime *wxdt, trTimeDatePlace *tdp)
{
	tdp->date.day = wxdt->GetDay();
	tdp->date.month = (int)(wxdt->GetMonth() - wxDateTime::Jan) + 1;
	tdp->date.year = wxdt->GetYear();
	tdp->time.hours = wxdt->GetHour();
	tdp->time.min = wxdt->GetMinute();
	tdp->time.sec = wxdt->GetSecond();
	tdp->is_dst = wxdt->IsDST();
}

static long int ToMilliseconds(double amt, const wxString &units)
{
	wxString unit = units.Lower();

	if(unit.Contains("second"))
		return (long)(amt*1000);
	if(unit.Contains("minute"))
		return (long)(amt*60*1000);
	if(unit.Contains("hour"))
		return (long)(amt*60*60*1000);
	if(unit.Contains("day"))
		return (long)(amt*24*60*60*1000);
	if(unit.Contains("week"))
		return (long)(amt*7*24*60*60*1000);
	if(unit.Contains("month"))
		return (long)(amt*APPROX_DAYS_IN_MONTH*24*60*60*1000);
	if(unit.Contains("year"))
		return (long)(amt*APPROX_DAYS_IN_YEAR*24*60*60*1000);

	return 0;
}

static wxLongLong ToMillisecondsLongLong(double amt, const wxString &units)
{
	wxLongLong ll(24*60*60*1000), llamt;
	wxString unit = units.Lower();

	if(unit.Contains("second"))
		return (wxLongLong)(long int)(amt*1000);
	if(unit.Contains("minute"))
		return (wxLongLong)(long int)(amt*60*1000);
	if(unit.Contains("hour"))
		return (wxLongLong)(long int)(amt*60*60*1000);
	if(unit.Contains("day"))
		return (wxLongLong)(long int)(amt*24*60*60*1000);
	if(unit.Contains("week"))
		return (wxLongLong)(long int)(amt*7*24*60*60*1000);
	if(unit.Contains("month"))
	{
		llamt = (wxLongLong)(long int)(amt*APPROX_DAYS_IN_MONTH);
		return llamt*ll;
	}
	if(unit.Contains("year"))
	{
		llamt = (wxLongLong)(long int)(amt*APPROX_DAYS_IN_YEAR);
		return llamt*ll;
	}

	return (wxLongLong)0;
}

/*static double FromMilliseconds(int i_amt, const wxString &units)
{
	double amt = (double)i_amt;
	wxString unit = units.Lower();

	if(unit.Contains("second"))
		return amt/1000.0;
	if(unit.Contains("minute"))
		return amt/(60*1000.0);
	if(unit.Contains("hour"))
		return amt/(60*60*1000.0);
	if(unit.Contains("day"))
		return amt/(24*60*60*1000.0);
	if(unit.Contains("week"))
		return amt/(7*24*60*60*1000.0);
	if(unit.Contains("month"))
		return amt/(APPROX_DAYS_IN_MONTH*24*60*60*1000.0);
	if(unit.Contains("year"))
		return amt/(APPROX_DAYS_IN_YEAR*24*60*60*1000.0);

	return 0.0;
}*/

// Implement the TransitsAnim class

IMPLEMENT_CLASS(TransitsAnim, wxEvtHandler)
BEGIN_EVENT_TABLE(TransitsAnim, wxEvtHandler)
	EVT_TIMER(Anim_Timer, TransitsAnim::OnAnimTimer)
	EVT_TIMER(Chart1_Timer, TransitsAnim::OnChart1Timer)
	EVT_TIMER(Chart2_Timer, TransitsAnim::OnChart2Timer)
	EVT_TIMER(Chart3_Timer, TransitsAnim::OnChart3Timer)
END_EVENT_TABLE()

TransitsAnim::TransitsAnim()
{
	int i;
	active = false;
	smooth_movement = single_rate = false;
	use_timespan = use_datespan = false;
	animtimer = new wxTimer;
	animtimer->SetOwner(this, Anim_Timer);
	for(i = 0; i < TM_NCHARTS; i++)
	{
		charttimer[i] = new wxTimer;
		charttimer[i]->SetOwner(this, Chart1_Timer + i);
	}
}

TransitsAnim::~TransitsAnim()
{
	int i;
	delete animtimer;
	for(i = 0; i < TM_NCHARTS; i++)
		delete charttimer[i];
}

void TransitsAnim::OnChart1Timer(wxTimerEvent &event)
{
	OnChartTimer(event, 0);
}

void TransitsAnim::OnChart2Timer(wxTimerEvent &event)
{
	OnChartTimer(event, 1);
}

void TransitsAnim::OnChart3Timer(wxTimerEvent &event)
{
	OnChartTimer(event, 2);
}

void TransitsAnim::OnChartTimer(wxTimerEvent &event, int n)
{
	// This may be called up to 100 times a second; make the
	// variables static so they don't always have to be
	// reinstantiated at every call.
	static int ping[TM_NCHARTS], i;
	static bool all_pinged;

	if(smooth_movement)
	{
		SmoothMoveOne(n, trfr, mult);

		// Only refresh when all the charts have been updated
		ping[n] = true;
		all_pinged = true;
		for(i = 0; i < TM_NCHARTS; i++)
			if(!(ping[i]))
				all_pinged = false;
		if(all_pinged)
		{
			DoRefresh(trfr);
			for(i = 0; i < TM_NCHARTS; i++)
				ping[i] = false;
		}
	}
	else
	{
		ShiftForwardOne(n, trfr, tmd, mult);
	}
}

void TransitsAnim::OnAnimTimer(wxTimerEvent &event)
{
	if(smooth_movement)
	{
		SmoothMove(trfr, mult);
		DoRefresh(trfr);
	}
	else
	{
		ShiftForward(trfr, tmd, mult);
	}
}

bool TransitsAnim::IsRunning() const
{
	return active;
}

void TransitsAnim::Stop()
{
	if(single_rate)
	{
		animtimer->Stop();
	}
	else
	{
		for(int i = 0; i < TM_NCHARTS; i++)
			charttimer[i]->Stop();
	}
	active = false;
}

void TransitsAnim::Restart()
{
	if(active)
		Stop();

	wxASSERT((trfr != NULL) && (tmd != NULL));

	if((trfr != NULL) && (tmd != NULL) && (mult != 0))
		GoForward(trfr, tmd, mult);
}

void TransitsAnim::DoRefresh(TransitsFrame *tfr)
{
	tfr->Recalculate();
	tfr->ChartCanvas->SetInfoPane(tfr->ChartInfo);
	tfr->ChartCanvas->SetChartData(tfr->chart, tfr->chartdata, tfr->chartdataext);
}

void TransitsAnim::GoForward(TransitsFrame *tfr, CTimeMotionDlg *tmdlg,
								int multiplier)
{
	int milliseconds, msec[TM_NCHARTS], mspf, i;
	double per_time, fps;//, user_chart_time, user_real_time;
	wxString user_chart_units, user_real_units, per_time_units, s;
	smooth_movement = tmdlg->smooth_movement->GetValue();
	single_rate = !(tmdlg->sep_timers->GetValue());
	for(i = 0; i < TM_NCHARTS; i++)
		which_chart[i] = tmdlg->which_chart[i]->GetValue();
	mult = multiplier;
	trfr = tfr;
	tmd = tmdlg;

	if(smooth_movement)
	{
		// Whether the user chooses single rate for all the charts or
		// multiple rates, the frames per second is the same, so we
		// use the same amount of time for all times, which is the
		// milliseconds per frame, or 1000/fps.
		fps = (double)tmdlg->frame_rate_slider->GetValue();
		mspf = (int)floor((1000.0/fps) + 0.5);
		if(mspf <= 0)
		{
			wxMessageBox("Error: Invalid frames per second", "Error!",
				wxOK | wxCENTRE | wxICON_ERROR);
			return;
		}

		// As the frame rate is the same, here it doesn't matter
		// whether there are separate movement rates or not.
		SetupSmoothMovement(tfr, tmdlg);

		if(active)
			Stop();
		active = true;
		if(single_rate)
			animtimer->Start(mspf);
		else
			for(i = 0; i < TM_NCHARTS; i++)
				charttimer[i]->Start(mspf);
	}
	else
	{
		// If there's no smooth movement, we just take the user's
		// "per-unit" value directly as the "frame rate".
		if(single_rate)
		{
			tmdlg->per_real_time_text->GetValue().ToDouble(&per_time);
			per_time_units = tmdlg->per_real_time_unit->GetStringSelection();
			milliseconds = ToMilliseconds(per_time, per_time_units);
			if(milliseconds <= 0)
			{
				wxMessageBox("Error: Invalid time step", "Error!",
					wxOK | wxCENTRE | wxICON_ERROR);
				return;
			}
			if(active)
				Stop();
			active = true;
			animtimer->Start(milliseconds);
		}
		else
		{
			for(i = 0; i < TM_NCHARTS; i++)
			{
				tmdlg->per_time_text[i]->GetValue().ToDouble(&per_time);
				per_time_units = tmdlg->per_time_unit[i]->GetStringSelection();
				msec[i] = ToMilliseconds(per_time, per_time_units);
				if(msec[i] <= 0)
				{
					s.Printf("Error: Invalid time step (%s chart)",
					    (i == 0) ? "native" :
					    (i == 1) ? "partner" :
					               "transit");
					wxMessageBox(s, "Error!",
						wxOK | wxCENTRE | wxICON_ERROR);
					return;
				}
			}
			if(active)
				Stop();
			active = true;
			for(i = 0; i < TM_NCHARTS; i++)
				charttimer[i]->Start(msec[i]);
		}
	}
}

void TransitsAnim::GoBackward(TransitsFrame *tfr, CTimeMotionDlg *tmdlg,
								int multiplier)
{
	GoForward(tfr, tmdlg, -multiplier);
}

void TransitsAnim::ShiftForward(TransitsFrame *tfr, CTimeMotionDlg *tmdlg,
								int multiplier)
{
	wxDateTime wxdt;
	wxDateSpan wxdtspan;
	wxTimeSpan wxtmspan;
	int i;

	// Loop over all animatable charts, only animating those that
	// are set to be animated.
	for(i = CHART_NATAL1; i < (CHART_NATAL1 + TM_NCHARTS); i++)
	{
		if(tmdlg->which_chart[i-CHART_NATAL1]->GetValue())
		{
			TDPtoWX(&(tfr->chartdata[i]), &wxdt);
			// First, handle the case of a single jump rate
			if(!(tmdlg->sep_timers->GetValue()))
			{
				if(tmdlg->IsDateSpan(false, 0))
				{
					wxdtspan = tmdlg->GetDateSpan(false, 0);
					wxdt += multiplier*wxdtspan;
				}
				if(tmdlg->IsTimeSpan(false, 0))
				{
					wxtmspan = tmdlg->GetTimeSpan(false, 0);
					wxdt += multiplier*wxtmspan;
				}
			}
			// Now, handle the case of separate jump rates
			else
			{
				if(tmdlg->IsDateSpan(true, i-CHART_NATAL1))
				{
					wxdtspan = tmdlg->GetDateSpan(true, i-CHART_NATAL1);
					wxdt += multiplier*wxdtspan;
				}
				if(tmdlg->IsTimeSpan(true, i-CHART_NATAL1))
				{
					wxtmspan = tmdlg->GetTimeSpan(true, i-CHART_NATAL1);
					wxdt += multiplier*wxtmspan;
				}
			}
			WXtoTDP(&wxdt, &(tfr->chartdata[i]));
		}
	}
	DoRefresh(tfr);
}

void TransitsAnim::ShiftForwardOne(int n, TransitsFrame *tfr,
								   CTimeMotionDlg *tmdlg, int multiplier)
{
	wxDateTime wxdt;
	wxDateSpan wxdtspan;
	wxTimeSpan wxtmspan;

	if(!(tmdlg->which_chart[n]->GetValue()))
		return;
	
	TDPtoWX(&(tfr->chartdata[n+CHART_NATAL1]), &wxdt);
	if(tmdlg->IsDateSpan(true, n))
	{
		wxdtspan = tmdlg->GetDateSpan(true, n);
		wxdt += multiplier*wxdtspan;
	}
	if(tmdlg->IsTimeSpan(true, n))
	{
		wxtmspan = tmdlg->GetTimeSpan(true, n);
		wxdt += multiplier*wxtmspan;
	}
	WXtoTDP(&wxdt, &(tfr->chartdata[n+CHART_NATAL1]));
	DoRefresh(tfr);
}

void TransitsAnim::ShiftBackward(TransitsFrame *tfr, CTimeMotionDlg *tmdlg,
								int multiplier)
{
	ShiftForward(tfr, tmdlg, -multiplier);
}

void TransitsAnim::SetupSmoothMovement(TransitsFrame *tfr, CTimeMotionDlg *tmdlg, int n)
{
	wxString moveunits, realunits;
	double movetim, realtim;
	int start, end, i;

#ifdef WXDEBUG
	if(n > TM_NCHARTS) wxMessageBox("ERROR: n > TM_CHARTS");
#endif

	if((n < 0) || (n > TM_NCHARTS))
	{
		start = 0;
		end = TM_NCHARTS;
	}
	else
	{
		start = n;
		end = n + 1;
	}

	if(single_rate)
	{
		tmdlg->movement_time_text->GetValue().ToDouble(&movetim);
		tmdlg->per_real_time_text->GetValue().ToDouble(&realtim);
		moveunits = tmdlg->movement_time_unit->GetValue();
		realunits = tmdlg->per_real_time_unit->GetValue();
		move_time = ToMillisecondsLongLong(movetim, moveunits);
		real_time = ToMillisecondsLongLong(realtim, realunits);
	}
	else
	{
		for(i = start; i < end; i++)
		{
			tmdlg->move_time_text[i]->GetValue().ToDouble(&movetim);
			tmdlg->per_time_text[i]->GetValue().ToDouble(&realtim);
			moveunits = tmdlg->move_time_unit[i]->GetValue();
			realunits = tmdlg->per_time_unit[i]->GetValue();
			move_times[i] = ToMillisecondsLongLong(movetim, moveunits);
			real_times[i] = ToMillisecondsLongLong(realtim, realunits);
		}
	}

	for(i = start; i < end; i++)
	{
		timer_start[i] = wxDateTime::UNow();
		TDPtoWX(&(tfr->chartdata[i+CHART_NATAL1]), &orig_chart_dt[i]);
	}
}

void TransitsAnim::SmoothMoveOne(int i, TransitsFrame *tfr, int multiplier)
{
	wxLongLong msec, deltat;
	wxTimeSpan wxts;
	wxDateTime wxdt;

	if(which_chart[i])
	{
		wxts = wxDateTime::UNow() - timer_start[i];
		msec = wxts.GetMilliseconds();
		if(single_rate)
			deltat = (((wxLongLong)multiplier)*(msec*move_time))/real_time;
		else
			deltat = (((wxLongLong)multiplier)*(msec*move_times[i]))/real_times[i];
		wxdt = orig_chart_dt[i] + wxTimeSpan(deltat);
		WXtoTDP(&wxdt, &(tfr->chartdata[i+CHART_NATAL1]));
	}
}

void TransitsAnim::SmoothMove(TransitsFrame *tfr, int multiplier)
{
	for(int i = 0; i < TM_NCHARTS; i++)
		SmoothMoveOne(i, tfr, multiplier);
}
