//TransitsAnim.h
//
//Copyright (C) 2003 mikesag
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//This is the class that handles anything related to the orderly
//movement of the times for one or more of the charts.  Animation
//in other words.  It is possible that in the future, it will be
//possible to move places too, a sort-of astrological "fly-by",
//perhaps in combination with changing times as well.  As this
//requires such intimate contact with the main frame, it is
//declared as a friend class.

#ifndef TransitsAnim_H
#define TransitsAnim_H

#include "InputDialogs.h"

class TransitsFrame;

class TransitsAnim : public wxEvtHandler
{
public:
	TransitsAnim();
	~TransitsAnim();

	void ShiftForward(TransitsFrame *tfr, CTimeMotionDlg *tmdlg, int multiplier = 1);
	void ShiftBackward(TransitsFrame *tfr, CTimeMotionDlg *tmdlg, int multiplier = 1);

	void GoForward(TransitsFrame *tfr, CTimeMotionDlg *tmdlg, int multiplier = 1);
	void GoBackward(TransitsFrame *tfr, CTimeMotionDlg *tmdlg, int multiplier = 1);

	void Stop();
	void Restart();
	bool IsRunning() const;

	void OnAnimTimer(wxTimerEvent &event);
	void OnChartTimer(wxTimerEvent &event, int n);

	void OnChart1Timer(wxTimerEvent &event);
	void OnChart2Timer(wxTimerEvent &event);
	void OnChart3Timer(wxTimerEvent &event);

protected:
	void ShiftForwardOne(int n, TransitsFrame *tfr, CTimeMotionDlg *tmdlg, int multiplier = 1);
	void SmoothMove(TransitsFrame *tfr, int multiplier = 1);
	void SmoothMoveOne(int n, TransitsFrame *tfr, int multiplier = 1);
	void SetupSmoothMovement(TransitsFrame *tfr, CTimeMotionDlg *tmdlg, int n = -1);
	void DoRefresh(TransitsFrame *tfr);

	int mult; // Multiplier for faster motion, larger jumps, or reversing course
	
	bool which_chart[TM_NCHARTS];
	bool smooth_movement, single_rate, active;
	wxTimer *animtimer, *charttimer[TM_NCHARTS];

	bool use_timespan, use_datespan;
	wxDateSpan datespan;
	wxTimeSpan timespan;

	bool use_ts[TM_NCHARTS], use_ds[TM_NCHARTS];
	wxDateSpan datespans[TM_NCHARTS];
	wxTimeSpan timespans[TM_NCHARTS];
	wxDateTime timer_start[TM_NCHARTS], orig_chart_dt[TM_NCHARTS];
	wxLongLong move_time, real_time;
	wxLongLong move_times[TM_NCHARTS], real_times[TM_NCHARTS];

	TransitsFrame *trfr;
	CTimeMotionDlg *tmd;

private:
	// wxWindows administrivia
    DECLARE_CLASS(TransitsAnim)
    // any class wishing to process wxWindows events must use this macro
    DECLARE_EVENT_TABLE()
};

#endif
