//TransitsApp.cpp: The application-level code that creates the first
//                 frame and shows it
//
//Copyright (C) 2003 mikesag
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// ----------------------------------------------------------------------------
// headers
// ----------------------------------------------------------------------------
#include "wx/wxprec.h"

#ifndef WX_PRECOMP
	#include "wx/wx.h"
#endif

#include "TransitsApp.h"
#include "TransitsFrm.h"
#include "TurtleDraw.h"
#include "AstroConst.h"
#include "AstroCalc.h"
#include "resource.h"



// Create a new application object: this macro will allow wxWindows to create
// the application object during program execution (it's better than using a
// static object for many reasons) and also declares the accessor function
// wxGetApp() which will return the reference of the right type (i.e. TransitsApp and
// not wxApp)
IMPLEMENT_APP(TransitsApp)

// ============================================================================
// implementation
// ============================================================================

// ----------------------------------------------------------------------------
// the application class
// ----------------------------------------------------------------------------

TransitsApp::TransitsApp()
{
}

TransitsApp::~TransitsApp()
{
}

// 'Main program' equivalent: the program execution "starts" here
bool TransitsApp::OnInit()
{
	int ScreenWidth, ScreenHeight;
	wxSize FrameSize;
	wxString apptitle;

	// Initialize the astrology library
	init_astro_library(argv[0]);

	// Initialize the glyph-drawing routines
	InitializeTurtle();
	
	wxDisplaySize(&ScreenWidth, &ScreenHeight);
	if(ScreenWidth >= 1200)
		FrameSize.Set(1100,900);
	else if(ScreenWidth >= 1024)
		FrameSize.Set(850,650);
	else if(ScreenWidth >= 640)
		FrameSize.Set(ScreenWidth - 150, ScreenHeight - 150);
	// Otherwise, just leave it at the default width and height
	// Note that it won't look very good much below 640 by 480
	
	apptitle = _("Transitarium Alpha 6");

	// create the main application window
	TransitsFrame *frame = new TransitsFrame(apptitle,
		wxPoint(50, 50), FrameSize,
		wxPathOnly(argv[0]));
	
	// and show it (the frames, unlike simple controls, are not shown when
	// created initially)
	frame->Show(TRUE);
	SetTopWindow(frame);
	
	// success: wxApp::OnRun() will be called which will enter the main message
	// loop and the application will run. If we returned FALSE here, the
	// application would exit immediately.
	return TRUE;
}

int TransitsApp::OnExit()
{
	close_astro_library();
	return wxApp::OnExit();
}
