//TransitsChart.h
//
//Copyright (C) 2003 mikesag
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//This is where the chart itself is drawn

#ifndef TransitsChart_H
#define TransitsChart_H

#ifndef CChartInfo
class CChartInfo;
#endif

// Display types, as in what is really showing on the screen
#define NATAL_CHART     0x01
#define COMPOSITE_CHART 0x02
#define DAVISON_CHART   0x04
#define SYNASTY_CHART   0x08
#define TRANSITS_CHART  0x10

#define ASPECTS_CHART     0x100
#define ASPCONFIG_CHART   0x200
#define DISPOSITORS_CHART 0x400
#define DISPOSITORS_HOUSE_CHART 0x800

#define OVER_NOTHING (-126)
#define OVER_ASPECTS (-127)

#define IS_ONE_CHART(m)  (((m) & NATAL_CHART) || ((m) & COMPOSITE_CHART) || ((m) & DAVISON_CHART))
#define IS_TWO_CHARTS(m) (((m) & SYNASTY_CHART) || ((m) & TRANSITS_CHART))

struct obj_loc
{
	int x;
	int y;
	int h;
	int w;
	double deg;
	double adj_deg;
	int n; // Object number, for showing and naming
};

struct house_loc
{
	double deg;
	int num;
};

typedef house_loc sign_loc;

struct aspect_def
{
	double dist;
	double orb;
	int n;
};

struct aspect_storage
{
	int nchart1, nchart2, n_aspect;
	int n1, n2;
	int x1, y1, x2, y2;
	double th1, th2, orb;
	wxArrayInt aspconf; // Part of which aspect configuration(s)?
};

class aspect_configuration;

WX_DECLARE_OBJARRAY(aspect_configuration, aspect_configuration_array);

class aspect_configuration
{
public:
	int kind;
	wxArrayInt aspect; // Points back to aspect_cache (of type aspect_storage)
	wxArrayInt planet; // A list of the planets involved

	void Clear(); // Clears the memory used in the wxArrayInts
	virtual ~aspect_configuration(); // Ditto
};

struct trPoint
{
	int x;
	int y;
};

// The name and the city name of the native are not really part of the
// astrological calculations; they are merely for show.  Therefore they
// can't be put in AstroCalc.h, where the main data structure is
// defined, as they are strings, which would be stored as wxStrings,
// and AstroCalc can't depend on wxWindows for it to be (theoretically)
// portable in the case of a toolkit change (or port).  The next
// structure therefore exists solely to hold the name and city.
// Also, put in a "dirty flag" on whether the chart has been set to now
// or not; this is used when saving a chart as default.
struct trChartDataExt
{
	wxString namestr;
	wxString citystr;
	bool set_to_now;
};

class CChartCanvas: public wxWindow
{
public:
	CChartCanvas(wxFrame *parent);
	
	void SetChartData(trChartInfo ci[NCHARTTYPES], trTimeDatePlace cd[NCHARTTYPES],
		trChartDataExt ce[NCHARTTYPES], bool do_refresh = TRUE);
	void SetInfoPane(CChartInfo *chartinfopane);
	void SetRestrictions(bool restr[NCHARTTYPES][NTOTOBJECTS+12], bool r_asp[NASPECTS],
		double asp_orbs[NASPECTS], bool do_refresh = TRUE);
	void SetMode(int newmode, int n, int n1, int n2, 
		bool recalc_aspects = TRUE, bool do_refresh = TRUE);
	void Initialize();

    void OnPaint(wxPaintEvent &event);
	void OnResize(wxSizeEvent &event);
    void OnMouseMove(wxMouseEvent &event);
	void OnMouseDown(wxMouseEvent &event);
	void OnMouseLeaveWindow(wxMouseEvent &event);
	void OnDropFiles(wxDropFilesEvent &event);

protected:
	int chart1, chart2, mode;
	trChartInfo chart[NCHARTTYPES];
	trTimeDatePlace chartdata[NCHARTTYPES];
	trChartDataExt chartdataext[NCHARTTYPES];
	aspect_def aspects[NASPECTS];
	CChartInfo *infopane;
    wxFrame *owner;

	void DrawChart(wxDC &dc, int x1, int y1, int w, int h, int n1, int n2,
	               int chart_kind);
	void DrawNatalChart(wxDC &dc, int x1, int y1, int w, int h, int n, int chart_kind);
	void DrawSynastyChart(wxDC &dc, int x1, int y1, int w, int h, int n1, int n2, int chart_kind);
	void DrawBareChart(wxDC &dc, int x1, int y1, int w, int h, int n, bool is_duel);
	void DrawPlanets(wxDC &dc, int x1, int y1, int w, int h, int n,
	                 double asc, bool is_duel, bool is_second,
					 bool by_sign, bool by_house);
	void DrawAspects(wxDC &dc, int x1, int y1, int w, int h, int n1, int n2,
	                 bool is_duel, bool is_aspconfig = false);
	void DrawDispositors(wxDC &dc, int x1, int y1, int w, int h, int n, bool by_sign);
	void DrawLineAtAngle(wxDC &dc, int x1, int y1, double r1, double r2,
		                 double angle_degrees1, double angle_degrees2, bool is_arrow = false);
	void DrawGlyphAtAngle(wxDC &dc, int n, int scale, int x1, int y1, double r,
		                  double angle_degrees);
	void DrawTextAtAngle(wxDC &dc, int x1, int y1, double r, wxString &text,
		                 double angle_degrees);
	void CalculateAspects(int n1, int n2, bool is_duel);
	void CalculateAspectConfigurations(int n);

	int mouse_over_obj, mouse_over_chart, mouse_over_mode;
	int n_aspects_over;
	bool aspect_over[(NTOTOBJECTS+12)*(NTOTOBJECTS+12)];
	
private:
	// works just like in Astrolog; a restricted planet/cusp/object is not shown
	bool restricted[NCHARTTYPES][NTOTOBJECTS+12];
	bool r_aspects[NASPECTS];
	double aspect_orb[NASPECTS];

	obj_loc obj[NCHARTTYPES][NTOTOBJECTS+12];
	house_loc house[NCHARTTYPES][14];
	sign_loc sign[NCHARTTYPES][14];

	trPoint chartloc[NCHARTTYPES];
	trPoint chartsize[NCHARTTYPES];
	int r_outer_sign[NCHARTTYPES];
	int r_inner_sign[NCHARTTYPES];
	int r_outer_aspect[NCHARTTYPES];

	int n_aspect_cache;
	aspect_storage aspect_cache[(NTOTOBJECTS+12)*(NTOTOBJECTS+12)];
	aspect_configuration_array aspect_conf;

	void get_chart_dimensions(int w, int h, bool is_duel, wxCoord *d, wxCoord *d2,
		int *glyph_scale, int *glyph_size, int *glyph_r, int *inner_circle_r);
	void spread_out(int n, double orb);
	void store_loc_radial_deg(int nchart, int nobj, int x1, int y1,
		double r, double angle_degrees, int w, int h);
	void store_aspect(int n_aspect, int nchart1, int nchart2, int obj1, int obj2,
		double theta1, double theta2, double orb, int replacing = -1);
	void fixate_aspect(int n, int x1, int y1, double r1, double r2);
	void aspect_conf_find(wxArrayInt &asps, const int *aspconfdef[],
		unsigned int max_pl, unsigned int max_depth,
		wxArrayInt aspectsleft[], bool outofrunning[]);

    DECLARE_CLASS(CChartCanvas)
    // any class wishing to process wxWindows events must use this macro
	DECLARE_EVENT_TABLE()
};

#include "TransitsInfo.h"

#endif
