//TransitsFiles.h
//
//Copyright (C) 2003 mikesag
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//This is the class that handles loading to and saving from files.
//For easy access to all the internal variables, it is declared a
//friend class of TransitsFrame.  This is mostly done in order to
//logically split the handling of GUI-related work and file I/O,
//and to prevent TransitsFrame from becoming too gigantic.

#ifndef TransitsFiles_H
#define TransitsFiles_H

#define trCONFIG_PLANETS_SHOWN 0x01
#define trCONFIG_ASPECTS_SHOWN 0x02
#define trCONFIG_ASPECT_ORBS   0x04
#define trCONFIG_CALC_SETTINGS 0x08
#define trCONFIG_ANIM_SETTINGS 0x10

#define trCONFIG_ALL           0x80000000 /* aka 2^31 */

wxString DMStoString(const trDMS *dms, const wxString &pos,
					 const wxString &neg);
int StringToDMS(const wxString &str, trDMS *dms,
				 const wxString &pos, const wxString &neg, bool max180);
wxString trDateToString(const trDate *trdt);
void StringToTrDate(const wxString &str, trDate *trdt);
wxString HMStoString(const trHMS *hms);
void StringToHMS(const wxString &str, trHMS *hms);

const wxString trPrefsDir(const wxString &progdir);
const wxString trPrefsFile(const wxString &progdir);
const wxString trPrefsDefChart(const wxString &progdir);

#ifdef __WXMSW__
#define trWIN32S 0 /* Windows 3.0, 3.1 */
#define trWIN9X  1 /* Windows 95, 98, ME */
#define trWINNT  2 /* Windows NT, 2K, XP*/
int GetWindowsVersion();
#endif

class TransitsFrame;

class TransitsFiles
{
public:
	// Configuration file routines
	bool IsValidConfigFile(const wxString &filename, bool verbose);
	void GetConfigFile(TransitsFrame *tfr, int nchart, const wxString &filename,
		wxUint32 flags);
	void PutConfigFile(TransitsFrame *tfr, int nchart, const wxString &filename,
		wxUint32 flags, bool overwrite);

	// Native file routines
	bool IsValidChartDataFile(const wxString &filename, bool verbose);
	void GetChartDataFile(TransitsFrame *tfr, int nchart, const wxString &filename);
	void PutChartDataFile(TransitsFrame *tfr, int nchart, const wxString &filename);

	// Astrolog file routines
	bool IsValidAstrologFile(const wxString &filename, bool verbose);
	void GetAstrologFile(TransitsFrame *tfr, int nchart, const wxString &filename);
	void PutAstrologFile(TransitsFrame *tfr, int nchart, const wxString &filename);
};

#endif
