//TransitsFrm.cpp: The main frame of the application
//
//Copyright (C) 2003 mikesag
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// ----------------------------------------------------------------------------
// headers
// ----------------------------------------------------------------------------
#include "wx/wxprec.h"

#ifndef WX_PRECOMP
	#include "wx/wx.h"
#endif

#include <math.h>

#include "wx/string.h"
#include "wx/tokenzr.h"
#include "wx/settings.h"
#include "wx/statusbr.h"
#include "wx/dynarray.h"
#include "wx/datetime.h"
#include "wx/notebook.h"
#include "wx/filedlg.h"
#include "wx/timer.h"
#if wxUSE_DRAG_AND_DROP
#include "wx/dnd.h"
#endif

#include "AstroConst.h"
#include "AstroCalc.h"

#include "TurtleDraw.h"
#include "Translation.h"

#include "TransitsFiles.h"
#include "TransitsChart.h"
#include "TransitsApp.h"
#include "TransitsFrm.h"

#ifdef __WXMSW__
#define USE_RADIO_MENU_ITEMS
#endif

#ifdef __WXMSW__
#define NATIVEFILEFILTER "Transitarium Chart File (*.trs)|*.trs|All Files (*.*)|*.*"
#define ASTFILEFILTER "All Files (*.*)|*.*"
#define CONFIGFILEFILTER "Transitarium Configuration File (*.trc)|*.trc|All Files (*.*)|*.*"
#else
#define NATIVEFILEFILTER "Transitarium Chart File (*.trs)|*.trs|All Files (*)|*"
#define ASTFILEFILTER "All Files (*)|*"
#define CONFIGFILEFILTER "Transitarium Configuration File (*.trc)|*.trc|All Files (*)|*"
#endif

// ----------------------------------------------------------------------------
// resources
// ----------------------------------------------------------------------------
// the application icon
#ifndef __WXMSW__
	#include "Transits.xpm"
#endif

// ----------------------------------------------------------------------------
// constants
// ----------------------------------------------------------------------------

// IDs for the controls and the menu commands
enum
{
	ImportConfiguration,
	ExportConfiguration,
	RestoreDefaultConfiguration,
	RestoreDefaultConfigurationAll,
	SaveConfigurationAsDefault,

	SaveAsSubMenuID,
	OpenSubMenuID,
	ImportALogSubMenuID,
	ExportALogSubMenuID,

	ChartInfoSubMenuID,
	RestrictionsSubMenuID,
	ChartForNowSubMenuID,

	OpenChart_1,
	SaveChartAs_1,
	Import_Astrolog_1,
	Export_Astrolog_1,

	OpenChart_2,
	SaveChartAs_2,
	Import_Astrolog_2,
	Export_Astrolog_2,

	OpenChart_3,
	SaveChartAs_3,
	Import_Astrolog_3,
	Export_Astrolog_3,

	SaveChartAs_Dav,
	Export_Astrolog_Dav,

	Set_To_Now_1,
	TDPInputDlg_1,
	RestrInputDlg_1,

	Set_To_Now_2,
	TDPInputDlg_2,
	RestrInputDlg_2,

	Set_To_Now_3,
	TDPInputDlg_3,
	RestrInputDlg_3,

	RestrInputDlg_4,
	RestrInputDlg_5,
	AspectRestrDlg,

	Set_To_Now_Def,
	TDPInputDlg_Def,
	RestrInputDlg_Def,
	OpenChart_Def,
	SaveChartAs_Def,
	Import_Astrolog_Def,
	Export_Astrolog_Def,

	Swap_1_2,
	Swap_1_3,
	Swap_2_3,
	Swap_1_Def,

	Calc_Settings,

    Quit_App,
    About_App,

	Now_Timer,

	TimeShiftForward,
	TimeShiftBackward,
	TimeGoForward,
	TimeGoBackward,
	TimeStop,
	TimeSettings,

	ViewNatal,
	ViewComposite,
	ViewDavison,
	ViewSynasty,
	ViewWithTransits,

	ViewAspects,
	ViewAspConfig,
	ViewDispositors,
	ViewDispositorsByHouse,

	AspectsViewAll,
	AspectsViewByPlanet1,
	AspectsViewByPlanet2,
	AspectsViewByOrb,
	AspectsSearch
};

static const int TIMER_INTERVAL = 10;

static const int NATIVE_FILE   = 0;
static const int ASTROLOG_FILE = 1;

// ----------------------------------------------------------------------------
// file drag-and-drop
// ----------------------------------------------------------------------------

#if wxUSE_DRAG_AND_DROP
class CDnDFile : public wxFileDropTarget
{
public:
    CDnDFile(TransitsFrame *p_owner) { owner = p_owner; }
    virtual bool OnDropFiles(wxCoord x, wxCoord y, const wxArrayString &filenames);
private:
    TransitsFrame *owner;
};

bool CDnDFile::OnDropFiles(wxCoord x, wxCoord y, const wxArrayString &filenames)
{
    size_t n, nfiles = filenames.GetCount();

	// Bring the frame to the top; otherwise, it gets confusing
	owner->Raise();

	// Make sure that all files dropped are valid
    for(n = 0; n < nfiles; n++)
	{
		if( (!owner->FileUtils.IsValidChartDataFile(filenames[n], FALSE)) &&
			(!owner->FileUtils.IsValidAstrologFile(filenames[n], FALSE)))
		{
			wxMessageBox("Not all files dropped are valid data files\n"
				"Please select different files and try again.",
				wxMessageBoxCaptionStr, wxOK | wxCENTER | wxICON_ERROR);
			return FALSE;
		}
    }

	// If one valid file is dropped, then set it to the natal chart
	// If two are dropped, set natal & partner, unless transits are showing
	// If three are dropped, set natal, partner & transits
	if(!((n == 2) && (owner->GetMode() == TRANSITS_CHART)))
	{
		if(n >= 1) owner->FileUtils.GetChartDataFile(owner, CHART_NATAL1, filenames[0]);
		if(n >= 2) owner->FileUtils.GetChartDataFile(owner, CHART_NATAL2, filenames[1]);
		if(n >= 3) owner->FileUtils.GetChartDataFile(owner, CHART_TRANSITS, filenames[2]);
	}
	else
	{
		owner->FileUtils.GetChartDataFile(owner, CHART_NATAL1, filenames[0]);
		owner->FileUtils.GetChartDataFile(owner, CHART_TRANSITS, filenames[1]);
	}

    return TRUE;
}
#endif

// ----------------------------------------------------------------------------
// event tables and other macros for wxWindows
// ----------------------------------------------------------------------------

// the event tables connect the wxWindows events with the functions (event
// handlers) which process them. It can be also done at run-time, but for the
// unchanging events like these the static method is much simpler.
IMPLEMENT_CLASS(TransitsFrame, wxFrame)
BEGIN_EVENT_TABLE(TransitsFrame, wxFrame)
	EVT_CLOSE(                  TransitsFrame::OnClose)

    EVT_MENU(Quit_App,          TransitsFrame::OnQuit)
    EVT_MENU(About_App,         TransitsFrame::OnAbout)

    EVT_MENU(ImportConfiguration,        TransitsFrame::OnImportConfig)
    EVT_MENU(ExportConfiguration,        TransitsFrame::OnExportConfig)
    EVT_MENU(SaveConfigurationAsDefault, TransitsFrame::OnSaveDefaultConfig)
    EVT_MENU(RestoreDefaultConfiguration,TransitsFrame::OnRestoreDefaultConfig)
    EVT_MENU(RestoreDefaultConfigurationAll,TransitsFrame::OnRestoreDefaultConfigAll)

	EVT_MENU(Set_To_Now_1,      TransitsFrame::OnSetToNow_1)
	EVT_MENU(TDPInputDlg_1,     TransitsFrame::OnTDPInputDlg_1)
	EVT_MENU(RestrInputDlg_1,   TransitsFrame::OnRestrInputDlg_1)
	EVT_MENU(OpenChart_1,       TransitsFrame::OnOpenChart_1)
	EVT_MENU(SaveChartAs_1,     TransitsFrame::OnSaveChartAs_1)
	EVT_MENU(Import_Astrolog_1, TransitsFrame::OnImportAstrolog_1)
	EVT_MENU(Export_Astrolog_1, TransitsFrame::OnExportAstrolog_1)
	
	EVT_MENU(Set_To_Now_2,      TransitsFrame::OnSetToNow_2)
	EVT_MENU(TDPInputDlg_2,     TransitsFrame::OnTDPInputDlg_2)
	EVT_MENU(RestrInputDlg_2,   TransitsFrame::OnRestrInputDlg_2)
	EVT_MENU(OpenChart_2,       TransitsFrame::OnOpenChart_2)
	EVT_MENU(SaveChartAs_2,     TransitsFrame::OnSaveChartAs_2)
	EVT_MENU(Import_Astrolog_2, TransitsFrame::OnImportAstrolog_2)
	EVT_MENU(Export_Astrolog_2, TransitsFrame::OnExportAstrolog_2)

	EVT_MENU(Set_To_Now_3,      TransitsFrame::OnSetToNow_3)
	EVT_MENU(TDPInputDlg_3,     TransitsFrame::OnTDPInputDlg_3)
	EVT_MENU(RestrInputDlg_3,   TransitsFrame::OnRestrInputDlg_3)
	EVT_MENU(OpenChart_3,       TransitsFrame::OnOpenChart_3)
	EVT_MENU(SaveChartAs_3,     TransitsFrame::OnSaveChartAs_3)
	EVT_MENU(Import_Astrolog_3, TransitsFrame::OnImportAstrolog_3)
	EVT_MENU(Export_Astrolog_3, TransitsFrame::OnExportAstrolog_3)	

	EVT_MENU(RestrInputDlg_4,   TransitsFrame::OnRestrInputDlg_4)
	EVT_MENU(RestrInputDlg_5,   TransitsFrame::OnRestrInputDlg_5)
	EVT_MENU(Export_Astrolog_Dav,TransitsFrame::OnExportAstrolog_Dav)
	EVT_MENU(SaveChartAs_Dav,   TransitsFrame::OnSaveChartAs_Dav)

	EVT_MENU(Set_To_Now_Def,      TransitsFrame::OnSetToNow_Def)
	EVT_MENU(TDPInputDlg_Def,     TransitsFrame::OnTDPInputDlg_Def)
	EVT_MENU(RestrInputDlg_Def,   TransitsFrame::OnRestrInputDlg_Def)
	EVT_MENU(OpenChart_Def,       TransitsFrame::OnOpenChart_Def)
	EVT_MENU(SaveChartAs_Def,     TransitsFrame::OnSaveChartAs_Def)
	EVT_MENU(Import_Astrolog_Def, TransitsFrame::OnImportAstrolog_Def)
	EVT_MENU(Export_Astrolog_Def, TransitsFrame::OnExportAstrolog_Def)	

	EVT_MENU(Swap_1_2,          TransitsFrame::OnSwap_1_2)
	EVT_MENU(Swap_1_3,          TransitsFrame::OnSwap_1_3)
	EVT_MENU(Swap_2_3,          TransitsFrame::OnSwap_2_3)
	EVT_MENU(Swap_1_Def,        TransitsFrame::OnSwap_1_Def)

	EVT_MENU(Calc_Settings,     TransitsFrame::OnCalc_Settings)

	EVT_TIMER(Now_Timer,        TransitsFrame::OnNowTimer)

	EVT_MENU(TimeShiftForward,  TransitsFrame::OnTimeShiftForward)
	EVT_MENU(TimeShiftBackward, TransitsFrame::OnTimeShiftBackward)
	EVT_MENU(TimeGoForward,     TransitsFrame::OnTimeGoForward)
	EVT_MENU(TimeGoBackward,    TransitsFrame::OnTimeGoBackward)
	EVT_MENU(TimeStop,          TransitsFrame::OnTimeStop)
	EVT_MENU(TimeSettings,      TransitsFrame::OnTimeSettings)

	EVT_MENU(ViewNatal, 	    TransitsFrame::OnViewNatal)
	EVT_MENU(ViewComposite,     TransitsFrame::OnViewComposite)
	EVT_MENU(ViewDavison ,      TransitsFrame::OnViewDavison)
	EVT_MENU(ViewSynasty, 	    TransitsFrame::OnViewSynasty)
	EVT_MENU(ViewWithTransits,  TransitsFrame::OnViewWithTransits)
		                                     
	EVT_MENU(ViewAspects, 	    TransitsFrame::OnViewAspects)
	EVT_MENU(ViewAspConfig,     TransitsFrame::OnViewAspConfig)
	EVT_MENU(ViewDispositors,   TransitsFrame::OnViewDispositors)
	EVT_MENU(ViewDispositorsByHouse, TransitsFrame::OnViewDispositorsByHouse)

	EVT_MENU(AspectsViewAll,    TransitsFrame::OnAspectsViewAll)
	EVT_MENU(AspectsViewByOrb,  TransitsFrame::OnAspectsViewByOrb)
	EVT_MENU(AspectsViewByPlanet1, TransitsFrame::OnAspectsViewByPlanet1)
	EVT_MENU(AspectsViewByPlanet2, TransitsFrame::OnAspectsViewByPlanet2)
	EVT_MENU(AspectsSearch,     TransitsFrame::OnAspectsSearch)
	EVT_MENU(AspectRestrDlg,    TransitsFrame::OnAspectRestrDlg)
END_EVENT_TABLE()

// ----------------------------------------------------------------------------
// main frame
// ----------------------------------------------------------------------------

// frame constructor
TransitsFrame::TransitsFrame(const wxString& title, const wxPoint& pos,
							 const wxSize& size, const wxString& progdir)
       : wxFrame((wxFrame *)NULL, -1, title, pos, size)
{
	int ScreenWidth, ScreenHeight;
	wxDisplaySize(&ScreenWidth, &ScreenHeight);
	wxMenu *chartinfoSubMenu, *restrictionsSubMenu, *chartfornowSubMenu;
	wxMenu *savechartSubMenu, *loadchartSubMenu, *importalogSubMenu, *exportalogSubMenu;

	programdir = progdir;

	// create the dialogs
	wxString DTPDlgStr(_("Enter Chart Info"));
	wxString RestrDlgStr(_("Restrictions"));
	wxString AspRestrDlgStr(_("Aspect Settings"));
	wxString CalcSetDlgStr(_("Calculation Settings"));
	wxString FindAspectsStr(_("Find Aspects"));
	wxString TimeMotionStr(_("Time Motion Settings"));
	InputDateTimePlaceDlg =
		new CInputDateTimePlaceDlg(this, -1, DTPDlgStr, ScreenHeight < 550);
	InputRestrictionsDlg  =
		new CInputRestrictionsDlg (this, -1, RestrDlgStr, ScreenHeight < 550);
	InputAspectRestrictionsDlg =
		new CInputAspectRestrictionsDlg(this, -1, AspRestrDlgStr, ScreenHeight < 700);
	CalculationSettingsDlg =
		new CCalculationSettingsDlg(this, -1, CalcSetDlgStr, ScreenHeight < 550);
	FindAspectsDlg =
		new CFindAspectsDlg(this, -1, FindAspectsStr, ScreenHeight < 550);
	TimeMotionDlg =
		new CTimeMotionDlg(this, -1, TimeMotionStr, ScreenHeight < 550);


    // set the frame icon
    SetIcon(wxICON(Transits));

    // create menu bars
    fileMenu = new wxMenu("", wxMENU_TEAROFF);
	dataMenu = new wxMenu("", wxMENU_TEAROFF);
    viewMenu = new wxMenu("", wxMENU_TEAROFF);
	infoMenu = new wxMenu("", wxMENU_TEAROFF);
	timeMenu = new wxMenu("", wxMENU_TEAROFF);
    toolMenu = new wxMenu("", wxMENU_TEAROFF);
    helpMenu = new wxMenu("", wxMENU_TEAROFF);

	// create submenus
	chartinfoSubMenu    = new wxMenu();
	restrictionsSubMenu = new wxMenu();
	chartfornowSubMenu  = new wxMenu();
	loadchartSubMenu    = new wxMenu();
	savechartSubMenu    = new wxMenu();
	importalogSubMenu   = new wxMenu();
	exportalogSubMenu   = new wxMenu();

	// append all menu items

	loadchartSubMenu->Append(OpenChart_1, _("Native...\tCtrl-O"));
	loadchartSubMenu->Append(OpenChart_2, _("Partner...\tCtrl-Shift-O"));
	loadchartSubMenu->Append(OpenChart_3, _("Transit...\tAlt-Shift-O"));
	loadchartSubMenu->Append(OpenChart_Def, _("Default..."));
	fileMenu->Append(OpenSubMenuID, _("&Open Chart File"), loadchartSubMenu);

	savechartSubMenu->Append(SaveChartAs_1, _("Native...\tCtrl-S"));
	savechartSubMenu->Append(SaveChartAs_2, _("Partner...\tCtrl-Shift-S"));
	savechartSubMenu->Append(SaveChartAs_3, _("Transit...\tAlt-Shift-S"));
	savechartSubMenu->Append(SaveChartAs_Dav, _("Davison...\tCtrl-Alt-S"));
	savechartSubMenu->Append(SaveChartAs_Def, _("Default..."));
	fileMenu->Append(SaveAsSubMenuID, _("&Save Chart File As"), savechartSubMenu);

	fileMenu->AppendSeparator();

	importalogSubMenu->Append(Import_Astrolog_1, _("Native...\tAlt-O"));
	importalogSubMenu->Append(Import_Astrolog_2, _("Partner..."));
	importalogSubMenu->Append(Import_Astrolog_3, _("Transit..."));
	importalogSubMenu->Append(Import_Astrolog_Def, _("Default..."));
	fileMenu->Append(ImportALogSubMenuID, _("I&mport Astrolog"), importalogSubMenu);

	exportalogSubMenu->Append(Export_Astrolog_1, _("Native...\tAlt-W"));
	exportalogSubMenu->Append(Export_Astrolog_2, _("Partner..."));
	exportalogSubMenu->Append(Export_Astrolog_3, _("Transit..."));
	exportalogSubMenu->Append(Export_Astrolog_Dav, _("Davison..."));
	exportalogSubMenu->Append(Export_Astrolog_Def, _("Default..."));
	fileMenu->Append(ExportALogSubMenuID, _("Export &Astrolog"), exportalogSubMenu);

	fileMenu->AppendSeparator();

	fileMenu->Append(ImportConfiguration, _("&Import Configuration File..."));
	fileMenu->Append(ExportConfiguration, _("&Export Current Configuration..."));
	fileMenu->Append(RestoreDefaultConfiguration,
		_("Restore Default Program Configuration (Natal Chart)"));
	fileMenu->Append(RestoreDefaultConfigurationAll,
		_("Restore Default Program Configuration (All Charts)"));
	fileMenu->Append(SaveConfigurationAsDefault,
		_("Save Current Program Configuration as Default"));

	fileMenu->AppendSeparator();

	fileMenu->Append(Quit_App, _("E&xit\tCtrl-Q"));

	chartinfoSubMenu->Append(TDPInputDlg_1, _("Native...\tAlt-Z"));
	chartinfoSubMenu->Append(TDPInputDlg_2, _("Partner...\tAlt-Shift-Z"));
	chartinfoSubMenu->Append(TDPInputDlg_3, _("Transit...\tCtrl-Shift-Z"));
	chartinfoSubMenu->Append(TDPInputDlg_Def, _("Default...\tCtrl-Alt-Z"));
	dataMenu->Append(ChartInfoSubMenuID, _("Chart I&nfo"), chartinfoSubMenu);

	restrictionsSubMenu->Append(RestrInputDlg_1, _("Native...\tAlt-R"));
	restrictionsSubMenu->Append(RestrInputDlg_2, _("Partner...\tAlt-Shift-R"));
	restrictionsSubMenu->Append(RestrInputDlg_3, _("Transit...\tCtrl-Shift-R"));
	restrictionsSubMenu->Append(RestrInputDlg_5, _("Composite...\tCtrl-Shift-C"));
	restrictionsSubMenu->Append(RestrInputDlg_4, _("Davison...\tCtrl-Shift-D"));
	/*restrictionsSubMenu->Append(RestrInputDlg_Def, _("Default...\tCtrl-Alt-R"));*/
	dataMenu->Append(RestrictionsSubMenuID, _("&Restrictions"), restrictionsSubMenu);

	chartfornowSubMenu->Append(Set_To_Now_1, _("Native\tAlt-N"));
	chartfornowSubMenu->Append(Set_To_Now_2, _("Partner\tAlt-Shift-N"));
	chartfornowSubMenu->Append(Set_To_Now_3, _("Transit\tCtrl-Shift-N"));
	chartfornowSubMenu->Append(Set_To_Now_Def, _("Default\tCtrl-Alt-N"));
	dataMenu->Append(ChartForNowSubMenuID, _("Chart for &Now"), chartfornowSubMenu);

	dataMenu->AppendSeparator();

	dataMenu->Append(Swap_1_2, _("&Swap Natal and Partner Data\tCtrl-Alt-1"));
	dataMenu->Append(Swap_1_3, _("Swap Natal and &Transit Data\tCtrl-Alt-2"));
	dataMenu->Append(Swap_2_3, _("Swap &Partner and Transit Data\tCtrl-Alt-3"));
	dataMenu->Append(Swap_1_Def, _("Swap Natal and De&fault Data\tCtrl-Alt-4"));
	dataMenu->AppendSeparator();

	dataMenu->Append(Calc_Settings, _("&Calculation Settings...\tF10"));

#ifdef USE_RADIO_MENU_ITEMS
	viewMenu->AppendRadioItem(ViewNatal, _("&Natal Chart\tCtrl-V"));
	viewMenu->AppendRadioItem(ViewComposite, _("&Composite Chart\tCtrl-C"));
	viewMenu->AppendRadioItem(ViewDavison, _("&Davison Relationship Chart\tCtrl-D"));
	viewMenu->AppendRadioItem(ViewSynasty, _("&Synastry Chart\tCtrl-Y"));
	viewMenu->AppendRadioItem(ViewWithTransits, _("Natal Chart With &Transits\tCtrl-T"));
	viewMenu->AppendSeparator();
	viewMenu->AppendRadioItem(ViewAspects, _("&Aspects\tAlt-Shift-A"));
	viewMenu->AppendRadioItem(ViewAspConfig, _("Aspect Con&figurations\tAlt-Shift-F"));
	viewMenu->AppendRadioItem(ViewDispositors, _("&Dispositors (By Sign)\tAlt-Shift-L"));
	viewMenu->AppendRadioItem(ViewDispositorsByHouse, _("&Dispositors (By House)\tCtrl-Alt-L"));
#else
	viewMenu->AppendCheckItem(ViewNatal, _("&Natal Chart\tCtrl-V"));
	viewMenu->AppendCheckItem(ViewComposite, _("&Composite Chart\tCtrl-C"));
	viewMenu->AppendCheckItem(ViewDavison, _("&Davison Relationship Chart\tCtrl-D"));
	viewMenu->AppendCheckItem(ViewSynasty, _("&Synastry Chart\tCtrl-Y"));
	viewMenu->AppendCheckItem(ViewWithTransits, _("Natal Chart With &Transits\tCtrl-T"));
	viewMenu->AppendSeparator();
	viewMenu->AppendCheckItem(ViewAspects, _("&Aspects\tAlt-Shift-A"));
	viewMenu->AppendCheckItem(ViewAspConfig, _("Aspect Con&figurations\tAlt-Shift-F"));
	viewMenu->AppendCheckItem(ViewDispositors, _("&Dispositors (By Sign)\tAlt-Shift-L"));
	viewMenu->AppendCheckItem(ViewDispositorsByHouse, _("&Dispositors (By House)\tCtrl-Alt-L"));
	viewMenu->Check(ViewNatal, TRUE);
	viewMenu->Check(ViewAspects, TRUE);
#endif

	/* aspeMenu->Append(AspectsViewAll, "View &All Aspects"); */
	infoMenu->Append(AspectsViewByOrb, _("View All Aspects by &Orb\tCtrl-A"));
	infoMenu->Append(AspectsViewByPlanet1, _("View All Aspects by &First Planet\tCtrl-Shift-A"));
	infoMenu->Append(AspectsViewByPlanet2, _("View All Aspects by &Second Planet\tCtrl-Alt-A"));
	infoMenu->AppendSeparator();
	infoMenu->Append(AspectsSearch, _("&Find Aspects...\tCtrl-F"));
	infoMenu->AppendSeparator();
	infoMenu->Append(AspectRestrDlg,  _("Aspect &Settings...\tF9"));

	timeMenu->Append(TimeShiftForward, _("Shift &Forward Once\tShift-Right"));
	timeMenu->Append(TimeShiftBackward, _("Shift &Backward Once\tShift-Left"));
	timeMenu->Append(TimeGoForward, _("&Go Forward\tCtrl-Right"));
	timeMenu->Append(TimeGoBackward, _("Go Bac&kward\tCtrl-Left"));
	timeMenu->Append(TimeStop, _("&Stop Time Motion\tCtrl-Down"));
	timeMenu->Append(TimeSettings, _("Time Motion Se&ttings...\tCtrl-Up"));

    helpMenu->Append(About_App, _("&About..."), _("Show about dialog"));

    // now append the freshly created menu to the menu bar...
    wxMenuBar *menuBar = new wxMenuBar();
    menuBar->Append(fileMenu, _("&File"));
	menuBar->Append(dataMenu, _("&Data"));
	menuBar->Append(viewMenu, _("&View"));
	menuBar->Append(infoMenu, _("&Aspects"));
	menuBar->Append(timeMenu, _("&Time"));
	/* menuBar->Append(toolMenu, _("T&ools")); */
	menuBar->Append(helpMenu, _("&Help"));

    // ... and attach this menu bar to the frame
    SetMenuBar(menuBar);

	CreateStatusBar();
	
	// create the controls
	topsizer = new wxBoxSizer(wxVERTICAL);

	wxBoxSizer *mainsizer = new wxBoxSizer(wxHORIZONTAL);

	ChartCanvas = new CChartCanvas(this);
    mainsizer->Add( ChartCanvas, 1, wxEXPAND, 0 );
	/* ChartCanvas->SetScrollbars(1, 1, 1024, 768); */

	ChartInfo = new CChartInfo(this, ChartCanvas);
	mainsizer->Add( ChartInfo, 0, wxEXPAND, 0 );

	topsizer->Add(mainsizer, 1, wxEXPAND);

    SetAutoLayout(TRUE);
    SetSizer(topsizer);

	is_now = true;
	nowtimer = new wxTimer;
	nowtimer->SetOwner(this, Now_Timer);
	//timer->Start(TIMER_INTERVAL, true);

#if wxUSE_DRAG_AND_DROP
	ChartCanvas->SetDropTarget(new CDnDFile(this));
	ChartInfo->SetDropTarget(new CDnDFile(this));
#endif

	ChartCanvas->SetInfoPane(ChartInfo);

	InitVariables();

	ChartCanvas->SetRestrictions(restricted, r_aspects, aspect_orb, FALSE);
	ChartCanvas->SetChartData(chart, chartdata, chartdataext, FALSE);
	ChartCanvas->SetMode(NATAL_CHART | ASPECTS_CHART, chart0, chart1, chart2, TRUE, FALSE);

}

// frame destructor
TransitsFrame::~TransitsFrame()
{
	delete nowtimer;
}

int TransitsFrame::GetMode()
{
	return mode;
}

void TransitsFrame::Recalculate()
{
	bool r_changed;
	int natch1, natch2; // short for "natal chart"; may or may not be progressed
	int i, j;

	r_changed = FALSE;

	// Take progressions into account
	progress_tdp(&(chartdata[CHART_NATAL1]), &(chartdata[CHART_TRANSITS]),
		&(chartdata[CHART_PROGRESSED1]));
	progress_tdp(&(chartdata[CHART_NATAL2]), &(chartdata[CHART_TRANSITS]),
		&(chartdata[CHART_PROGRESSED2]));

	natch1 = chartdata[CHART_NATAL1].is_progressed ? CHART_PROGRESSED1 : CHART_NATAL1;
	natch2 = chartdata[CHART_NATAL2].is_progressed ? CHART_PROGRESSED2 : CHART_NATAL2;
	// Get the position and time for the Davison and Composite charts
	midpoint_tdp(&(chartdata[natch1]), &(chartdata[natch2]),
		&(chartdata[CHART_DAVISON]));
	midpoint_tdp(&(chartdata[natch1]), &(chartdata[natch2]),
		&(chartdata[CHART_COMPOSITE]));

	// Calculate all the charts
	for(i = 0; i < NCHARTTYPES; i++)
	{
		// Note that a Davison chart is computed exactly like a natal chart
		if((i != CHART_PROGRESSED1) && (i != CHART_PROGRESSED2) && (i != CHART_COMPOSITE))
			astro_calc(&(chartdata[i]), hsys, &(chart[i]));
		else if(i == CHART_PROGRESSED1)
			astro_calc_progressed(&(chartdata[CHART_NATAL1]), &(chartdata[CHART_TRANSITS]),
			           &(chartdata[i]), hsys, &(chart[i]));
		else if(i == CHART_PROGRESSED2)
			astro_calc_progressed(&(chartdata[CHART_NATAL2]), &(chartdata[CHART_TRANSITS]),
			           &(chartdata[i]), hsys, &(chart[i]));
		else if(i == CHART_COMPOSITE)
			astro_calc_composite(&(chartdata[natch1]), &(chartdata[natch2]),
			           chart[natch1].armc, chart[natch2].armc, hsys, &(chart[i]));
	}

	// Update the labels
	chartdataext[CHART_COMPOSITE].namestr = "";
	chartdataext[CHART_COMPOSITE].citystr = "";
	chartdataext[CHART_DAVISON].namestr = "";
	chartdataext[CHART_DAVISON].citystr = "";
	if(chartdataext[CHART_NATAL1].namestr.length() > 0)
		chartdataext[CHART_PROGRESSED1].namestr = chartdataext[CHART_NATAL1].namestr + " (Prg)";
	else
		chartdataext[CHART_PROGRESSED1].namestr = "";
	chartdataext[CHART_PROGRESSED1].citystr = chartdataext[CHART_NATAL1].citystr;
	if(chartdataext[CHART_NATAL2].namestr.length() > 0)
		chartdataext[CHART_PROGRESSED2].namestr = chartdataext[CHART_NATAL2].namestr + " (Prg)";
	else
		chartdataext[CHART_PROGRESSED2].namestr = "";
	chartdataext[CHART_PROGRESSED2].citystr = chartdataext[CHART_NATAL2].citystr;

	// If anything can't be calculated (and 0 Ari 0'0" is assumed to
	// be an error, prevent it from showing)
	for(j = 0; j < NCHARTTYPES; j++)
	{
		for(i = 0; i < NTOTOBJECTS; i++)
			cantcalculate[j][i] = (chart[j].planet[i].longitude < 0.000001);
		for(i = NTOTOBJECTS; i < (NTOTOBJECTS + 12); i++)
			cantcalculate[j][i] = FALSE;
		for(i = 0; i < NTOTOBJECTS; i++)
			if(cantcalculate[j][i] && !restricted[j][i])
			{
				restricted[j][i] = TRUE;
				r_changed = TRUE;
			}
	}

	if(r_changed)
		ChartCanvas->SetRestrictions(restricted, r_aspects, aspect_orb, FALSE);
}

void TransitsFrame::SetChartsToView(int nmode)
{
	if(nmode & NATAL_CHART)
	{
		// A natal chart is the simplest case
		chart0 = chartdata[CHART_NATAL1].is_progressed ? CHART_PROGRESSED1 : CHART_NATAL1;
		chart1 = chart2 = 0;
	}
	else if(IS_ONE_CHART(nmode))
	{
		// Other kinds of single charts need their component charts also passed in
		if(nmode & COMPOSITE_CHART)
			chart0 = CHART_COMPOSITE;
		else if(nmode & DAVISON_CHART)
			chart0 = CHART_DAVISON;
		else
			chart0 = CHART_NATAL1; // ERROR!
		chart1 = chartdata[CHART_NATAL1].is_progressed ? CHART_PROGRESSED1 : CHART_NATAL1;
		chart2 = chartdata[CHART_NATAL2].is_progressed ? CHART_PROGRESSED2 : CHART_NATAL2;
	}
	else
	{
		// Two charts, aka synasty; simply pass in the charts shown
		chart0 = 0;
		chart1 = chartdata[CHART_NATAL1].is_progressed ? CHART_PROGRESSED1 : CHART_NATAL1;
		if(nmode & TRANSITS_CHART)
			chart2 = CHART_TRANSITS;
		else
			chart2 = chartdata[CHART_NATAL2].is_progressed ? CHART_PROGRESSED2 : CHART_NATAL2;
	}
}

wxColour TransitsFrame::GetSysColour(wxSystemColour index)
{
	return wxSystemSettings::GetColour(index);
}

void TransitsFrame::InitVariables()
{
	wxDateTime now, tmpdt;
	wxTimeSpan tspan, tspan_nodst;
	int i, j, k;

	hsys = PLACIDUS;
	nodemode = get_node_type();
	apogmode = get_apog_type();

	mode = NATAL_CHART;
	viewmode = ASPECTS_CHART;
	chart0 = CHART_NATAL1;
	chart1 = CHART_NATAL1;
	chart2 = CHART_NATAL2;
	nowchart = CHART_TRANSITS;

	srand(time(NULL));
	for(i = 0; i < NCHARTTYPES; i++)
	{
		now = wxDateTime::Now();
		tmpdt = now.ToGMT();
		tspan = tmpdt.Subtract(now);
		tspan_nodst = tspan;
		if(now.IsDST()) tspan_nodst -= wxTimeSpan(1);
		chartdata[i].date.day      = now.GetDay();
		chartdata[i].date.month    = (now.GetMonth() - wxDateTime::Jan) + 1;
		chartdata[i].date.year     = now.GetYear();
		chartdata[i].time.hours    = now.GetHour();
		chartdata[i].time.min      = now.GetMinute();
		chartdata[i].time.sec      = now.GetSecond();
		chartdata[i].timezone      = tspan_nodst.GetHours();
		chartdata[i].is_dst        = (now.IsDST() > 0) ? TRUE : FALSE;
		chartdataext[i].set_to_now = true;
		// Do a rough estimate of lat. and long. based on time zone
		switch(chartdata[i].timezone)
		{
		case 0:
			// When in GMT, go to London by default!
			chartdata[i].latitude.deg  = 51;
			chartdata[i].latitude.min  = 40;
			chartdata[i].longitude.deg = 0;
			chartdata[i].longitude.min = 0;
			chartdataext[i].citystr = "London, England";
			break;
		case 1:
			// For Western European time, use Geneva, Switzerland
			chartdata[i].latitude.deg  = 46;
			chartdata[i].latitude.min  = 12;
			chartdata[i].longitude.deg = 6;
			chartdata[i].longitude.min = 9;
			chartdataext[i].citystr = "Geneva, Switzerland";
			break;
		case 2:
			// In Central European Time, Jerusalem
			chartdata[i].latitude.deg  = 31;
			chartdata[i].latitude.min  = 46;
			chartdata[i].longitude.deg = 35;
			chartdata[i].longitude.min = 14;
			chartdataext[i].citystr = "Jerusalem";
			break;
		case 3:
			// Moscow
			chartdata[i].latitude.deg  = 55;
			chartdata[i].latitude.min  = 45;
			chartdata[i].longitude.deg = 37;
			chartdata[i].longitude.min = 37;
			chartdataext[i].citystr = "Moscow, Russia";
			break;
		case 8:
			// Hong Kong
			chartdata[i].latitude.deg  = 22;
			chartdata[i].latitude.min  = 17;
			chartdata[i].longitude.deg = 114;
			chartdata[i].longitude.min = 9;
			chartdataext[i].citystr = "Hong Kong";
			break;
		case 9:
			// Tokyo, Japan
			chartdata[i].latitude.deg  = 35;
			chartdata[i].latitude.min  = 42;
			chartdata[i].longitude.deg = 139;
			chartdata[i].longitude.min = 46;
			chartdata[i].is_dst = false;
			chartdataext[i].citystr = "Tokyo, Japan";
			break;
		case 10:
			// Sydney, Australia
			chartdata[i].latitude.deg  = -33;
			chartdata[i].latitude.min  = 52;
			chartdata[i].longitude.deg = 151;
			chartdata[i].longitude.min = 13;
			chartdataext[i].citystr = "Sydney, Australia";
			break;
		case 12:
			// Auckland, New Zealand
			chartdata[i].latitude.deg  = -36;
			chartdata[i].latitude.min  = 52;
			chartdata[i].longitude.deg = 174;
			chartdata[i].longitude.min = 46;
			chartdataext[i].citystr = "Auckland, New Zealand";
			break;
		case -3:
			// Sao Paulo, Brazil
			chartdata[i].latitude.deg  = -23;
			chartdata[i].latitude.min  = 32;
			chartdata[i].longitude.deg = -46;
			chartdata[i].longitude.min = 38;
			chartdataext[i].citystr = "Sao Paulo, Brazil";
			break;
		case -4:
			// In Canadian Atlantic Time, use Halifax as the default
			chartdata[i].latitude.deg  = 44;
			chartdata[i].latitude.min  = 39;
			chartdata[i].longitude.deg = -63;
			chartdata[i].longitude.min = 36;
			chartdataext[i].citystr = "Halifax, Nova Scotia";
			break;
		case -5:
			// In US/Canadian Eastern Time, use New York as the default
			chartdata[i].latitude.deg  = 40;
			chartdata[i].latitude.min  = 43;
			chartdata[i].longitude.deg = -74;
			chartdata[i].longitude.min = 0;
			chartdataext[i].citystr = "New York, NY";
			break;
		case -6:
			// In Central Time, use Chicago as the default
			chartdata[i].latitude.deg  = 41;
			chartdata[i].latitude.min  = 51;
			chartdata[i].longitude.deg = -87;
			chartdata[i].longitude.min = 39;
			chartdataext[i].citystr = "Chicago, IL";
			break;
		case -7:
			// In Mountain Time, use Denver as the default
			chartdata[i].latitude.deg  = 39;
			chartdata[i].latitude.min  = 44;
			chartdata[i].longitude.deg = -104;
			chartdata[i].longitude.min = 59;
			chartdataext[i].citystr = "Denver, CO";
			break;
		case -8:
			// In Pacific Time, use Los Angeles as the default
			chartdata[i].latitude.deg  = 34;
			chartdata[i].latitude.min  = 3;
			chartdata[i].longitude.deg = -118;
			chartdata[i].longitude.min = 15;
			chartdataext[i].citystr = "Los Angeles, CA";
			break;
		case -9:
			// In Alaskan Time, use Anchorage as the default
			chartdata[i].latitude.deg  = 61;
			chartdata[i].latitude.min  = 13;
			chartdata[i].longitude.deg = -149;
			chartdata[i].longitude.min = 54;
			chartdataext[i].citystr = "Anchorage, AK";
			break;
		case -10:
			// In Hawaiian Time, use Honolulu as the default
			chartdata[i].latitude.deg  = 21;
			chartdata[i].latitude.min  = 18;
			chartdata[i].longitude.deg = -157;
			chartdata[i].longitude.min = 52;
			chartdata[i].is_dst = false;
			chartdataext[i].citystr = "Honolulu, HI";
			break;
		default:
			// By default, use 30' North, for no good reason except that it's round
			chartdata[i].latitude.deg  = 30;
			chartdata[i].latitude.min  = 0;
			chartdata[i].longitude.deg = 180*tspan_nodst.GetHours()/12;
			chartdata[i].longitude.min = 0;
			chartdataext[i].citystr = "";
		}
		chartdataext[i].namestr = "";
		chartdata[i].latitude.sec  = 0;
		chartdata[i].longitude.sec = 0;
		chartdata[i].is_progressed = 0;
	}
	// For the Default chart, is_progressed determines whether it will
	// automatically be set to "Now" on startup
	chartdata[CHART_DEFAULT].is_progressed = 1;

	ChartInfo->SetChartSel(CHART_NATAL1, CHART_NATAL1, CHART_NATAL1);

	for(i = 0; i < NCHARTTYPES; i++)
	{
		for(j = 0; j < (NTOTOBJECTS + 12); j++)
		{
			restricted[i][j] = !(IS_PLANET(FIRST_OBJECT + j) ||
				(FIRST_OBJECT + j == NNODE) || (FIRST_OBJECT + j == CHIRON));
		}
	}

	for(k = 0; k < NASPECTS; k++)
	{
		aspect_orb[k] = AspectOrbDefaults(FIRST_ASPECT + k);
		r_aspects[k] = ((FIRST_ASPECT + k) > SEXTILE);
	}

	// Create the preferences directory if it doesn't exist
	if(!wxDirExists(trPrefsDir(programdir)))
	{
		if(!wxMkdir(trPrefsDir(programdir)))
		{
			wxMessageBox(_("Cannot create directory ") + trPrefsDir(programdir) +
				_(", thus cannot use a preferences file."),
				"Error!", wxOK | wxCENTRE | wxICON_ERROR, this);
			Recalculate();
			return;
		}
	}

	// If a configuration file exists, read in the default values from it
	if(wxFileExists(trPrefsFile(programdir)))
	{
		for(i = 0; i < NCHARTTYPES; i++)
			FileUtils.GetConfigFile(this, i, trPrefsFile(programdir),
				trCONFIG_ALL);
	}
	// If not, write the preferences set above into a new configuration file
	else
	{
		FileUtils.PutConfigFile(this, CHART_NATAL1, trPrefsFile(programdir),
			trCONFIG_ALL, false);
	}

	// Now read in the default chart if it exists, and create it otherwise
	if(wxFileExists(trPrefsDefChart(programdir)))
	{
		for(i = 0; i < NCHARTTYPES; i++)
			FileUtils.GetChartDataFile(this, i, trPrefsDefChart(programdir));
	}
	else
	{
		FileUtils.PutChartDataFile(this, CHART_NATAL1, trPrefsDefChart(programdir));
	}

	Recalculate();
}

// event handlers
void TransitsFrame::OnSetToNow_1(wxCommandEvent &event)
{
	nowchart = 0;
	SetToNow(nowchart, TRUE);
}

void TransitsFrame::OnSetToNow_2(wxCommandEvent &event)
{
	nowchart = 1;
	SetToNow(nowchart, TRUE);
}

void TransitsFrame::OnSetToNow_3(wxCommandEvent &event)
{
	nowchart = 2;
	SetToNow(nowchart, TRUE);
}

void TransitsFrame::OnSetToNow_Def(wxCommandEvent &event)
{
	nowchart = CHART_DEFAULT;
	SetToNow(nowchart, TRUE);
}




void TransitsFrame::OnImportConfig(wxCommandEvent &event)
{
	wxString filename;

	filename = wxFileSelector("Import Configuration File", "", "", "",
		CONFIGFILEFILTER, wxOPEN | wxFILE_MUST_EXIST | wxHIDE_READONLY);

	if( !filename.empty() )
		FileUtils.GetConfigFile(this, CHART_NATAL1, filename, trCONFIG_ALL);
}

void TransitsFrame::OnExportConfig(wxCommandEvent &event)
{
	wxString filename;

	filename = wxFileSelector("Save Current Configuration Into File", "", "", "",
		CONFIGFILEFILTER, wxSAVE | wxOVERWRITE_PROMPT);

	if( !filename.empty() )
		FileUtils.PutConfigFile(this, CHART_NATAL1, filename, trCONFIG_ALL, true);
}

void TransitsFrame::OnRestoreDefaultConfigAll(wxCommandEvent &event)
{
	int i;
	for(i = 0; i < NCHARTTYPES; i++)
		FileUtils.GetConfigFile(this, i, trPrefsFile(programdir),
			trCONFIG_ALL);
}

void TransitsFrame::OnRestoreDefaultConfig(wxCommandEvent &event)
{
	FileUtils.GetConfigFile(this, CHART_NATAL1, trPrefsFile(programdir),
		trCONFIG_ALL);
}

void TransitsFrame::OnSaveDefaultConfig(wxCommandEvent &event)
{
	FileUtils.PutConfigFile(this, CHART_NATAL1, trPrefsFile(programdir),
		trCONFIG_ALL, true);
}

void TransitsFrame::ViewMenuCheckHelper(int itemid)
{
#ifndef USE_RADIO_MENU_ITEMS
	if((itemid == ViewNatal) || (itemid == ViewSynasty) || (itemid == ViewDavison) ||
		(itemid == ViewComposite) || (itemid == ViewWithTransits))
	{
		viewMenu->Check(ViewNatal, itemid == ViewNatal);
		viewMenu->Check(ViewComposite, itemid == ViewComposite);
		viewMenu->Check(ViewDavison, itemid == ViewDavison);
		viewMenu->Check(ViewSynasty, itemid == ViewSynasty);
		viewMenu->Check(ViewWithTransits, itemid == ViewWithTransits);
	}
	else if((itemid == ViewAspects) || (itemid == ViewAspConfig) ||
		(itemid == ViewDispositors) || (itemid == ViewDispositorsByHouse))
	{
		viewMenu->Check(ViewAspects, itemid == ViewAspects);
		viewMenu->Check(ViewAspConfig, itemid == ViewAspConfig);
		viewMenu->Check(ViewDispositors, itemid == ViewDispositors);
		viewMenu->Check(ViewDispositorsByHouse, itemid == ViewDispositorsByHouse);
	}
#endif
}

void TransitsFrame::OnViewNatal(wxCommandEvent &event)
{
	ViewMenuCheckHelper(event.GetId());
	viewMenu->Enable(ViewAspConfig, TRUE);
	viewMenu->Enable(ViewDispositors, TRUE);
	viewMenu->Enable(ViewDispositorsByHouse, TRUE);

	mode = NATAL_CHART;
	SetChartsToView(mode);
	ChartCanvas->SetMode(mode | viewmode, chart0, chart0, chart0);
}

void TransitsFrame::OnViewComposite(wxCommandEvent &event)
{
	ViewMenuCheckHelper(event.GetId());
	viewMenu->Enable(ViewAspConfig, TRUE);
	viewMenu->Enable(ViewDispositors, TRUE);
	viewMenu->Enable(ViewDispositorsByHouse, TRUE);

	mode = COMPOSITE_CHART;
	SetChartsToView(mode);
	ChartCanvas->SetMode(mode | viewmode, chart0, chart1, chart2);
}

void TransitsFrame::OnViewDavison(wxCommandEvent &event)
{
	ViewMenuCheckHelper(event.GetId());
	viewMenu->Enable(ViewAspConfig, TRUE);
	viewMenu->Enable(ViewDispositors, TRUE);
	viewMenu->Enable(ViewDispositorsByHouse, TRUE);

	mode = DAVISON_CHART;
	SetChartsToView(mode);
	ChartCanvas->SetMode(mode | viewmode, chart0, chart1, chart2);
}

void TransitsFrame::OnViewSynasty(wxCommandEvent &event)
{
	ViewMenuCheckHelper(event.GetId());
	viewMenu->Enable(ViewAspConfig, FALSE);
	viewMenu->Enable(ViewDispositors, FALSE);
	viewMenu->Enable(ViewDispositorsByHouse, FALSE);
	if(!viewMenu->IsChecked(ViewAspects))
	{
		viewMenu->Check(ViewAspects, TRUE);
		ViewMenuCheckHelper(ViewAspects);
	}

	mode = SYNASTY_CHART;
	viewmode = ASPECTS_CHART;
	SetChartsToView(mode);
	ChartCanvas->SetMode(mode | viewmode, chart0, chart1, chart2);
}

void TransitsFrame::OnViewWithTransits(wxCommandEvent &event)
{
	ViewMenuCheckHelper(event.GetId());
	viewMenu->Enable(ViewAspConfig, FALSE);
	viewMenu->Enable(ViewDispositors, FALSE);
	viewMenu->Enable(ViewDispositorsByHouse, FALSE);
	if(!viewMenu->IsChecked(ViewAspects))
	{
		viewMenu->Check(ViewAspects, TRUE);
		ViewMenuCheckHelper(ViewAspects);
	}

	mode = TRANSITS_CHART;
	viewmode = ASPECTS_CHART;
	SetChartsToView(mode);
	ChartCanvas->SetMode(mode | viewmode, chart0, chart1, chart2);
}

void TransitsFrame::OnViewAspects(wxCommandEvent &event)
{
	ViewMenuCheckHelper(event.GetId());

	viewmode = ASPECTS_CHART;
	ChartCanvas->SetMode(mode | viewmode, chart0, chart1, chart2);
}

void TransitsFrame::OnViewAspConfig(wxCommandEvent &event)
{
	ViewMenuCheckHelper(event.GetId());

	viewmode = ASPCONFIG_CHART;
	ChartCanvas->SetMode(mode | viewmode, chart0, chart1, chart2);
}

void TransitsFrame::OnViewDispositors(wxCommandEvent &event)
{
	ViewMenuCheckHelper(event.GetId());

	viewmode = DISPOSITORS_CHART;
	ChartCanvas->SetMode(mode | viewmode, chart0, chart1, chart2);
}

void TransitsFrame::OnViewDispositorsByHouse(wxCommandEvent &event)
{
	ViewMenuCheckHelper(event.GetId());

	viewmode = DISPOSITORS_HOUSE_CHART;
	ChartCanvas->SetMode(mode | viewmode, chart0, chart1, chart2);
}




void TransitsFrame::SetToNow(int nchart, bool do_refresh, bool do_write)
{
	wxDateTime now = wxDateTime::Now();
	wxDateTime tmpdt = now.ToGMT();
	wxTimeSpan tspan = tmpdt.Subtract(now);
	wxTimeSpan tspan_nodst = tspan;
	int curtz;

	if((nchart == CHART_DEFAULT) && do_write &&
			(wxMessageBox(_("This will overwrite the current default chart that shows \
when the application opens.  Are you sure you want to continue?"),
			"Warning", wxYES_NO | wxICON_WARNING) == wxNO))
	{
		return;
	}

	if(now.IsDST()) tspan_nodst -= wxTimeSpan(1);
	curtz = tspan_nodst.GetHours();
	now += wxTimeSpan(chartdata[nchart].timezone - curtz);

	chartdata[nchart].date.day      = now.GetDay();
	chartdata[nchart].date.month    = (now.GetMonth() - wxDateTime::Jan) + 1;
	chartdata[nchart].date.year     = now.GetYear();
	chartdata[nchart].time.hours    = now.GetHour();
	chartdata[nchart].time.min      = now.GetMinute();
	chartdata[nchart].time.sec      = now.GetSecond();
	chartdata[nchart].is_dst        = (now.IsDST() > 0) ? 1 : 0;
	chartdata[nchart].is_progressed = (nchart == CHART_DEFAULT) ? TRUE : FALSE;
	chartdataext[nchart].namestr    = "";
	chartdataext[nchart].set_to_now = true;

	if(do_refresh)
	{
		Recalculate();

		ChartCanvas->SetInfoPane(ChartInfo);
		ChartCanvas->SetChartData(chart, chartdata, chartdataext);
	}

	if((nchart == CHART_DEFAULT) && do_write)
		FileUtils.PutChartDataFile(this, CHART_DEFAULT,
			trPrefsDefChart(programdir));
}

void TransitsFrame::OnTimer(wxTimerEvent &event)
{
}

void TransitsFrame::OnNowTimer(wxTimerEvent &event)
{
	SetToNow(nowchart, TRUE);
}

void TransitsFrame::OnTimeShiftBackward(wxCommandEvent &event)
{
	Animator.ShiftBackward(this, TimeMotionDlg);
}

void TransitsFrame::OnTimeShiftForward(wxCommandEvent &event)
{
	Animator.ShiftForward(this, TimeMotionDlg);
}

void TransitsFrame::OnTimeGoBackward(wxCommandEvent &event)
{
	Animator.GoBackward(this, TimeMotionDlg);
}

void TransitsFrame::OnTimeGoForward(wxCommandEvent &event)
{
	Animator.GoForward(this, TimeMotionDlg);
}

void TransitsFrame::OnTimeStop(wxCommandEvent &event)
{
	Animator.Stop();
}

void TransitsFrame::OnTimeSettings(wxCommandEvent &event)
{
	TimeMotionDlg->notebook->SetSelection(0);
	if(TimeMotionDlg->ShowModal() == wxID_OK)
	{
		TimeMotionDlg->UpdateSpans(
			TimeMotionDlg->step_method->GetSelection() == 0);
		if(Animator.IsRunning())
			Animator.Restart();
	}
	SetFocus(); // Oddly enough, the main frame doesn't get back the
	            // focus when the dialog is closed otherwise.
}

void TransitsFrame::OnClose(wxCloseEvent &event)
{
	Animator.Stop();
	event.Skip();
}

void TransitsFrame::OnQuit(wxCommandEvent& event)
{
    // TRUE is to force the frame to close
    Close(TRUE);
}

void TransitsFrame::OnAbout(wxCommandEvent& event)
{
    wxString msg;
	msg = "Transitarium Alpha 6\n"
		"Note that this is an Alpha version;\n"
		"you are encouraged to stay updated\n"
		"on new versions at least until out\n"
		"of Alpha testing and officially released\n\n"
		"Copyright (C) 2003-2005 mikesag\n\n"
		"Thanks to Walter Pullen for the glyphs,\n"
		"Chris Mitchell for early testing, Missy\n"
		"for all her support and encouragement,\n"
		"and Node-o-Rama for fascinating\n"
		"discussions about astrology.\n\n"
		"Uses WXVER\n"
		"and v1.66 of the Swiss Ephemeris\n\n"
		"Licensed under the GNU GPL, version 2\n"
		"NO WARRENTIES, EXPRESSED OR IMPLIED";
	msg.Replace("WXVER", wxVERSION_STRING, FALSE);
    wxMessageBox(msg, _("About Transitarium"), wxOK | wxICON_INFORMATION, this);
}

void TransitsFrame::GetTDPInput(int nchart, const wxString &title)
{
	long n;
	wxString str, tok, tmpstr;
	wxStringTokenizer tokstime;
	wxDateTime wxdt(
		chartdata[nchart].date.day,
		wxDateTime::Month(wxDateTime::Jan + chartdata[nchart].date.month - 1),
		chartdata[nchart].date.year, chartdata[nchart].time.hours,
		chartdata[nchart].time.min, (int)chartdata[nchart].time.sec);
	str = trDateToString(&(chartdata[nchart].date)); // to stop compiler complaining
	InputDateTimePlaceDlg->cal->SetDate(wxdt);
	InputDateTimePlaceDlg->nametext->SetValue(chartdataext[nchart].namestr);
	InputDateTimePlaceDlg->citytext->SetValue(chartdataext[nchart].citystr);
	InputDateTimePlaceDlg->timetext->SetValue(HMStoString(&(chartdata[nchart].time)));
	InputDateTimePlaceDlg->lattext->SetValue(
		DMStoString(&(chartdata[nchart].latitude), "N", "S"));
	InputDateTimePlaceDlg->lontext->SetValue(
		DMStoString(&(chartdata[nchart].longitude), "E", "W"));
	tmpstr.Printf("%d", chartdata[nchart].timezone);
	InputDateTimePlaceDlg->tztext->SetValue(tmpstr);
	InputDateTimePlaceDlg->dst_check->SetValue(chartdata[nchart].is_dst != 0);
	InputDateTimePlaceDlg->progressed_check->SetValue(chartdata[nchart].is_progressed != 0);
	InputDateTimePlaceDlg->progressed_check->Enable(nchart != CHART_TRANSITS);
	if(nchart == CHART_DEFAULT)
	{
		tmpstr = InputDateTimePlaceDlg->progressed_check->GetTitle();
		InputDateTimePlaceDlg->progressed_check->SetTitle("Set Time to Now on Startup");
		InputDateTimePlaceDlg->progressed_check->SetValue(
			chartdataext[CHART_DEFAULT].set_to_now);
	}
	InputDateTimePlaceDlg->SetTitle(title);
	if(InputDateTimePlaceDlg->ShowModal() == wxID_OK)
	{
		if((nchart == CHART_DEFAULT) &&
			(wxMessageBox("This will overwrite the current default chart that shows \
when the application opens.  Are you sure you want to continue?",
			"Warning", wxYES_NO | wxICON_WARNING) == wxNO))
		{
			InputDateTimePlaceDlg->progressed_check->SetTitle(tmpstr);

			return;
		}
		wxdt = InputDateTimePlaceDlg->cal->GetDate();
		chartdata[nchart].date.day   = wxdt.GetDay();
		chartdata[nchart].date.month = (wxdt.GetMonth() - wxDateTime::Jan + 1);
		chartdata[nchart].date.year  = wxdt.GetYear();

		wxdt.Set(0, 0, 0); // clear time (leave date untouched)
		if(wxdt.ParseTime(InputDateTimePlaceDlg->timetext->GetValue()) != NULL)
		{
			chartdata[nchart].time.hours = wxdt.GetHour();
			chartdata[nchart].time.min   = wxdt.GetMinute();
			chartdata[nchart].time.sec   = wxdt.GetSecond();
			chartdataext[nchart].set_to_now = false;
		}
		else
		{
			wxMessageBox("ERROR: Invalid Time Entered", "ERROR!",
				wxOK | wxCENTER | wxICON_ERROR, this);
		}

		str = InputDateTimePlaceDlg->tztext->GetValue();
		if(!str.IsNumber())
		{
			wxMessageBox("ERROR: Time Zone must be a number!", "ERROR!",
				wxOK | wxCENTER | wxICON_ERROR, this);
		}
		else
		{
			str.ToLong(&n);
			if((n < -12) || (n > 13))
				wxMessageBox("ERROR: Time Zone must be between -12 and 12", "ERROR!",
					wxOK | wxCENTER | wxICON_ERROR, this);
			else
				chartdata[nchart].timezone = n;
		}

		chartdata[nchart].is_dst = InputDateTimePlaceDlg->dst_check->GetValue() ? 1 : 0;
		chartdata[nchart].is_progressed =
			InputDateTimePlaceDlg->progressed_check->GetValue() ? 1 : 0;

		// latitude and longitude
		StringToDMS(InputDateTimePlaceDlg->lontext->GetValue(),
			&(chartdata[nchart].longitude), "E", "W", TRUE);
		StringToDMS(InputDateTimePlaceDlg->lattext->GetValue(),
			&(chartdata[nchart].latitude), "N", "S", FALSE);

		chartdataext[nchart].namestr = InputDateTimePlaceDlg->nametext->GetValue();
		chartdataext[nchart].citystr = InputDateTimePlaceDlg->citytext->GetValue();

		if(nchart == CHART_DEFAULT)
		{
			FileUtils.PutChartDataFile(this, CHART_DEFAULT,
				trPrefsDefChart(programdir));
			
			InputDateTimePlaceDlg->progressed_check->SetTitle(tmpstr);
			return;
		}

		Recalculate();
		
		ChartCanvas->SetChartData(chart, chartdata, chartdataext, FALSE);

		if((nchart == CHART_NATAL1) && (chart0 == CHART_NATAL1))
			if(chartdata[nchart].is_progressed) chart0 = CHART_PROGRESSED1;
		if((nchart == CHART_NATAL1) && (chart1 == CHART_NATAL1))
			if(chartdata[nchart].is_progressed) chart1 = CHART_PROGRESSED1;
		if((nchart == CHART_NATAL1) && (chart2 == CHART_NATAL1))
			if(chartdata[nchart].is_progressed) chart2 = CHART_PROGRESSED1;
		if((nchart == CHART_NATAL2) && (chart0 == CHART_NATAL2))
			if(chartdata[nchart].is_progressed) chart0 = CHART_PROGRESSED1;
		if((nchart == CHART_NATAL2) && (chart1 == CHART_NATAL2))
			if(chartdata[nchart].is_progressed) chart1 = CHART_PROGRESSED2;
		if((nchart == CHART_NATAL2) && (chart2 == CHART_NATAL2))
			if(chartdata[nchart].is_progressed) chart2 = CHART_PROGRESSED2;

		if((nchart == CHART_NATAL1) && (chart0 == CHART_PROGRESSED1))
			if(!chartdata[nchart].is_progressed) chart0 = CHART_NATAL1;
		if((nchart == CHART_NATAL1) && (chart1 == CHART_PROGRESSED1))
			if(!chartdata[nchart].is_progressed) chart1 = CHART_NATAL1;
		if((nchart == CHART_NATAL1) && (chart2 == CHART_PROGRESSED1))
			if(!chartdata[nchart].is_progressed) chart2 = CHART_NATAL1;
		if((nchart == CHART_NATAL2) && (chart0 == CHART_PROGRESSED2))
			if(!chartdata[nchart].is_progressed) chart0 = CHART_NATAL2;
		if((nchart == CHART_NATAL2) && (chart1 == CHART_PROGRESSED2))
			if(!chartdata[nchart].is_progressed) chart1 = CHART_NATAL2;
		if((nchart == CHART_NATAL2) && (chart2 == CHART_PROGRESSED2))
			if(!chartdata[nchart].is_progressed) chart2 = CHART_NATAL2;
			
		ChartCanvas->SetMode(mode | viewmode, chart0, chart1, chart2, TRUE, TRUE);

		if(Animator.IsRunning())
			Animator.Restart();
	}
}

void TransitsFrame::GetRestrictionsInput(int nchart, const wxString &title)
{
	int i;

	InputRestrictionsDlg->SetTitle(title);
	for(i = 0; i < (NTOTOBJECTS+12); i++)
	{
		InputRestrictionsDlg->r_check[i]->SetValue(restricted[nchart][i]);
		InputRestrictionsDlg->r_check[i]->Enable(!cantcalculate[nchart][i]);
	}
	if(InputRestrictionsDlg->ShowModal() == wxID_OK)
	{
		for(i = 0; i < (NTOTOBJECTS+12); i++)
			restricted[nchart][i] = InputRestrictionsDlg->r_check[i]->GetValue();

		if(nchart == CHART_NATAL1)
			for(i = 0; i < (NTOTOBJECTS+12); i++)
				restricted[CHART_PROGRESSED1][i] = restricted[CHART_NATAL1][i];

		if(nchart == CHART_NATAL2)
			for(i = 0; i < (NTOTOBJECTS+12); i++)
				restricted[CHART_PROGRESSED2][i] = restricted[CHART_NATAL2][i];

		ChartCanvas->SetRestrictions(restricted, r_aspects, aspect_orb);
	}
}

void TransitsFrame::GetAspectRestrictionsInput(const wxString &title)
{
	wxString s;
	int i;

	InputAspectRestrictionsDlg->SetTitle(title);
	for(i = 0; i < NASPECTS; i++)
	{
		InputAspectRestrictionsDlg->r_check[i]->SetValue(r_aspects[i]);
		s.Printf("%.1f", aspect_orb[i]);
		InputAspectRestrictionsDlg->orb_str[i] = s;
	}
	if( InputAspectRestrictionsDlg->ShowModal() == wxID_OK)
	{
		for(i = 0; i < NASPECTS; i++)
		{
			r_aspects[i] = InputAspectRestrictionsDlg->r_check[i]->GetValue();
			s = InputAspectRestrictionsDlg->orb_str[i];
			s.ToDouble(&(aspect_orb[i]));
			aspect_orb[i] = fmod(aspect_orb[i], 180.0);
			if(aspect_orb[i] < 0.0) aspect_orb[i] += 180.0;
		}

		ChartCanvas->SetRestrictions(restricted, r_aspects, aspect_orb);
	}
}

void TransitsFrame::OnTDPInputDlg_1(wxCommandEvent &event)
{
	GetTDPInput(0, "Enter Chart Info");
}

void TransitsFrame::OnTDPInputDlg_2(wxCommandEvent &event)
{
	GetTDPInput(1, "Enter Partner's Chart Info");
}

void TransitsFrame::OnTDPInputDlg_3(wxCommandEvent &event)
{
	GetTDPInput(2, "Enter Transit Chart Info");
}

void TransitsFrame::OnTDPInputDlg_Def(wxCommandEvent &event)
{
	GetTDPInput(CHART_DEFAULT, "Enter Default Chart Info");
}

void TransitsFrame::OnRestrInputDlg_1(wxCommandEvent &event)
{
	GetRestrictionsInput(0, "Restrictions");
}

void TransitsFrame::OnRestrInputDlg_2(wxCommandEvent &event)
{
	GetRestrictionsInput(1, "Partner's Restrictions");
}

void TransitsFrame::OnRestrInputDlg_3(wxCommandEvent &event)
{
	GetRestrictionsInput(2, "Transit Restrictions");
}

void TransitsFrame::OnRestrInputDlg_4(wxCommandEvent &event)
{
	GetRestrictionsInput(3, "Davison Chart Restrictions");
}

void TransitsFrame::OnRestrInputDlg_5(wxCommandEvent &event)
{
	GetRestrictionsInput(4, "Composite Chart Restrictions");
}

void TransitsFrame::OnRestrInputDlg_Def(wxCommandEvent &event)
{
	GetRestrictionsInput(CHART_DEFAULT, "Default Chart Restrictions");
}

void TransitsFrame::OnAspectRestrDlg(wxCommandEvent &event)
{
	GetAspectRestrictionsInput("Aspect Settings");
}




void TransitsFrame::LoadChartDataFromFile(const int nchart, const int filetype)
{
	wxString filename, dialog_title, who;

	switch(filetype)
	{
	case ASTROLOG_FILE:
		dialog_title = _("Import _ Astrolog Chart Info File");
		break;
	case NATIVE_FILE:
	default:
		dialog_title = _("Open _ Chart Data");
	}

	switch(nchart)
	{
	case CHART_NATAL1: who = ""; break;
	case CHART_NATAL2: who = _("Partner's "); break;
	case CHART_TRANSITS: who = _("Transit "); break;
	case CHART_DEFAULT: who = _("Default "); break;
	}
	dialog_title.Replace("_ ", who, TRUE);

	filename = wxFileSelector(dialog_title, "", "", "",
		(filetype == ASTROLOG_FILE) ? ASTFILEFILTER : NATIVEFILEFILTER,
		wxOPEN | wxFILE_MUST_EXIST | wxHIDE_READONLY);

	if((nchart == CHART_DEFAULT) && (!filename.empty()) &&
		(wxMessageBox("This will overwrite the current default chart that shows \
when the application opens.  Are you sure you want to continue?",
		"Warning", wxYES_NO | wxICON_WARNING) == wxNO))
	{
		return;
	}

	if( !filename.empty() )
	{
		if(filetype == ASTROLOG_FILE)
			FileUtils.GetAstrologFile(this, nchart, filename);
		else
			FileUtils.GetChartDataFile(this, nchart, filename);

		if(nchart == CHART_DEFAULT)
		{
			FileUtils.PutChartDataFile(this, CHART_DEFAULT,
				trPrefsDefChart(programdir));
		}
	}
}

void TransitsFrame::SaveChartDataToFile(const int nchart, const int filetype)
{
	wxString filename, dialog_title, who;

	switch(filetype)
	{
	case ASTROLOG_FILE:
		dialog_title = _("Export _ Chart Data to an Astrolog File");
		break;
	case NATIVE_FILE:
	default:
		dialog_title = _("Save _ Chart Data");
	}

	switch(nchart)
	{
	case CHART_NATAL1: who = ""; break;
	case CHART_NATAL2: who = _("Partner's "); break;
	case CHART_TRANSITS: who = _("Transit "); break;
	case CHART_DEFAULT: who = _("a Copy of the Default "); break;
	}
	dialog_title.Replace("_ ", who, TRUE);

	filename = wxFileSelector(dialog_title, "", "", "",
		(filetype == ASTROLOG_FILE) ? ASTFILEFILTER : NATIVEFILEFILTER,
		wxSAVE | wxOVERWRITE_PROMPT);

	if( !filename.empty() )
	{
		if(filetype == ASTROLOG_FILE)
			FileUtils.PutAstrologFile(this, nchart, filename);
		else
			FileUtils.PutChartDataFile(this, nchart, filename);
	}
}

void TransitsFrame::OnOpenChart_1(wxCommandEvent &event)
{
	LoadChartDataFromFile(0, NATIVE_FILE);
}

void TransitsFrame::OnOpenChart_2(wxCommandEvent &event)
{
	LoadChartDataFromFile(1, NATIVE_FILE);
}

void TransitsFrame::OnOpenChart_3(wxCommandEvent &event)
{
	LoadChartDataFromFile(2, NATIVE_FILE);
}

void TransitsFrame::OnOpenChart_Def(wxCommandEvent &event)
{
	LoadChartDataFromFile(CHART_DEFAULT, NATIVE_FILE);
}

void TransitsFrame::OnSaveChartAs_1(wxCommandEvent &event)
{
	SaveChartDataToFile(0, NATIVE_FILE);
}

void TransitsFrame::OnSaveChartAs_2(wxCommandEvent &event)
{
	SaveChartDataToFile(1, NATIVE_FILE);
}

void TransitsFrame::OnSaveChartAs_3(wxCommandEvent &event)
{
	SaveChartDataToFile(2, NATIVE_FILE);
}

void TransitsFrame::OnSaveChartAs_Dav(wxCommandEvent &event)
{
	SaveChartDataToFile(CHART_DAVISON, NATIVE_FILE);
}

void TransitsFrame::OnSaveChartAs_Def(wxCommandEvent &event)
{
	SaveChartDataToFile(CHART_DEFAULT, NATIVE_FILE);
}

void TransitsFrame::OnImportAstrolog_1(wxCommandEvent &event)
{
	LoadChartDataFromFile(0, ASTROLOG_FILE);
}

void TransitsFrame::OnImportAstrolog_2(wxCommandEvent &event)
{
	LoadChartDataFromFile(1, ASTROLOG_FILE);
}

void TransitsFrame::OnImportAstrolog_3(wxCommandEvent &event)
{
	LoadChartDataFromFile(2, ASTROLOG_FILE);
}

void TransitsFrame::OnImportAstrolog_Def(wxCommandEvent &event)
{
	LoadChartDataFromFile(CHART_DEFAULT, ASTROLOG_FILE);
}

void TransitsFrame::OnExportAstrolog_1(wxCommandEvent &event)
{
	SaveChartDataToFile(0, ASTROLOG_FILE);
}

void TransitsFrame::OnExportAstrolog_2(wxCommandEvent &event)
{
	SaveChartDataToFile(1, ASTROLOG_FILE);
}

void TransitsFrame::OnExportAstrolog_3(wxCommandEvent &event)
{
	SaveChartDataToFile(2, ASTROLOG_FILE);
}

void TransitsFrame::OnExportAstrolog_Dav(wxCommandEvent &event)
{
	SaveChartDataToFile(CHART_DAVISON, ASTROLOG_FILE);
}

void TransitsFrame::OnExportAstrolog_Def(wxCommandEvent &event)
{
	SaveChartDataToFile(CHART_DEFAULT, ASTROLOG_FILE);
}




void TransitsFrame::SwapCharts(int n1, int n2)
{
	trTimeDatePlace tmptdp;
	wxString tmpstr;
	bool b;

	// As only natal charts may be progressed (other types of charts
	// are progressed indirectly by progressing their requisite natal
	// charts), we only have to worry about the case where we're
	// swapping the natal charts.  If there are two non-natal charts,
	// neither can be progressed to start with; if one is a natal
	// chart and the other is not, the other will not be progressed
	// as a matter of course, whereas that one that is will no longer
	// be allowed to be progressed when it is in its new station.
	if(((n1 == CHART_NATAL1) && (n2 == CHART_NATAL2)) ||
	   ((n1 == CHART_NATAL2) && (n2 == CHART_NATAL1)) )
	{
		// Leave it alone for now; it will be swapped when
		// the entire structure is swapped
	}
	else if((n1 == CHART_DEFAULT) || (n2 == CHART_DEFAULT))
	{
		// is_progressed for a default chart means something
		// completely different; it means you should set the chart
		// to "Now" on startup.
		if(wxMessageBox(
_("This will change the default chart that loads on program startup.  Continue?"),
			"Warning", wxYES_NO | wxCENTRE | wxICON_WARNING, this) == wxNO)
		{
			return;
		}

		if(n1 == CHART_DEFAULT)
		{
			chartdata[n1].is_progressed = FALSE;
			chartdata[n2].is_progressed = chartdataext[n2].set_to_now;
		}
		else
		{
			chartdata[n2].is_progressed = FALSE;
			chartdata[n1].is_progressed = chartdataext[n1].set_to_now;
		}
	}
	else
	{
		chartdata[n1].is_progressed = FALSE;
		chartdata[n2].is_progressed = FALSE;
	}

	tmptdp = chartdata[n1];
	chartdata[n1] = chartdata[n2];
	chartdata[n2] = tmptdp;

	tmpstr = chartdataext[n1].namestr;
	chartdataext[n1].namestr = chartdataext[n2].namestr;
	chartdataext[n2].namestr = tmpstr;

	tmpstr = chartdataext[n1].citystr;
	chartdataext[n1].citystr = chartdataext[n2].citystr;
	chartdataext[n2].citystr = tmpstr;

	b = chartdataext[n1].set_to_now;
	chartdataext[n1].set_to_now = chartdataext[n2].set_to_now;
	chartdataext[n2].set_to_now = b;

	if((n1 == CHART_DEFAULT) || (n2 == CHART_DEFAULT))
	{
		FileUtils.PutChartDataFile(this, CHART_DEFAULT,
			trPrefsDefChart(programdir));
	}

	Recalculate();
	SetChartsToView(mode);

	ChartCanvas->SetMode(mode | viewmode, chart0, chart1, chart2, FALSE, FALSE);
	ChartCanvas->SetChartData(chart, chartdata, chartdataext);
}

void TransitsFrame::OnSwap_1_2(wxCommandEvent &event)
{
	SwapCharts(CHART_NATAL1, CHART_NATAL2);
}

void TransitsFrame::OnSwap_1_3(wxCommandEvent &event)
{
	SwapCharts(CHART_NATAL1, CHART_TRANSITS);
}

void TransitsFrame::OnSwap_2_3(wxCommandEvent &event)
{
	SwapCharts(CHART_NATAL2, CHART_TRANSITS);
}

void TransitsFrame::OnSwap_1_Def(wxCommandEvent &event)
{
	SwapCharts(CHART_NATAL1, CHART_DEFAULT);
}

void TransitsFrame::OnCalc_Settings(wxCommandEvent &event)
{
	CalculationSettingsDlg->housesys->SetSelection(hsys - FIRST_HSYSTEM);
	CalculationSettingsDlg->nnodetype->SetSelection(
		(nodemode == USE_MEAN) ? 0 : 1);
	CalculationSettingsDlg->lilithtype->SetSelection(
		(apogmode == USE_MEAN) ? 0 : 1);
	if(CalculationSettingsDlg->ShowModal() == wxID_OK)
	{
		hsys = CalculationSettingsDlg->housesys->GetSelection()+FIRST_HSYSTEM;
		nodemode = (CalculationSettingsDlg->nnodetype->GetSelection() == 0) ?
			USE_MEAN : USE_TRUE;
		apogmode = (CalculationSettingsDlg->lilithtype->GetSelection() == 0) ?
			USE_MEAN : USE_TRUE;
		set_node_type(nodemode);
		set_apog_type(apogmode);

		Recalculate();
		ChartCanvas->SetMode(mode | viewmode, chart0, chart1, chart2, FALSE, FALSE);
		ChartCanvas->SetChartData(chart, chartdata, chartdataext);
	}
}






void TransitsFrame::OnAspectsViewAll(wxCommandEvent &event)
{
	ChartInfo->SetAspectsMode(INFO_ASPECTS_ALL);
}

void TransitsFrame::OnAspectsViewByOrb(wxCommandEvent &event)
{
	ChartInfo->SetAspectsMode(INFO_ASPECTS_ALL | INFO_ASPECTS_BYORB);
}

void TransitsFrame::OnAspectsViewByPlanet1(wxCommandEvent &event)
{
	ChartInfo->SetAspectsMode(INFO_ASPECTS_ALL | INFO_ASPECTS_BYPL1);
}

void TransitsFrame::OnAspectsViewByPlanet2(wxCommandEvent &event)
{
	ChartInfo->SetAspectsMode(INFO_ASPECTS_ALL | INFO_ASPECTS_BYPL2);
}

void TransitsFrame::OnAspectsSearch(wxCommandEvent &event)
{
	FindAspects();
}

void TransitsFrame::FindAspects()
{
	wxArrayInt sels;
	bool find_pl1[NTOTOBJECTS+12], find_pl2[NTOTOBJECTS+12];
	bool find_asps[NASPECTS], need_both;
	double min_orb, max_orb;
	bool is_restr1, is_restr2;
	int i, n1, n2, n_sels;

	// Bring up the search dialog
	FindAspectsDlg->planet1->Clear();
	FindAspectsDlg->planet2->Clear();
	FindAspectsDlg->aspect->Clear();
	FindAspectsDlg->n_pl1 = 0;
	FindAspectsDlg->n_pl2 = 0;
	FindAspectsDlg->n_asps = 0;
	for(i = 0; i < (NTOTOBJECTS + 12); i++)
	{
		if(IS_ONE_CHART(mode))
		{
			FindAspectsDlg->is_single_chart = TRUE;
			is_restr1 = is_restr2 = restricted[chart0][i];
		}
		else
		{
			FindAspectsDlg->is_single_chart = FALSE;
			is_restr1 = restricted[chart1][i];
			is_restr2 = restricted[chart2][i];
		}
		if(!is_restr1) FindAspectsDlg->lbpl1[(FindAspectsDlg->n_pl1)++] = i;
		if(!is_restr2) FindAspectsDlg->lbpl2[(FindAspectsDlg->n_pl2)++] = i;
		n1 = FindAspectsDlg->n_pl1 - 1;
		n2 = FindAspectsDlg->n_pl2 - 1;
		if(i < NTOTOBJECTS)
		{
			if(!is_restr1)
				FindAspectsDlg->planet1->Append(trGetName(FIRST_PLANET+i));
			if(!is_restr2)
				FindAspectsDlg->planet2->Append(trGetName(FIRST_PLANET+i));
		}
		else
		{
			if(!is_restr1)
				FindAspectsDlg->planet1->Append(trGetName(FIRST_HOUSE+i-NTOTOBJECTS));
			if(!is_restr2)
				FindAspectsDlg->planet2->Append(trGetName(FIRST_HOUSE+i-NTOTOBJECTS));
		}
		if(!is_restr1) FindAspectsDlg->planet1->SetSelection(n1);
		if(!is_restr2) FindAspectsDlg->planet2->SetSelection(n2);
	}
	for(i = 0; i < NASPECTS; i++)
	{
		if(!r_aspects[i])
		{
			FindAspectsDlg->lbasps[(FindAspectsDlg->n_asps)++] = i;
			FindAspectsDlg->aspect->Append(trGetName(FIRST_ASPECT+i));
			FindAspectsDlg->aspect->SetSelection(FindAspectsDlg->n_asps - 1);
		}
	}
	FindAspectsDlg->min_orb_str = "0.0";
	FindAspectsDlg->max_orb_str = "360.0";
#ifdef __WXMSW__
	FindAspectsDlg->planet1->SetFirstItem(0);
	FindAspectsDlg->aspect->SetFirstItem( 0);
	FindAspectsDlg->planet2->SetFirstItem(0);
#endif
	if( FindAspectsDlg->ShowModal() == wxID_OK )
	{
		// Tell the Chart Info window the criteria selected and let it
		// do the search
		for(i = 0; i < (NTOTOBJECTS+12); i++)
		{
			find_pl1[i] = FALSE;
			find_pl2[i] = FALSE;
		}
		for(i = 0; i < NASPECTS; i++)
			find_asps[i] = FALSE;

		FindAspectsDlg->min_orb_str.ToDouble(&min_orb);
		FindAspectsDlg->max_orb_str.ToDouble(&max_orb);

		n_sels = FindAspectsDlg->planet1->GetSelections(sels);
		for(i = 0; i < n_sels; i ++)
			find_pl1[FindAspectsDlg->lbpl1[sels[i]]] = TRUE;
		n_sels = FindAspectsDlg->planet2->GetSelections(sels);
		for(i = 0; i < n_sels; i ++)
			find_pl2[FindAspectsDlg->lbpl2[sels[i]]] = TRUE;
		n_sels = FindAspectsDlg->aspect->GetSelections(sels);
		for(i = 0; i < n_sels; i ++)
			find_asps[FindAspectsDlg->lbasps[sels[i]]] = TRUE;

		need_both = FindAspectsDlg->needboth->GetValue();

		ChartInfo->SetFindParams(find_pl1, find_pl2, find_asps,
		                         need_both, min_orb, max_orb);
		if     (FindAspectsDlg->sortorb->GetValue())
			ChartInfo->SetAspectsMode(INFO_ASPECTS_RESULTS | INFO_ASPECTS_BYORB);
		else if(FindAspectsDlg->sortpl1->GetValue())
			ChartInfo->SetAspectsMode(INFO_ASPECTS_RESULTS | INFO_ASPECTS_BYPL1);
		else if(FindAspectsDlg->sortpl2->GetValue())
			ChartInfo->SetAspectsMode(INFO_ASPECTS_RESULTS | INFO_ASPECTS_BYPL2);
	}
	SetFocus();
}
