//TransitsFrm.h
//
//Copyright (C) 2003 mikesag
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//This is the main form, which is mainly just a container for the chart,
//information panel, and handles the menus and events

#ifndef TransitsFrame_H
#define TransitsFrame_H

#include "AstroCalc.h"
#include "TransitsAnim.h"
#include "TransitsFiles.h"
#include "TransitsChart.h"
#include "TransitsInfo.h"
#include "InputDialogs.h"

class TransitsAnim;
class TransitsFiles;

// Define a new frame type: this is going to be our main frame
class TransitsFrame : public wxFrame
{
	friend class TransitsAnim;
	friend class TransitsFiles;
public:
    // ctor(s)
    TransitsFrame(const wxString& title, const wxPoint& pos, const wxSize& size,
		const wxString& progdir);
	virtual ~TransitsFrame();

	trChartInfo chart[NCHARTTYPES];
	trChartDataExt chartdataext[NCHARTTYPES];
	trTimeDatePlace chartdata[NCHARTTYPES];

	CChartCanvas *ChartCanvas;
	CChartInfo *ChartInfo;

	CInputDateTimePlaceDlg *InputDateTimePlaceDlg;
	CInputRestrictionsDlg  *InputRestrictionsDlg;
	CInputAspectRestrictionsDlg *InputAspectRestrictionsDlg;
	CCalculationSettingsDlg *CalculationSettingsDlg;
	CFindAspectsDlg *FindAspectsDlg;
	CTimeMotionDlg *TimeMotionDlg;

	TransitsAnim Animator;
	TransitsFiles FileUtils;

	void Recalculate();

    // event handlers (these functions should _not_ be virtual)
	void OnClose(wxCloseEvent &event);
    void OnQuit(wxCommandEvent &event);
    void OnAbout(wxCommandEvent &event);

	void OnTimer(wxTimerEvent &event);
	void OnNowTimer(wxTimerEvent &event);

	void OnTimeShiftForward(wxCommandEvent &event);
	void OnTimeShiftBackward(wxCommandEvent &event);
	void OnTimeGoForward(wxCommandEvent &event);
	void OnTimeGoBackward(wxCommandEvent &event);
	void OnTimeStop(wxCommandEvent &event);
	void OnTimeSettings(wxCommandEvent &event);

	void OnImportConfig(wxCommandEvent &event);
	void OnExportConfig(wxCommandEvent &event);
	void OnSaveDefaultConfig(wxCommandEvent &event);
	void OnRestoreDefaultConfig(wxCommandEvent &event);
	void OnRestoreDefaultConfigAll(wxCommandEvent &event);

	void OnSetToNow_1(wxCommandEvent &event);
	void OnTDPInputDlg_1(wxCommandEvent &event);
	void OnRestrInputDlg_1(wxCommandEvent &event);
	void OnOpenChart_1(wxCommandEvent &event);
	void OnSaveChartAs_1(wxCommandEvent &event);
	void OnImportAstrolog_1(wxCommandEvent &event);
	void OnExportAstrolog_1(wxCommandEvent &event);
	
	void OnSetToNow_2(wxCommandEvent &event);
	void OnTDPInputDlg_2(wxCommandEvent &event);
	void OnRestrInputDlg_2(wxCommandEvent &event);
	void OnOpenChart_2(wxCommandEvent &event);
	void OnSaveChartAs_2(wxCommandEvent &event);
	void OnImportAstrolog_2(wxCommandEvent &event);
	void OnExportAstrolog_2(wxCommandEvent &event);

	void OnSetToNow_3(wxCommandEvent &event);
	void OnTDPInputDlg_3(wxCommandEvent &event);
	void OnRestrInputDlg_3(wxCommandEvent &event);
	void OnOpenChart_3(wxCommandEvent &event);
	void OnSaveChartAs_3(wxCommandEvent &event);
	void OnImportAstrolog_3(wxCommandEvent &event);
	void OnExportAstrolog_3(wxCommandEvent &event);

	void OnRestrInputDlg_4(wxCommandEvent &event);
	void OnRestrInputDlg_5(wxCommandEvent &event);
	void OnExportAstrolog_Dav(wxCommandEvent &event);
	void OnSaveChartAs_Dav(wxCommandEvent &event);

	void OnSetToNow_Def(wxCommandEvent &event);
	void OnTDPInputDlg_Def(wxCommandEvent &event);
	void OnRestrInputDlg_Def(wxCommandEvent &event);
	void OnOpenChart_Def(wxCommandEvent &event);
	void OnSaveChartAs_Def(wxCommandEvent &event);
	void OnImportAstrolog_Def(wxCommandEvent &event);
	void OnExportAstrolog_Def(wxCommandEvent &event);

	void OnSwap_1_2(wxCommandEvent &event);
	void OnSwap_1_3(wxCommandEvent &event);
	void OnSwap_2_3(wxCommandEvent &event);
	void OnSwap_1_Def(wxCommandEvent &event);

	void OnCalc_Settings(wxCommandEvent &event);

	void OnViewNatal(wxCommandEvent &event);
	void OnViewComposite(wxCommandEvent &event);
	void OnViewDavison(wxCommandEvent &event);
	void OnViewSynasty(wxCommandEvent &event);
	void OnViewWithTransits(wxCommandEvent &event);
	void OnViewAspects(wxCommandEvent &event);
	void OnViewAspConfig(wxCommandEvent &event);
	void OnViewDispositors(wxCommandEvent &event);
	void OnViewDispositorsByHouse(wxCommandEvent &event);

	void OnAspectsViewAll(wxCommandEvent &event);
	void OnAspectsViewByOrb(wxCommandEvent &event);
	void OnAspectsViewByPlanet1(wxCommandEvent &event);
	void OnAspectsViewByPlanet2(wxCommandEvent &event);
	void OnAspectsSearch(wxCommandEvent &event);
	void OnAspectRestrDlg(wxCommandEvent &event);

	int  GetMode();
private:
	void InitVariables();
	void SetChartsToView(int nmode);
	void ViewMenuCheckHelper(int itemid);
	void SetToNow(int nchart, bool do_refresh, bool do_write = true);
	wxColour GetSysColour(wxSystemColour index);
	void GetRestrictionsInput(int nchart, const wxString &title);
	void GetAspectRestrictionsInput(const wxString &title);
	void GetTDPInput(int nchart, const wxString &title);
	void SaveChartDataToFile(const int nchart, const int filetype);
	void LoadChartDataFromFile(const int nchart, const int filetype);
	void FindAspects();
	void SwapCharts(int n1, int n2);

	bool cantcalculate[NCHARTTYPES][NTOTOBJECTS+12];
	bool restricted[NCHARTTYPES][NTOTOBJECTS+12];
	bool r_aspects[NASPECTS];
	double aspect_orb[NASPECTS];

	bool is_now;

	int hsys;
	int nodemode, apogmode;
	int mode, viewmode;
	int chart0, chart1, chart2, nowchart;

	wxMenu *fileMenu, *dataMenu, *viewMenu, *infoMenu, *timeMenu, *toolMenu, *helpMenu;

	wxSizer *topsizer;
	wxToolBar *Toolbar[3];
	wxTimer *nowtimer;

	wxString programdir;

    DECLARE_CLASS(TransitsFrame)
    // any class wishing to process wxWindows events must use this macro
    DECLARE_EVENT_TABLE()
};

#endif // TransitsFrame_H
