//TransitsInfo.cpp: The info pane that sits off to the side of the chart
//
//Copyright (C) 2003 mikesag
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// ----------------------------------------------------------------------------
// headers
// ----------------------------------------------------------------------------
#include "wx/wxprec.h"

#ifndef WX_PRECOMP
	#include "wx/wx.h"
#endif

#include "wx/settings.h"
#include "wx/tokenzr.h"

#include <math.h>
#include <locale.h>

#include "AstroConst.h"
#include "AstroCalc.h"

#include "TurtleDraw.h"
#include "Translation.h"
#include "Typewriter.h"

#include "TransitsFiles.h"
#include "TransitsChart.h"
#include "TransitsInfo.h"

#define IS_SELECTED_ASPECTS(mode, aspmode) ((mode == INFO_MODE_ASPECTS) && \
		!(aspmode & INFO_ASPECTS_RESULTS) && !(aspmode & INFO_ASPECTS_ALL))

static const double N_PI = acos(0)*2.0;

static const wxFont default_font = wxFont(8, wxMODERN, wxNORMAL, wxNORMAL);
static const wxFont title_font =   wxFont(12, wxSWISS, wxNORMAL, wxBOLD);

// ----------------------------------------------------------------------------
// local utility functions
// ----------------------------------------------------------------------------

#ifndef inline
#define inline
#endif

inline static int iround(double x)
{
	if(x > 0.0)
		return ((int)floor(x + 0.5));
	else
		return ((int)ceil(x - 0.5));
}

/*static wxString HMStoString(const trHMS *hms)
{
	wxString str;
	char s[256];
	tm tp;

	tp.tm_hour = hms->hours;
	tp.tm_min  = hms->min;
	tp.tm_sec  = hms->sec;

	setlocale(LC_ALL, "");
	strftime(s, 255, "%X", &tp);

	str = s;
	if((str.GetChar(0) == '0') &&
		((str.Find('a', 0) >= 0) || (str.Find('A', 0) >= 0) || 
		 (str.Find('p', 0) >= 0) || (str.Find('P', 0) >= 0)))
	{
		str = str.Right(str.length() - 1);
	}
	
	return str;
}

static wxString trDateToString(const trDate *trdt)
{
	wxString str;
	char s[256];
	tm tp;

	tp.tm_mday = trdt->day;
	tp.tm_mon  = trdt->month - 1;
	tp.tm_year = trdt->year - 1900;
	tp.tm_yday = 0;
	tp.tm_wday = 0;

	setlocale(LC_ALL, "");
	strftime(s, 255, "%d %b %Y", &tp);

	str = s;
	if(str.GetChar(0) == '0')
		str = str.Mid(1, str.Length() - 1);
	
	return str;	
}*/

// ----------------------------------------------------------------------------
// chart information window
// ----------------------------------------------------------------------------

IMPLEMENT_CLASS(CChartInfo, wxScrolledWindow)
BEGIN_EVENT_TABLE(CChartInfo, wxScrolledWindow)
	EVT_SIZE      (CChartInfo::OnResize)
    EVT_PAINT     (CChartInfo::OnPaint)
	EVT_LEFT_DOWN (CChartInfo::OnMouseDown)
END_EVENT_TABLE()

CChartInfo::CChartInfo(wxFrame *parent, CChartCanvas *chartcanvas)
        : wxScrolledWindow(parent, -1, wxDefaultPosition, wxSize(200,100),
                           wxHSCROLL | wxVSCROLL)
{
	nchart  = 0;
	nchart1 = 0;
	nchart2 = 1;
    owner = parent;
	chartdrawing = chartcanvas;
	glsize = 10;
	mode = INFO_MODE_SUMMARY;
	aspmode = 0;
	selected = -1;
	SetBackgroundColour(*wxBLACK);
	n_aspect_order = 0;
}

void CChartInfo::SetChartData(trChartInfo ci[NCHARTTYPES], trTimeDatePlace cd[NCHARTTYPES],
							  trChartDataExt ce[NCHARTTYPES])
{
	int i;

	for(i = 0; i < NCHARTTYPES; i++)
	{
		chart[i] = ci[i];
		chartdata[i] = cd[i];
		chartdataext[i] = ce[i];
	}

	Refresh(FALSE);
}

void CChartInfo::SetRestrictions(bool restr[NCHARTTYPES][NTOTOBJECTS+12], bool rasps[NASPECTS],
								 bool do_refresh)
{
	int i, j;

	for(i = 0; i < NCHARTTYPES; i++)
		for(j = 0; j < (NTOTOBJECTS+12); j++)
			restricted[i][j] = restr[i][j];

	for(i = 0; i < NASPECTS; i++)
		r_aspects[i] = rasps[i];

	if(IS_SELECTED_ASPECTS(mode, aspmode))
	{
		SetMode(INFO_MODE_SUMMARY, -1, CHART_NATAL1);
	}

	if(do_refresh)
		Refresh(FALSE);
}

void CChartInfo::SetAspects(int n_aspects, aspect_storage cache[MAX_ASPECT_CACHE])
{
	int i;
	n_aspect_cache = n_aspects;
	n_aspect_order = n_aspect_cache;

	for(i = 0; i < n_aspect_cache; i++)
		aspect_cache[i] = cache[i];

	sort_aspects(selected, aspmode, aspect_order, n_aspect_cache);
}

void CChartInfo::SetAspectOver(int n_over, bool over[MAX_ASPECT_CACHE])
{
	int i;
	n_aspects_over = n_over;

	for(i = 0; i < MAX_ASPECT_CACHE; i++)
		aspect_over[i]  = over[i];

	sort_aspects(selected, aspmode, aspect_order, n_aspects_over);
}

void CChartInfo::SetAspectsMode(int amode, bool do_refresh)
{
	aspmode = amode;
	selected = -1;
	nchartsel = IS_ONE_CHART(viewmode) ? nchart : nchart1;
	SetMode(INFO_MODE_ASPECTS, selected, nchartsel);
	sort_aspects(selected, aspmode, aspect_order, n_aspect_cache);
	if(do_refresh)
		Refresh(FALSE);
}

void CChartInfo::SetFindParams(bool find_pl1[NTOTOBJECTS+12],
							   bool find_pl2[NTOTOBJECTS+12],
							   bool find_asps[NASPECTS],
							   bool need_both,
							   double min_orb, double max_orb)
{
	int i;

	find_orb_min = min_orb;
	find_orb_max = max_orb;

	find_need_both = need_both;

	for(i = 0; i < (NTOTOBJECTS + 12); i++)
	{
		find_planet1[i] = find_pl1[i];
		find_planet2[i] = find_pl2[i];
	}

	for(i = 0; i < NASPECTS; i++)
		find_aspects[i] = find_asps[i];
}

void CChartInfo::SetChartSel(int nch, int nch1, int nch2)
{
	nchart  = nch;
	nchart1 = nch1;
	nchart2 = nch2;
}

void CChartInfo::SetViewMode(int newmode)
{
	if(IS_SELECTED_ASPECTS(mode, aspmode))
		SetMode(INFO_MODE_SUMMARY, -1, nchart);
	else
		SetMode(mode, selected, IS_TWO_CHARTS(mode) ? nchart1 : nchart);

	viewmode = newmode;
	
	Refresh(FALSE);
}

void CChartInfo::SetMode(int nmode, int nobj, int nchsel)
{
	mode = nmode;
	nchartsel = nchsel;
	if(nobj >= 0) selected = nobj; // is all that's needed for all but aspect lists
	else selected = -1;
}

int CChartInfo::GetMode()
{
	return mode;
}

int CChartInfo::GetObj()
{
	return selected;
}

int CChartInfo::CompareMode(int nmode, int nobj, int nchs)
{
	if(nobj < 0) nobj = -1;
	return ((mode == nmode) && (selected == nobj) && (nchs == nchartsel)) ? 0 : -1;
}

void CChartInfo::OnMouseDown(wxMouseEvent &event)
{
}

void CChartInfo::OnResize(wxSizeEvent &event)
{
	Refresh(FALSE);
}



// Painting routines

void CChartInfo::OnPaint(wxPaintEvent &event)
{
	int a, xx, yy, xs, ys, y1, y2;
	wxCoord w, h;
	wxPaintDC dc(this);
	PrepareDC(dc);

/*#ifdef __WXMSW__
	GetVirtualSize(&w, &h);
#else
	GetSize(&w, &h);
#endif*/
	GetClientSize(&w, &h);
	w = dc.DeviceToLogicalX(w);
	h = dc.DeviceToLogicalY(h);

	wxBitmap bitbuf(w, h);
	wxMemoryDC memdc;
	memdc.SelectObject(bitbuf);
	memdc.SetBackground(wxBrush(*wxBLACK, wxSOLID));
	memdc.Clear();

	// draw the border line to the left to visually separate from the chart
	memdc.SetPen(wxPen(*wxWHITE, 1, wxSOLID));
	memdc.DrawLine(0, 0, 0, h);

	a = aspmode;
	GetViewStart(&xx, &yy);
	GetScrollPixelsPerUnit(&xs, &ys);
	y1 = yy*ys;
	y2 = y1 + h;

	switch(mode)
	{
	case INFO_MODE_PLANET:  DrawPlanetInfo(memdc, w, h, selected);    break;
	case INFO_MODE_SIGN:    DrawSignInfo(memdc, w, h, selected);      break;
	case INFO_MODE_HOUSE:   DrawHouseInfo(memdc, w, h, selected);     break;
	case INFO_MODE_ASPECTS: DrawAspectInfo(memdc, w, h, -1, a,
								TRUE, y1, y2);                        break;
	case INFO_MODE_SUMMARY: DrawChartSummary(memdc, w, h);            break;
	default: DrawChartSummary(memdc, w, h);
	}

	dc.Blit(0, 0, w, h, &memdc, 0, 0);
}

void CChartInfo::UpdateVScroll(int newscrollheight)
{
	static int sb_len = 0;
	int newlen = twr.GetY();
	if(sb_len != newlen)
	{
		SetScrollbars(1, 1, 0, twr.GetY());
		sb_len = newlen;
	}
}

void CChartInfo::DrawChartSummary(wxDC &dc, int w, int h)
{
	int i, j, lft;
	int nch, nhouse;
	wxString str;
	wxColour col1, hdrcol;
	trSignPos signpos;

	lft = (w - 2*glsize - twr.TextExtentX(dc, ": 00  00'00\"  ", glsize))/2;
	twr.Reset(lft, 10, w, h);

	// Type the header
	if(viewmode & NATAL_CHART)
		twr.TypeText(dc, "Natal Chart", *wxWHITE, glsize+2, true, wxCENTER, wxSWISS);
	else if(viewmode & COMPOSITE_CHART)
		twr.TypeText(dc, "Composite Chart", *wxWHITE, glsize+2, true, wxCENTER, wxSWISS);
	else if(viewmode & DAVISON_CHART)
		twr.TypeText(dc, "Davison Chart", *wxWHITE, glsize+2, true, wxCENTER, wxSWISS);
	else if(viewmode & SYNASTY_CHART)
		twr.TypeText(dc, "Synastry", *wxWHITE, glsize+2, true, wxCENTER, wxSWISS);
	else if(viewmode & TRANSITS_CHART)
		twr.TypeText(dc, "Transits", *wxWHITE, glsize+2, true, wxCENTER, wxSWISS);
	twr.NewLine(3*(glsize+2));

	nch = nchart;
	for(i = 0; i < 3; i++)
	{
		if(viewmode & NATAL_CHART)
		{
			nch = nchart;
			if(i > 0)
				break;
		}
		else if((viewmode & COMPOSITE_CHART) || (viewmode & DAVISON_CHART))
		{
			if(i == 0)
			{
				twr.TypeText(dc, "Native", *wxWHITE, glsize, true, wxCENTER, wxSWISS);
				twr.NewLine(glsize + glsize/2);
				nch = nchart1;
			}
			else if(i == 1)
			{
				twr.TypeText(dc, "Partner", *wxWHITE, glsize, true, wxCENTER, wxSWISS);
				twr.NewLine(glsize + glsize/2);
				nch = nchart2;
			}
			else
			{
				if(!(viewmode & DAVISON_CHART))
					break;

				twr.TypeText(dc, "Midpoint", *wxWHITE, glsize, true, wxCENTER, wxSWISS);
				twr.NewLine(glsize + glsize/2);
				nch = nchart;				
			}
		}
		else if((viewmode & SYNASTY_CHART) || (viewmode & TRANSITS_CHART))
		{
			if(i == 0)
			{
				twr.TypeText(dc, "Native", *wxWHITE, glsize, true, wxCENTER, wxSWISS);
				twr.NewLine(glsize + glsize/2);
				nch = nchart1;
			}
			else if(i == 1)
			{
				if(viewmode & SYNASTY_CHART)
					twr.TypeText(dc, "Partner", *wxWHITE, glsize, true, wxCENTER, wxSWISS);
				else if(viewmode & TRANSITS_CHART)
					twr.TypeText(dc, "Transits", *wxWHITE, glsize, true, wxCENTER, wxSWISS);
				twr.NewLine(glsize + glsize/2);
				nch = nchart2;
			}
			else
				break;
		}
		else
			wxMessageBox("ERROR: Should not have gotten here (DrawChartSummary)",
				wxMessageBoxCaptionStr, wxICON_ERROR);
		
		hdrcol = wxColour(192, 192, 192);
		if(chartdataext[nch].namestr.length() > 0)
			twr.TypeText(dc, chartdataext[nch].namestr + "\n", hdrcol, glsize,
				false, wxCENTER, wxMODERN);
		if(chartdataext[nch].citystr.length() > 0)
			twr.TypeText(dc, chartdataext[nch].citystr + "\n", hdrcol, glsize,
				false, wxCENTER, wxMODERN);
		if((chartdataext[nch].namestr.length() > 0) &&
				(chartdataext[nch].citystr.length() > 0))
			twr.NewLine(glsize);
		if(chartdata[nch].is_progressed)
			twr.TypeText(dc, "Progressed to\n", hdrcol, glsize, false, wxCENTER, wxMODERN);
		if((chartdata[nch].date.year >= 1000) && (chartdata[nch].date.year <= 4000))
			str = trDateToString(&(chartdata[nch].date)) + "\n";
		else
			str.Printf("%d %.3s %d\n", chartdata[nch].date.day,
				trGetName(FIRST_MONTH+chartdata[nch].date.month-1),
				chartdata[nch].date.year);
		twr.TypeText(dc, str, hdrcol, glsize, false, wxCENTER, wxMODERN);
		str = HMStoString(&(chartdata[nch].time)) + "\n";
		twr.TypeText(dc, str, hdrcol, glsize, false, wxCENTER, wxMODERN);
		str.Printf("Timezone: %d", chartdata[nch].timezone);
		str += (chartdata[nch].is_dst ? " (+DST)\n" : "\n");
		twr.TypeText(dc, str, hdrcol, glsize, false, wxCENTER, wxMODERN);
		if((chartdataext[nch].citystr.length() > 0) &&
				!(chartdataext[nch].namestr.length() > 0))
			twr.NewLine(glsize);


		str.Printf("Lat.: %2d\xb0%02d'%02d\"", abs(chartdata[nch].latitude.deg),
			abs(chartdata[nch].latitude.min), abs(iround(chartdata[nch].latitude.sec)));
		if((chartdata[nch].latitude.deg > 0) || (chartdata[nch].latitude.deg == 0 &&
			(chartdata[nch].latitude.min > 0) || (chartdata[nch].latitude.min == 0 &&
			(chartdata[nch].latitude.sec >= 0.0) ) ))
			str += "N\n";
		else
			str += "S\n";
		twr.TypeText(dc, str, hdrcol, glsize, false, wxCENTER, wxMODERN);
		str.Printf("Lon.: %2d\xb0%02d'%02d\"", abs(chartdata[nch].longitude.deg),
			abs(chartdata[nch].longitude.min), abs(iround(chartdata[nch].longitude.sec)));
		if((chartdata[nch].longitude.deg > 0) || (chartdata[nch].longitude.deg == 0 &&
			(chartdata[nch].longitude.min > 0) || (chartdata[nch].longitude.min == 0 &&
			(chartdata[nch].longitude.sec > 0.0) ) ))
			str += "E\n";
		else
			str += "W\n";
		twr.TypeText(dc, str, hdrcol, glsize, false, wxCENTER, wxMODERN);
		twr.NewLine(2*glsize);

		if(((viewmode & NATAL_CHART)      && (i != 0)) || 
			((viewmode & COMPOSITE_CHART) && (i != 1)) ||
			((viewmode & DAVISON_CHART)   && (i != 2)) ||
			((viewmode & SYNASTY_CHART)   && (i >= 2)) ||
			((viewmode & TRANSITS_CHART)  && (i >= 2)))
		{
			continue;
		}

		if(viewmode & COMPOSITE_CHART)
		{
			twr.TypeText(dc, "Composite Positions", *wxWHITE, glsize, true, wxCENTER, wxSWISS);
			twr.NewLine(2*glsize);
		}

		// Draw the currently showing planets
		if(IS_ONE_CHART(viewmode))
			nch = nchart;
		for(j = 0; j < NTOTOBJECTS; j++)
		{
			if(restricted[nch][j]) continue;
			
			split_longitute(chart[nch].planet[j].longitude, &signpos, FALSE);
			col1 = DefaultObjectColour(trGetElement(j+FIRST_PLANET), 255);
			
			str.Printf("/g%d/: ", j+FIRST_PLANET);
			twr.TypeLine(dc, col1, 1, wxLEFT, FALSE, str);
			twr.TypeSignPos(dc, j+FIRST_PLANET, chart[nch].planet[j].longitude,
				chart[nch].planet[j].long_speed, 1, 1, wxNO_ALIGN, FALSE);
			if(IS_ONE_CHART(viewmode))
				nhouse = chart[nchart2].planet[j].in_house;
			else
				nhouse = get_which_house(j, &(chart[nch]),
					chart[(nch == nchart1) ? nchart2 : nchart1].house);
			str.Printf(" %2d", nhouse);
			col1 = DefaultObjectColour(trGetElement(nhouse+FIRST_HOUSE-1),255);
			twr.TypeLine(dc, col1, 1, wxNO_ALIGN, FALSE, str);
			twr.NewLine(3*glsize/2);
		}
		twr.NewLine(1*10);
		
		// Draw the house positions
		for(j = 1; j <= 12; j++)
		{
			split_longitute(chart[nch].house[j].cusp, &signpos, FALSE);
			col1 = DefaultObjectColour(trGetElement(j+FIRST_HOUSE-1), 255);
			
			str.Printf("/g%d/: ", j+FIRST_HOUSE-1);
			twr.TypeLine(dc, col1, 1, wxLEFT, FALSE, str);
			twr.TypeSignPos(dc, j+FIRST_HOUSE, chart[nch].house[j].cusp,
				0.0001 /* positive always */, 1, 1, wxNO_ALIGN, TRUE);
		}
		twr.NewLine(glsize);
	}

	UpdateVScroll(twr.GetY());
}

void CChartInfo::DrawPlanetInfo(wxDC &dc, int w, int h, int n)
{
	planet_rules pr;
	int i, ndis, nch, nch2, round_secs, icusp;
	double sectemp, c1, c2;
	trDMS dms;
	wxString str;
	wxColour col;

	nch = nchartsel;

	twr.Reset(10, 10, w, h);

	// Show a big glyph of the object
	col = DefaultObjectColour(trGetElement(n), 255);
	twr.TypeGlyph(dc, n, col, 4, 2, TRUE, TRUE);
	if(IS_TWO_CHARTS(viewmode))
	{
		if(nch == nchart1)
			str = "Native's\n";
		else if(viewmode & TRANSITS_CHART)
			str = "Transit\n";
		else
			str = "Partner's\n";
		twr.TypeText(dc, str, col, 12, TRUE, wxCENTER);
	}
	twr.TypeText(dc, trGetName(n), col, 12, TRUE, wxCENTER);
	twr.NewLine(glsize + glsize/2);
	
	// Display which sign/signs it rules
	pr = Rules(n);
	if(pr.n == 1)
	{
		col = DefaultObjectColour(trGetElement(pr.sign1), 255);
		str.Printf("/s1/Rules /s2//g%d/", pr.sign1);
		twr.TypeLine(dc, col, 1, wxCENTER, TRUE, str);
	}
	else if(pr.n == 2)
	{
		str.Printf("/s1/Rules /s2//g%d//s1/ and /s2//g%d/", pr.sign1, pr.sign2);
		twr.TypeLine(dc, wxColour(128,128,128), 1, wxCENTER, TRUE, str);
	}
	else if(IS_HOUSE(n))
	{
		col = DefaultObjectColour(trGetElement(n), 255);
		str.Printf("/s1/Corresponds to /s2//g%d/", ARIES+(n-FIRST_HOUSE));
		twr.TypeLine(dc, col, 1, wxCENTER, TRUE, str);
	}

	twr.NewLine(2*glsize + glsize/2);

	// Display its location by sign and by house
	// First: The sign
	if(IS_PLANETARY_BODY(n))
	{
		col = DefaultObjectColour(trGetElement(chart[nch].planet[n-SUN].sgn.sign), 255);
		twr.SetX((w - 2*glsize - twr.TextExtentX(dc, "00  00'00", glsize))/2);
		str.Printf("%2d ", chart[nch].planet[n-SUN].sgn.degrees);
		twr.TypeText(dc, str, col, glsize);
		twr.SetY(twr.GetY() - glsize);
		twr.TypeGlyph(dc, chart[nch].planet[n-SUN].sgn.sign, col, 2, 1);
		twr.SetY(twr.GetY() + glsize);
		sectemp = chart[nch].planet[n-SUN].sgn.seconds;
		round_secs = ((sectemp - floor(sectemp)) < 0.5) ?
			iround(sectemp) : iround((sectemp + 1.0));
		str.Printf(" %02d'%02d\"", chart[nch].planet[n-SUN].sgn.minutes, round_secs);
		if((n != NNODE) && (chart[nch].planet[n-SUN].long_speed < 0.0))
			str += "r";
		else if((n == NNODE) && (chart[nch].planet[n-SUN].long_speed >= 0.0))
			str += "d";
		twr.TypeText(dc, str, col, glsize);
		twr.NewLine(3*glsize);
	}
	else if(IS_HOUSE(n))
	{
		col = DefaultObjectColour(trGetElement(chart[nch].house[n-HOUSE1+1].sgn.sign), 255);
		twr.SetX((w - 2*glsize - twr.TextExtentX(dc, "00  00'00", glsize))/2);
		str.Printf("%2d ", chart[nch].house[n-HOUSE1+1].sgn.degrees);
		twr.TypeText(dc, str, col, glsize);
		twr.SetY(twr.GetY() - glsize);
		twr.TypeGlyph(dc, chart[nch].house[n-HOUSE1+1].sgn.sign, col, 2, 1);
		twr.SetY(twr.GetY() + glsize);
		sectemp = chart[nch].house[n-HOUSE1+1].sgn.seconds;
		round_secs = ((sectemp - floor(sectemp)) < 0.5) ?
			iround(sectemp) : iround(sectemp + 1.0);
		str.Printf(" %02d'%02d\"", chart[nch].house[n-HOUSE1+1].sgn.minutes, round_secs);
		twr.TypeText(dc, str, col, glsize);
		twr.NewLine(3*glsize);
	}

	if(IS_PLANETARY_BODY(n))
	{
		// Get the planet's position by house into DMS format
		sectemp = chart[nch].planet[n-SUN].longitude;
		icusp = -1;
		nch2 = (IS_ONE_CHART(viewmode)) ? nch :
		       ((nch == nchart1) ? nchart2 : nchart1);
		for(i = 1; i <= 12; i++)
		{
			c1 = chart[nch2].house[i].cusp;  c2 = chart[nch2].house[i+1].cusp;
			if(c1 < c2)
			{
				if((c1 <= sectemp) && (sectemp < c2))
				{
					icusp = i;
					break;
				}
			}
			else
			{
				if((c1 <= sectemp) || (sectemp < c2))
				{
					icusp = i;
					break;
				}
			}
		}
		if(icusp == -1)
		{
			wxMessageBox("Error!  Invalid House Cusp Position!");
			icusp = 1;
		}
		sectemp -= chart[nch2].house[icusp].cusp;
		if(sectemp < 0.0) sectemp += 360.0;
		split_dms(sectemp, &dms);
		round_secs = ((dms.sec - floor(dms.sec)) < 0.5) ?
			iround(dms.sec) : iround(dms.sec + 1.0);
		
		// Display its house location
		col = DefaultObjectColour(trGetElement(FIRST_HOUSE+icusp-1), 255);
		twr.SetX((w - 2*glsize - twr.TextExtentX(dc, "00  00'00", glsize))/2);
		str.Printf("%2d ", dms.deg);
		twr.TypeText(dc, str, col, glsize);
		twr.SetY(twr.GetY() - glsize);
		twr.TypeGlyph(dc, FIRST_HOUSE+icusp-1, col, 2, 1);
		twr.SetY(twr.GetY() + glsize);
		str.Printf(" %02d'%02d\"", dms.min, round_secs);
		twr.TypeText(dc, str, col, glsize);
		twr.NewLine(3*glsize);
		
		// Display the speed
		str.Printf("Speed: %.4f", chart[nch].planet[n-SUN].long_speed);
		twr.TypeText(dc, str, wxColour(128,128,128), glsize, FALSE, wxCENTER);
		twr.NewLine(4*glsize);
	}
	else // if it's a house cusp, not a planet(oid)
	{
		twr.NewLine(glsize);
	}

	// Display its dispositors, again by sign and house
	if(IS_PLANETARY_BODY(n))
	{
		twr.TypeText(dc, "Dispositors\n", *wxWHITE, 14, TRUE, wxCENTER, wxSWISS);
		twr.NewLine(glsize);

		for(i = 0; i < 2; i++)
		{
			if(i == 0)
				ndis = RulerOf(chart[nch].planet[n-SUN].sgn.sign);
			else
				ndis = RulerOf(chart[nch].planet[n-SUN].in_house-1+ARIES);
			if(ndis != n)
			{
				if(i == 0)
					str.Printf("/s2//g%d//s1/ in /s2//g%d/", ndis,
						chart[nch].planet[ndis-SUN].sgn.sign);
				else
					str.Printf("/s2//g%d//s1/ in /s2//g%d/", ndis,
						chart[nch].planet[ndis-SUN].in_house - 1 + FIRST_HOUSE );
				if(i == 0)
					str = "/s1/By Sign: " + str;
				else
					str = "/s1/By House: " + str;
				twr.TypeLine(dc, wxColour(128,128,128), 1, wxCENTER, TRUE, str);
				twr.NewLine(glsize/2);
			}
			else
			{
				str.Printf("/s2//g%d//s1/ is final", n);
				if(i == 0)
					str = "/s1/By Sign: " + str;
				else
					str = "/s1/By House: " + str;
				twr.TypeLine(dc, wxColour(128,128,128), 1, wxCENTER, TRUE, str);
				twr.NewLine(glsize/2);
			}
		}
		twr.NewLine(2*glsize);
	}

	// now, draw the aspects (left-justified) and their orbs (right-justified)
	DrawAspectInfo(dc, w, h, n, 0, FALSE);
	twr.NewLine(glsize);

	UpdateVScroll(twr.GetY());
}

void CChartInfo::DrawSignInfo(wxDC &dc, int w, int h, int n)
{
	int i, j, ihouse, ruler, nch, nch2, nn;
	wxString str;
	wxColour col;

	nch = nchartsel;
	nch2 = (IS_ONE_CHART(viewmode)) ? nch :
		   ((nch == nchart1) ? nchart2 : nchart1);

	twr.Reset(glsize, glsize, w, h);

	// Show a big glyph of the object
	col = DefaultObjectColour(trGetElement(n), 255);
	twr.TypeGlyph(dc, n, col, 4, 2, TRUE, TRUE);
	twr.TypeText(dc, trGetName(n), col, 12, TRUE, wxCENTER);
	twr.NewLine(3*glsize);

	// Display its planetary ruler
	ruler = RulerOf(n);
	col = DefaultObjectColour(trGetElement(ruler), 255);
	str.Printf("/s1/Ruled by /s2//g%d/", ruler);
	twr.TypeLine(dc, col, 1, wxCENTER, TRUE, str);
	twr.NewLine(4*glsize);

	// Display the contained planets
	for(j = 0; j < 2; j++)
	{
		if(j == 0)
			nn = nch;
		else if(IS_TWO_CHARTS(viewmode))
			nn = nch2;
		else
			break;

		for(i = 0; i < (NTOTOBJECTS + 12); i++)
		{
			if(restricted[nn][i]) continue;
			
			if((i < NTOTOBJECTS) &&
				((chart[nn].planet[i].longitude >= 30.0*(n-ARIES)) &&
				(chart[nn].planet[i].longitude <  30.0*(n-ARIES+1))))
			{
				if(IS_ONE_CHART(viewmode))
					str.Printf("/s2//g%d//s1/ in /s2//g%d/", FIRST_PLANET+i, n);
				else if(j == 0)
					str.Printf("/s2//g-//g%d//s1/ in /s2//g%d/", FIRST_PLANET+i, n);
				else
					str.Printf("/s2//g%d//g-//s1/ in /s2//g%d/", FIRST_PLANET+i, n);
				twr.TypeLine(dc, wxColour(128,128,128), 2, wxCENTER, TRUE, str);
				//twr.NewLine(glsize/2);
			}
			else if((i >= NTOTOBJECTS) &&
				((chart[nn].house[i-NTOTOBJECTS+1].cusp >= 30.0*(n-ARIES)) &&
				(chart[nn].house[i-NTOTOBJECTS+1].cusp <  30.0*(n-ARIES+1))))
			{
				ihouse = i - NTOTOBJECTS;
				if(IS_ONE_CHART(viewmode))
					str.Printf("/s2//g%d//s1/ in /s2//g%d/", FIRST_HOUSE+ihouse, n);
				else if(j == 0)
					str.Printf("/s2//g-//g%d//s1/ in /s2//g%d/", FIRST_HOUSE+ihouse, n);
				else
					str.Printf("/s2//g%d//g-//s1/ in /s2//g%d/", FIRST_HOUSE+ihouse, n);
				twr.TypeLine(dc, wxColour(128,128,128), 2, wxCENTER, TRUE, str);
				//twr.NewLine(glsize/2);
			}
		}
	}

	twr.NewLine(glsize);

	UpdateVScroll(twr.GetY());
}

void CChartInfo::DrawHouseInfo(wxDC &dc, int w, int h, int n)
{
	int i, j, nn, nch, nch2, ruler, round_secs;
	double sectemp;
	wxString str, hname;
	wxColour col;

	nch = nchartsel;
	nch2 = (IS_ONE_CHART(viewmode)) ? nch :
		   ((nch == nchart1) ? nchart2 : nchart1);

	twr.Reset(10, 10, w, h);
	
	switch(n - FIRST_HOUSE + 1)
	{
	case 1:  hname = _("First");    break;
	case 2:  hname = _("Second");   break;
	case 3:  hname = _("Third");    break;
	case 4:  hname = _("Fourth");   break;
	case 5:  hname = _("Fifth");    break;
	case 6:  hname = _("Sixth");    break;
	case 7:  hname = _("Seventh");  break;
	case 8:  hname = _("Eigth");    break;
	case 9:  hname = _("Ninth");    break;
	case 10: hname = _("Tenth");    break;
	case 11: hname = _("Eleventh"); break;
	case 12: hname = _("Twelfth");  break;
	default: hname = _("Invalid");
	}
				
	// Show a big glyph of the house
	col = DefaultObjectColour(trGetElement(n), 255);
	twr.TypeGlyph(dc, n, col, 4, 2, TRUE, TRUE);

	// Give the name of the house
	if(IS_TWO_CHARTS(viewmode))
	{
		if(nch == nchart1)
			str = "Native's\n";
		else if(viewmode & TRANSITS_CHART)
			str = "Transit\n";
		else
			str = "Partner's\n";
		twr.TypeText(dc, str, col, 12, TRUE, wxCENTER);
	}
	str = hname + " House";
	twr.TypeText(dc, str, col, 12, true, wxCENTER);
	twr.NewLine(3*glsize);

	// Display its planetary rulers; first natural, than natal
	ruler = RulerOf(ARIES + (n - FIRST_HOUSE));
	str.Printf("/s1/Natural Ruler: /s2//g%d/", ruler);
	twr.TypeLine(dc, col, 1, wxCENTER, TRUE, str);

	ruler = RulerOf(chart[nch].house[n-HOUSE1+1].sgn.sign);
	str.Printf("/s1/Natal Ruler: /s2//g%d/", ruler);
	twr.TypeLine(dc, col, 1, wxCENTER, TRUE, str);
	twr.NewLine(2*glsize);

	// Display its location
	twr.SetX((w - 2*glsize - twr.TextExtentX(dc, "From 00  00'00", glsize))/2);
	col = DefaultObjectColour(trGetElement(n), 255);
	twr.TypeText(dc, "From ", col, glsize);
	col = DefaultObjectColour(trGetElement(chart[nch].house[n-HOUSE1+1].sgn.sign), 255);
	str.Printf("%2d ", chart[nch].house[n-HOUSE1+1].sgn.degrees);
	twr.TypeText(dc, str, col, glsize);
	twr.SetY(twr.GetY() - glsize);
	twr.TypeGlyph(dc, chart[nch].house[n-HOUSE1+1].sgn.sign, col, 2, 1);
	twr.SetY(twr.GetY() + glsize);
	sectemp = chart[nch].house[n-HOUSE1+1].sgn.seconds;
	round_secs = ((sectemp - floor(sectemp)) < 0.5) ?
		iround(sectemp) : iround(sectemp + 1.0);
	str.Printf(" %02d'%02d\"", chart[nch].house[n-HOUSE1+1].sgn.minutes, round_secs);
	twr.TypeText(dc, str, col, glsize);
	twr.NewLine(3*glsize);

	twr.SetX((w - 2*glsize - twr.TextExtentX(dc, "  To 00  00'00", glsize))/2);
	col = DefaultObjectColour(trGetElement(n), 255);
	twr.TypeText(dc, "  To ", col, glsize);
	col = DefaultObjectColour(trGetElement(chart[nch].house[n-HOUSE1+2].sgn.sign), 255);
	str.Printf("%2d ", chart[nch].house[n-HOUSE1+2].sgn.degrees);
	twr.TypeText(dc, str, col, glsize);
	twr.SetY(twr.GetY() - glsize);
	twr.TypeGlyph(dc, chart[nch].house[n-HOUSE1+2].sgn.sign, col, 2, 1);
	twr.SetY(twr.GetY() + glsize);
	sectemp = chart[nch].house[n-HOUSE1+2].sgn.seconds;
	round_secs = ((sectemp - floor(sectemp)) < 0.5) ?
		iround(sectemp) : iround(sectemp + 1.0);
	str.Printf(" %02d'%02d\"", chart[nch].house[n-HOUSE1+2].sgn.minutes, round_secs);
	twr.TypeText(dc, str, col, glsize);
	twr.NewLine(6*glsize);

	// Display the contained planets
	for(j = 0; j < 2; j++)
	{
		if(j == 0)
			nn = nch;
		else if(IS_TWO_CHARTS(viewmode))
			nn = nch2;
		else
			continue;

		for(i = 0; i < NTOTOBJECTS; i++)
		{
			if(restricted[nn][i]) continue;
			
			if((chart[nn].planet[i].longitude >= chart[nch].house[n-HOUSE1+1].cusp) &&
				(chart[nn].planet[i].longitude <  chart[nch].house[n-HOUSE1+2].cusp))
			{
				if(IS_ONE_CHART(viewmode))
					str.Printf("/s2//g%d//s1/ in /s2//g%d/", FIRST_PLANET+i, n);
				else if(j == 0)
					str.Printf("/s2//g-//g%d//s1/ in /s2//g%d/", FIRST_PLANET+i, n);
				else
					str.Printf("/s2//g%d//g-//s1/ in /s2//g%d/", FIRST_PLANET+i, n);
				twr.TypeLine(dc, wxColour(128,128,128), 2, wxCENTER, TRUE, str);
				//twr.NewLine(glsize/2);
			}
		}
	}

	twr.NewLine(glsize);

	UpdateVScroll(twr.GetY());
}

#define TRINF_TOARRNUM(n) (IS_HOUSE(n)) ? ((n) - FIRST_HOUSE + NTOTOBJECTS) : \
	((n) - FIRST_OBJECT)

void CChartInfo::DrawAspectInfo(wxDC &dc, int w, int h, int n, int flags,
								bool do_reset, int y1, int y2)
{
	int i, n1, n2, gh;
	int *ord = aspect_order;
	bool show_this_aspect;
	wxString str;
	wxColour cols[3];

	/* dc.SetBrush(wxBrush(wxColour(64,64,64), wxSOLID)); */
	/* dc.DrawRectangle(10, 10, w-20, h-20); */

	gh = 2*glsize + glsize/2;

	if(do_reset)
		twr.Reset(10, 10, w, h);

	// draw the aspects (left-justified) and their orbs (right-justified)
	if(flags & INFO_ASPECTS_RESULTS)
		str = "Aspects Found\n";
	else
		str = "Aspects\n";
	twr.TypeText(dc, str, *wxWHITE, 14, TRUE, wxCENTER, wxSWISS);
	twr.NewLine(glsize);

	for(i = 0; i < n_aspect_order; i++)
	{
		if(flags & INFO_ASPECTS_ALL)
			show_this_aspect = TRUE;
		else if(flags & INFO_ASPECTS_RESULTS) // Display search results
		{
			if(find_need_both)
				show_this_aspect = (aspect_cache[ord[i]].orb >= find_orb_min) &&
					(aspect_cache[ord[i]].orb <= find_orb_max) &&
					(find_aspects[aspect_cache[ord[i]].n_aspect - FIRST_ASPECT]) &&
					(find_planet1[TRINF_TOARRNUM(aspect_cache[ord[i]].n1)]) &&
					(find_planet2[TRINF_TOARRNUM(aspect_cache[ord[i]].n2)]);
			else
				show_this_aspect = (aspect_cache[ord[i]].orb >= find_orb_min) &&
					(aspect_cache[ord[i]].orb <= find_orb_max) &&
					(find_aspects[aspect_cache[ord[i]].n_aspect - FIRST_ASPECT]) &&
					((find_planet1[TRINF_TOARRNUM(aspect_cache[ord[i]].n1)]) ||
					(find_planet2[TRINF_TOARRNUM(aspect_cache[ord[i]].n2)]));
		}
		else
			show_this_aspect =
				((n >= 0) && ((aspect_cache[ord[i]].n1 == n) || (aspect_cache[ord[i]].n2 == n))) ||
				((n < 0) && (aspect_over[ord[i]]));
		if(show_this_aspect && (n >= 0) && (IS_TWO_CHARTS(viewmode)))
		{
			if(nchartsel == nchart1)
				show_this_aspect = (aspect_cache[ord[i]].n1 == n);
			else
				show_this_aspect = (aspect_cache[ord[i]].n2 == n);
		}
		if(show_this_aspect && (y1 >= 0) && (y2 >= 0))
		{
			show_this_aspect = (twr.GetY() >= (y1 - gh)) && (twr.GetY() <= (y2 + gh));
			if(!show_this_aspect)
				twr.NewLine(gh);
		}
		if(show_this_aspect)
		{
			get_swapped_aspects(ord[i], n, &n1, &n2);
			if(aspect_cache[ord[i]].n_aspect != DISPOSITOR_ASPECT)
			{
				if(!(viewmode & TRANSITS_CHART))
					str.Printf("/s2//g%d//g%d//g%d//r//s1/Orb: %4.2f\xb0",
						n1, aspect_cache[ord[i]].n_aspect, n2, aspect_cache[ord[i]].orb);
				else // for transit charts, reverse the order of planets displayed
					str.Printf("/s2//g%d//g%d//g%d//r//s1/Orb: %4.2f\xb0",
						n2, aspect_cache[ord[i]].n_aspect, n1, aspect_cache[ord[i]].orb);
				twr.TypeLine(dc, *wxWHITE, 2, wxLEFT, TRUE, str);
			}
			else
			{
				str.Printf("/s2//g%d//g%d//g%d/",
					n1, aspect_cache[ord[i]].n_aspect, n2);
				twr.TypeLine(dc, *wxWHITE, 2, wxCENTER, TRUE, str);
			}
		}
	}

	// Iff we reset at the beginning, we have full control over the panel
	// contents, so set the scrollbars as well
	if(do_reset)
	{
		twr.NewLine(glsize);
		UpdateVScroll(twr.GetY());
	}
}

void CChartInfo::sort_aspects(int n, int flags, int *ord, int n_ord)
{
#ifndef __WXDEBUG__
	sort_aspects_heapsort(n, flags, ord, n_ord);
#else
	sort_aspects_insertion(n, flags, ord, n_ord);
	sort_aspects_heapsort(n, flags, ord, n_ord);
	//sort_aspects_quicksort(n, flags, ord, n_ord);
#endif
}

#ifndef inline
#define inline
#endif

inline void CChartInfo::get_swapped_aspects(int i, int n, int *n1, int *n2)
{
	bool swap_order;
	if((aspect_cache[i].nchart1 == aspect_cache[i].nchart2) &&
		(aspect_cache[i].n_aspect != DISPOSITOR_ASPECT))
	{
		if(n >= 0)
			swap_order = (aspect_cache[i].n1 != n);
		else
			swap_order = (aspect_cache[i].n1 > aspect_cache[i].n2);
	}
	else
	{
		swap_order = false;
	}
	*n1 = swap_order ? aspect_cache[i].n2 : aspect_cache[i].n1;
	*n2 = swap_order ? aspect_cache[i].n1 : aspect_cache[i].n2;
}

static inline void correct_for_houses(int *n1a, int *n1b, int *n2a, int *n2b)
{
	if(IS_HOUSE(*n1a)) (*n1a) = (*n1a) - FIRST_HOUSE + FIRST_PLANET + NTOTOBJECTS;
	if(IS_HOUSE(*n2a)) (*n2a) = (*n2a) - FIRST_HOUSE + FIRST_PLANET + NTOTOBJECTS;
	if(IS_HOUSE(*n1b)) (*n1b) = (*n1b) - FIRST_HOUSE + FIRST_PLANET + NTOTOBJECTS;
	if(IS_HOUSE(*n2b)) (*n2b) = (*n2b) - FIRST_HOUSE + FIRST_PLANET + NTOTOBJECTS;
}

#ifdef __WXDEBUG__
#define NCOMPSPTR , &n_comparisons
#else
#define NCOMPSPTR
#endif

// Returns whether aspects i and j should be swapped or not,
// with n1a, n1b, etc... being the planets/objects involved
static inline bool compare_aspects_lt(int *ord, aspect_storage *aspect_cache,
                                     int n1a, int n1b, int n2a, int n2b,
                                     int i, int j, int viewmode, int flags
#ifdef __WXDEBUG__
									 , int *n_comparisons
#endif
									 )
{
#ifdef __WXDEBUG__
	(*n_comparisons)++;
#endif
	if(flags & INFO_ASPECTS_BYORB)
		return aspect_cache[i].orb < aspect_cache[j].orb;
	else if(!(viewmode & TRANSITS_CHART) && (flags & INFO_ASPECTS_BYPL1))
		return (n1a < n1b) || ((n1a == n1b) && (n2a < n2b));
	else if(!(viewmode & TRANSITS_CHART) && (flags & INFO_ASPECTS_BYPL2))
		return (n2a < n2b) || ((n2a == n2b) && (n1a < n1b));
	else if((viewmode & TRANSITS_CHART) && (flags & INFO_ASPECTS_BYPL1))
		return (n2a < n2b) || ((n2a == n2b) && (n1a < n1b));
	else if((viewmode & TRANSITS_CHART) && (flags & INFO_ASPECTS_BYPL2))
		return (n1a < n1b) || ((n1a == n1b) && (n2a < n2b));
	else
		return false;
}

void CChartInfo::sort_aspects_insertion(int n, int flags, int *ord, int n_ord)
{
	int i, j, tmp, n1a, n1b, n2a, n2b;
#ifdef __WXDEBUG__
	int ncompare, nswap;
	wxString s;
	ncompare = nswap = 0;
#endif

	if((n < 0) && ((flags & INFO_ASPECTS_BYORB) ||
	   (flags & INFO_ASPECTS_BYPL1) || (flags & INFO_ASPECTS_BYPL2)))
	{
		for(i = 0; i < n_ord; i++)
		{
			ord[i] = i;
			for(j = i; j > 0; j--)
			{
				// Correct for transit charts, which have the order
				// of their planets reversed
				get_swapped_aspects(ord[j],     n, &n1a, &n2a);
				get_swapped_aspects(ord[j - 1], n, &n1b, &n2b);
				correct_for_houses(&n1a, &n1b, &n2a, &n2b);
#ifdef __WXDEBUG__
				ncompare++;
#endif
				if(compare_aspects_lt(ord, aspect_cache, n1a, n1b, n2a, n2b,
					ord[j], ord[j - 1], viewmode, flags NCOMPSPTR))
				{
#ifdef __WXDEBUG__
					nswap++;
#endif
					tmp = ord[j - 1];
					ord[j - 1] = ord[j];
					ord[j] = tmp;
				}
			}
		}
	}
	else
	{
		for(i = 0; i < n_ord; i++)
			ord[i] = i;
	}
#ifdef __WXDEBUG__
	s.Printf("insertion sort:\nn_ord = %d\nncompare = %d\nnswap = %d", n_ord, ncompare, nswap);
	if((ncompare > 0) || (nswap > 0))
		wxMessageBox(s);
#endif
}

inline int CChartInfo::heapsort_siftdown(int n, int flags, int *ord,
										  int n_ord, int root, int bottom)
{
	int n1a, n2a, n1b, n2b;
	int done, max_child, tempnum;
	
	done = 0;
	while(((2*root) <= bottom) && (!done))
	{
		if((2*root) == bottom)
			max_child = 2*root;
		else
		{
			get_swapped_aspects(ord[2*root + 1], n, &n1a, &n2a);
			get_swapped_aspects(ord[2*root], n, &n1b, &n2b);
			correct_for_houses(&n1a, &n1b, &n2a, &n2b);
			if(compare_aspects_lt(ord, aspect_cache, n1a, n1b, n2a, n2b,
			                      ord[2*root + 1], ord[2*root], viewmode, flags NCOMPSPTR))
				max_child = 2*root;
			else
				max_child = 2*root + 1;
		}
		get_swapped_aspects(ord[root], n, &n1a, &n2a);
		get_swapped_aspects(ord[max_child], n, &n1b, &n2b);
		correct_for_houses(&n1a, &n1b, &n2a, &n2b);
		if(compare_aspects_lt(ord, aspect_cache, n1a, n1b, n2a, n2b,
			                  ord[root], ord[max_child], viewmode, flags NCOMPSPTR))
		{
#ifdef __WXDEBUG__
			n_swaps++;
#endif
			tempnum = ord[root];
			ord[root] = ord[max_child];
			ord[max_child] = tempnum;
			root = max_child;
		}
		else
			done = 1;
	}

#ifdef __WXDEBUG__
	return n_comparisons;
#else
	return 0;
#endif
}

void CChartInfo::sort_aspects_heapsort(int n, int flags, int *ord, int n_ord)
{
	int i, tempnum;
#ifdef __WXDEBUG__
	wxString s;
	n_comparisons = n_swaps = 0;
#endif

	if((n < 0) && ((flags & INFO_ASPECTS_BYORB) ||
	   (flags & INFO_ASPECTS_BYPL1) || (flags & INFO_ASPECTS_BYPL2)))
	{
		for(i = 0; i < n_ord; i++)
			ord[i] = i;
		
		for(i = (n_ord/2) - 1; i >= 0; i--)
		{
			wxASSERT((i >= 0) && (2*i < n_ord));
			heapsort_siftdown(n, flags, ord, n_ord, i, n_ord - 1);
		}

		for(i = n_ord - 1; i >= 1; i--)
		{
#ifdef __WXDEBUG__
			n_swaps++;
#endif
			tempnum = ord[0];
			ord[0] = ord[i];
			ord[i] = tempnum;
			heapsort_siftdown(n, flags, ord, n_ord, 0, i - 1);
		}
	}
	else
	{
		for(i = 0; i < n_ord; i++)
			ord[i] = i;
	}
#ifdef __WXDEBUG__
	s.Printf("heap sort:\nn_ord = %d\nncompare = %d\nnswap = %d", n_ord, n_comparisons, n_swaps);
	if((n_comparisons > 0) || (n_swaps > 0))
		wxMessageBox(s);
#endif
}

int CChartInfo::quicksort_recurse(int n, int flags, int *ord, int n_ord,
								  int left, int right)
{
	int pivot, l_hold, r_hold;
	int n1a, n2a, n1b, n2b;
	
	l_hold = left;
	r_hold = right;
	// ASSIGN PIVOT
	// used to be: pivot = numbers[left]
	// should:
	//   in debug mode, pick left, or average of three
	//   in release mode, use random pivot (?)
	pivot = ord[left]; //TEMP; DEBUG!!!!
	// Also To Do:
	//     Find out what the code inside the 'if's is
	//     doing, and rewrite it to work here
#ifdef __WXDEBUG__
#else
#endif
	while (left < right)
	{
		get_swapped_aspects(ord[pivot], n, &n1a, &n2a);
		get_swapped_aspects(ord[right], n, &n1b, &n2b);
		correct_for_houses(&n1a, &n1b, &n2a, &n2b);
		while((left < right) && compare_aspects_lt(ord, aspect_cache,
				n1a, n1b, n2a, n2b, ord[pivot], ord[right], viewmode,
				flags NCOMPSPTR))
		{
			right--;
			
			get_swapped_aspects(ord[pivot], n, &n1a, &n2a);
			get_swapped_aspects(ord[right], n, &n1b, &n2b);
			correct_for_houses(&n1a, &n1b, &n2a, &n2b);
		}
		if (left != right)
		{
#ifdef __WXDEBUG__
			n_swaps++;
#endif
			ord[left] = ord[right];
			left++;
		}
		get_swapped_aspects(ord[left], n, &n1a, &n2a);
		get_swapped_aspects(ord[pivot], n, &n1b, &n2b);
		correct_for_houses(&n1a, &n1b, &n2a, &n2b);
		while((left < right) && compare_aspects_lt(ord, aspect_cache,
				n1a, n1b, n2a, n2b, ord[left], ord[pivot], viewmode,
				flags NCOMPSPTR))
		{
			left++;

			get_swapped_aspects(ord[left], n, &n1a, &n2a);
			get_swapped_aspects(ord[pivot], n, &n1b, &n2b);
			correct_for_houses(&n1a, &n1b, &n2a, &n2b);
		}
		if (left != right)
		{
#ifdef __WXDEBUG__
			n_swaps++;
#endif
			ord[right] = ord[left];
			ord--;
		}
	}
#ifdef __WXDEBUG__
	n_swaps++;
#endif
	ord[left] = pivot;
	pivot = left;
	left = l_hold;
	right = r_hold;
	if (left < pivot)
		quicksort_recurse(n, flags, ord, n_ord, left, pivot - 1);
	if (right > pivot)
		quicksort_recurse(n, flags, ord, n_ord, pivot + 1, right);
#ifdef __WXDEBUG__
	return n_comparisons;
#else
	return 0;
#endif
}

void CChartInfo::sort_aspects_quicksort(int n, int flags, int *ord, int n_ord)
{
	int i;
#ifdef __WXDEBUG__
	wxString s;
	n_comparisons = n_swaps = 0;
#endif

	if((n < 0) && ((flags & INFO_ASPECTS_BYORB) ||
	   (flags & INFO_ASPECTS_BYPL1) || (flags & INFO_ASPECTS_BYPL2)))
	{
		quicksort_recurse(n, flags, ord, n_ord, 0, n_ord - 1);
	}
	else
	{
		for(i = 0; i < n_ord; i++)
			ord[i] = i;
	}
#ifdef __WXDEBUG__
	s.Printf("quick sort:\nn_ord = %d\nncompare = %d\nnswap = %d",
		n_ord, n_comparisons, n_swaps);
	if((n_comparisons > 0) || (n_swaps > 0))
		wxMessageBox(s);
#endif
}
