//TransitsInfo.h
//
//Copyright (C) 2003 mikesag
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//This is the information panel to the side of the chart

#ifndef TransitsInfo_H
#define TransitsInfo_H

#include "TransitsChart.h"
#include "Typewriter.h"

#define INFO_MODE_SUMMARY 0
#define INFO_MODE_SIGN    1
#define INFO_MODE_HOUSE   2
#define INFO_MODE_PLANET  3

#define INFO_MODE_ASPECTS 4 /* works for both natal and synasty */

#define INFO_MODE_SYNSUMMARY 5
#define INFO_MODE_SYNSIGN    6
#define INFO_MODE_SYNHOUSE   7

#define INFO_ASPECTS_UNSORTED  0
#define INFO_ASPECTS_BYORB     1
#define INFO_ASPECTS_BYPL1     2
#define INFO_ASPECTS_BYPL2     4

#define INFO_ASPECTS_SELECTED 0
#define INFO_ASPECTS_ALL      8
#define INFO_ASPECTS_RESULTS  16

#define MAX_ASPECT_CACHE ((NTOTOBJECTS+12)*(NTOTOBJECTS+12))

#ifndef CChartCanvas
class CChartCanvas;
#endif

class CChartInfo : public wxScrolledWindow
{
public:
	CChartInfo(wxFrame *parent, CChartCanvas *chartcanvas);

	void SetMode(int nmode, int nobj, int nchsel);
	void SetViewMode(int newmode);
	void SetAspectsMode(int amode, bool do_refresh = TRUE);
	void SetChartSel(int nch, int nch1, int nch2);
	int  GetMode();
	int  GetObj();
	int  CompareMode(int nmode, int nobj, int nchs);
	void SetAspects(int n_aspects, aspect_storage cache[MAX_ASPECT_CACHE]);
	void SetAspectOver(int n_over, bool over[MAX_ASPECT_CACHE]);
	void SetChartData(trChartInfo ci[NCHARTTYPES], trTimeDatePlace cd[NCHARTTYPES],
	                  trChartDataExt ce[NCHARTTYPES]);
	void SetRestrictions(bool restr[NCHARTTYPES][NTOTOBJECTS+12], bool rasps[NASPECTS],
		bool do_refresh = TRUE);
	void SetFindParams(bool find_pl1[NTOTOBJECTS+12], bool find_pl2[NTOTOBJECTS+12],
		bool find_asps[NASPECTS], bool need_both, double min_orb, double max_orb);

	void UpdateVScroll(int newscrollheight);

	void OnResize(wxSizeEvent &event);
	void OnPaint(wxPaintEvent &event);
	void OnMouseDown(wxMouseEvent &event);

protected:
	trChartInfo chart[NCHARTTYPES];
	trTimeDatePlace chartdata[NCHARTTYPES];
	trChartDataExt chartdataext[NCHARTTYPES];
	aspect_def aspects[NASPECTS];

	void DrawChartSummary(wxDC &dc, int w, int h);
	void DrawAspectInfo(wxDC &dc, int w, int h, int n, int flags, bool do_reset,
		int y1 = -1, int y2 = -1);
	void DrawPlanetInfo(wxDC &dc, int w, int h, int n);
	void DrawHouseInfo(wxDC &dc, int w, int h, int n);
	void DrawSignInfo(wxDC &dc, int w, int h, int n);

private:
	wxFrame *owner;
	CChartCanvas *chartdrawing;

	CTypewriterDraw twr;

	void get_swapped_aspects(int i, int n, int *n1, int *n2);
	void sort_aspects(int n, int flags, int *ord, int n_ord);
	void sort_aspects_insertion(int n, int flags, int *ord, int n_ord);
	void sort_aspects_heapsort(int n, int flags, int *ord, int n_ord);
	void sort_aspects_quicksort(int n, int flags, int *ord, int n_ord);

#ifdef __WXDEBUG__
	int n_comparisons, n_swaps;
#endif
	int heapsort_siftdown(int n, int flags, int *ord, int n_ord,
		int root, int bottom);
	int quicksort_recurse(int n, int flags, int *ord, int n_ord,
		int left, int right);

	int nchart, nchart1, nchart2, nchartsel;
	int selected;
	int mode, viewmode, aspmode;
	int glsize;

	int n_aspect_cache;
	aspect_storage aspect_cache[MAX_ASPECT_CACHE];

	int n_aspects_over;
	bool aspect_over[MAX_ASPECT_CACHE];

	int n_aspect_order;
	int aspect_order[MAX_ASPECT_CACHE];

	bool restricted[NCHARTTYPES][NTOTOBJECTS+12];
	bool r_aspects[NASPECTS];

	bool find_planet1[NTOTOBJECTS+12];
	bool find_planet2[NTOTOBJECTS+12];
	bool find_aspects[NASPECTS];
	bool find_need_both;
	double find_orb_min, find_orb_max;

	DECLARE_CLASS(CChartInfo)
	DECLARE_EVENT_TABLE()
};

#endif
