//Translation.cpp: Translates among various constants, and defines some of
//                 them itself.  This is where long, tedious switch()
//                 statements and long if()/else if() chains should go.
//
//Copyright (C) 2003 mikesag
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// Translation.cpp

#include "Translation.h"
#include "AstroConst.h"
#include "AstroCalc.h"

#include "sweph/swephexp.h"
#include "sweph/sweph.h"

int IntToHouse(int n)
{
	return FIRST_HOUSE + n - 1;
}

int HouseToInt(int h)
{
	return h - FIRST_HOUSE + 1;
}

int IntToSign(int n)
{
	return FIRST_SIGN + n - 1;
}

int SignToInt(int h)
{
	return h - FIRST_SIGN + 1;
}

int ToSwissEph(int n)
{
	switch(n)
	{
        case ARIES: return 1;
        case TAURUS: return 2;
        case GEMINI: return 3;
        case CANCER: return 4;
        case LEO: return 5;
        case VIRGO: return 6;
        case LIBRA: return 7;
        case SCORPIO: return 8;
        case SAGITTARIUS: return 9;
        case CAPRICORN: return 10;
        case AQUARIUS: return 11;
        case PISCES: return 12;
        case HOUSE1: return 1;
        case HOUSE2: return 2;
        case HOUSE3: return 3;
        case HOUSE4: return 4;
        case HOUSE5: return 5;
        case HOUSE6: return 6;
        case HOUSE7: return 7;
        case HOUSE8: return 8;
        case HOUSE9: return 9;
        case HOUSE10: return 10;
        case HOUSE11: return 11;
        case HOUSE12: return 12;
        case SUN: return SE_SUN;
        case MOON: return SE_MOON;
        case MERCURY: return SE_MERCURY;
        case VENUS: return SE_VENUS;
        case MARS: return SE_MARS;
        case JUPITER: return SE_JUPITER;
        case SATURN: return SE_SATURN;
        case URANUS: return SE_URANUS;
        case NEPTUNE: return SE_NEPTUNE;
        case PLUTO: return SE_PLUTO;
        case CHIRON: return SE_CHIRON;
        case LILITH: return SE_MEAN_APOG;
        case CERES: return SE_CERES;
        case PALLAS: return SE_PALLAS;
        case JUNO: return SE_JUNO;
        case VESTA: return SE_VESTA;
        case NNODE: return SE_TRUE_NODE;
        case SNODE: return -1;
        case PFORTUNE: return -1;
        case VERTEX: return SE_VERTEX;
        case EPOINT: return -1;
		case EARTH: return SE_EARTH;
		case ECLIPTIC_OBLIQUITY: return SE_ECL_NUT;
        case MEAN_NODE: return SE_MEAN_NODE;
        case TRUE_NODE: return SE_TRUE_NODE;
        case MEAN_APOG: return SE_MEAN_APOG;
        case OSCU_APOG: return SE_OSCU_APOG;
        case U_CUPIDO: return SE_CUPIDO;
        case U_HADES: return SE_HADES;
        case U_ZEUS: return SE_ZEUS;
        case U_KRONOS: return SE_KRONOS;
        case U_APOLLON: return SE_APOLLON;
        case U_ADMETOS: return SE_ADMETOS;
        case U_VULKANUS: return SE_VULKANUS;
        case U_POSEIDON: return SE_POSEIDON;
        case U_PROSERPINA: return SE_PROSERPINA;

		// Remember to convert back to char before passing it to sweph!
		case PLACIDUS: return int('P');
        case KOCH: return int('K');
        case EQUAL: return int('E');
		case NULL_HOUSES: return int('E'); //Not strictly true, but SE
        case CAMPANUS: return int('C');    //  doesn't do null houses
        case HORIZON: return int('H');
        case PORPHYRY: return int('O');
        case REGIOMONTANUS: return int('R');
        case VEHLOW: return int('V');
        case MERIDIAN: return int('X');
		case ALCABITIUS: return int('B');
		case TOPOCENTRIC: return int('T');
		case MORINUS: return int('M');
        case GAUQUELIN: return int('G');
	}
	return (-1);
}

int FromSwissEph(int n)
{
	switch(n)
	{
        case SE_SUN: return SUN;
        case SE_MOON: return MOON;
        case SE_MERCURY: return MERCURY;
        case SE_VENUS: return VENUS;
        case SE_MARS: return MARS;
        case SE_JUPITER: return JUPITER;
        case SE_SATURN: return SATURN;
        case SE_URANUS: return URANUS;
        case SE_NEPTUNE: return NEPTUNE;
        case SE_PLUTO: return PLUTO;
        case SE_CHIRON: return CHIRON;
        case SE_CERES: return CERES;
        case SE_PALLAS: return PALLAS;
        case SE_JUNO: return JUNO;
        case SE_VESTA: return VESTA;
		case SE_MEAN_NODE: return MEAN_NODE;
        case SE_TRUE_NODE: return NNODE;
		case SE_MEAN_APOG: return LILITH;
		case SE_OSCU_APOG: return OSCU_APOG;
		case SE_EARTH: return EARTH;
		case SE_ECL_NUT: return ECLIPTIC_OBLIQUITY;
        case SE_CUPIDO: return U_CUPIDO;
        case SE_HADES: return U_HADES;
        case SE_ZEUS: return U_ZEUS;
        case SE_KRONOS: return U_KRONOS;
        case SE_APOLLON: return U_APOLLON;
        case SE_ADMETOS: return U_ADMETOS;
        case SE_VULKANUS: return U_VULKANUS;
        case SE_POSEIDON: return U_POSEIDON;
        case SE_PROSERPINA: return U_PROSERPINA;
	}
	return (-1);
}

int FromSwissEphHouseSys(char c)
{
	switch(c)
	{
	    case 'P': return PLACIDUS;
        case 'K': return KOCH;
        case 'E': return EQUAL;
        case 'C': return CAMPANUS;
        case 'H': return HORIZON;
        case 'O': return PORPHYRY;
        case 'R': return REGIOMONTANUS;
        case 'V': return VEHLOW;
        case 'X': return MERIDIAN;
		case 'B': return ALCABITIUS;
		case 'T': return TOPOCENTRIC;
		case 'M': return MORINUS;
        case 'G': return GAUQUELIN;
	}
	
	return -1;
}

const char *trGetName(int n)
{
	switch(n)
	{
		case ARIES: return "Aries";
		case TAURUS: return "Taurus";
		case GEMINI: return "Gemini";
		case CANCER: return "Cancer";
		case LEO: return "Leo";
		case VIRGO: return "Virgo";
		case LIBRA: return "Libra";
		case SCORPIO: return "Scorpio";
		case SAGITTARIUS: return "Sagittarius";
		case CAPRICORN: return "Capricorn";
		case AQUARIUS: return "Aquarius";
		case PISCES: return "Pisces";

		case HOUSE1: return "Ascendant";
		case HOUSE2: return "Second House";
		case HOUSE3: return "Third House";
		case HOUSE4: return "Nadir";
		case HOUSE5: return "Fifth House";
		case HOUSE6: return "Sixth House";
		case HOUSE7: return "Descendant";
		case HOUSE8: return "Eigth House";
		case HOUSE9: return "Ninth House";
		case HOUSE10: return "Midheaven";
		case HOUSE11: return "Eleventh House";
		case HOUSE12: return "Twelfth House";
		case PLACIDUS: return "Placidus";

		case KOCH: return "Koch";
		case EQUAL: return "Equal";
		case NULL_HOUSES: return "Null";
		case CAMPANUS: return "Campanus";
		case HORIZON: return "Horizon";
		case PORPHYRY: return "Porphyry";
		case REGIOMONTANUS: return "Regiomontanus";
		case VEHLOW: return "Vehlow";
		case MERIDIAN: return "Meridian";
		case GAUQUELIN: return "Gauquelin";
		case ALCABITIUS: return "Alcabitius";
		case TOPOCENTRIC: return "Topocentric";

		case MORINUS: return "Morinus";
		case SUN: return "Sun";
		case MOON: return "Moon";
		case MERCURY: return "Mercury";
		case VENUS: return "Venus";
		case MARS: return "Mars";
		case JUPITER: return "Jupiter";
		case SATURN: return "Saturn";
		case URANUS: return "Uranus";
		case NEPTUNE: return "Neptune";
		case PLUTO: return "Pluto";
		case CHIRON: return "Chiron";
		case LILITH: return "Lilith";
		case CERES: return "Ceres";
		case PALLAS: return "Pallas";
		case JUNO: return "Juno";
		case VESTA: return "Vesta";
		case PHOLUS: return "Pholus";
		case NNODE: return "North Node";
		case SNODE: return "South Node";
		case PFORTUNE: return "Part of Fortune";
		case VERTEX: return "Vertex";
		case EPOINT: return "East Point";
		case EARTH: return "Earth";
		case ECLIPTIC_OBLIQUITY: return "Ecliptic Obliquity";
		case MEAN_NODE: return "Mean Node";
		case TRUE_NODE: return "True Node";
		case MEAN_APOG: return "Mean Apogee";
		case OSCU_APOG: return "Oscillating Apogee";
		case U_CUPIDO: return "Cupido";
		case U_HADES: return "Hades";
		case U_ZEUS: return "Zeus";
		case U_KRONOS: return "Kronos";
		case U_APOLLON: return "Apollon";
		case U_ADMETOS: return "Admetos";
		case U_VULKANUS: return "Vulkanus";
		case U_POSEIDON: return "Poseidon";
		case U_PROSERPINA: return "Proserpina";

		case CONJUNCTION: return "Conjunction";
		case OPPOSITION: return "Opposition";
		case SQUARE: return "Square";
		case TRINE: return "Trine";
		case SEXTILE: return "Sextile";
		case INCONJUNCT: return "Inconjunct";
		case SEMISEXTILE: return "Semisextile";
		case SEMISQUARE: return "Semisquare";
		case SESQUIQUADRATURE: return "Sesquiquadrature";
		case QUINTILE: return "Quintile";
		case BIQUINTILE: return "Biquintile";
		case SEMIQUINTILE: return "Semiquintile";
		case SEPTILE: return "Septile";
		case NOVILE: return "Novile";
		case BINOVILE: return "Binovile";
		case BISEPTILE: return "Biseptile";
		case TRISEPTILE: return "Triseptile";
		case QUATRONOVILE: return "Quatronovile";
		case PARALLEL: return "Parallel";
		case CONTRAPARALLEL: return "Contraparallel";
		case DISPOSITOR_ASPECT: return "Disposits";

		case STELLIUM: return "Stellium";
		case GRAND_TRINE: return "Grand Trine";
		case T_SQUARE: return "T-Square";
		case YOD_CONF: return "Yod";
		case KITE_CONF: return "Kite";
		case GRAND_CROSS: return "Grand Cross";
		case MYSTIC_RECT: return "Mystic Rectangle";
		case BOOMERANG: return "Boomerang";
		case GR_QUINTILE: return "Grand Quintile";
		case GR_SEXTILE: return "Grand Sextile";

		case JANUARY: return "January";
		case FEBRUARY: return "February";
		case MARCH: return "March";
		case APRIL: return "April";
		case MAY: return "May";
		case JUNE: return "June";
		case JULY: return "July";
		case AUGUST: return "August";
		case SEPTEMBER: return "September";
		case OCTOBER: return "October";
		case NOVEMBER: return "November";
		case DECEMBER: return "December";
	}
	return "";
}

const char *trGetAbbrevName(int n)
{
	switch(n)
	{
		case ARIES: return "Ari";
		case TAURUS: return "Tau";
		case GEMINI: return "Gem";
		case CANCER: return "Can";
		case LEO: return "Leo";
		case VIRGO: return "Vir";
		case LIBRA: return "Lib";
		case SCORPIO: return "Sco";
		case SAGITTARIUS: return "Sag";
		case CAPRICORN: return "Cap";
		case AQUARIUS: return "Aqu";
		case PISCES: return "Pis";

		case HOUSE1: return "Asc";
		case HOUSE2: return "2nd";
		case HOUSE3: return "3rd";
		case HOUSE4: return "Nad";
		case HOUSE5: return "5th";
		case HOUSE6: return "6th";
		case HOUSE7: return "Des";
		case HOUSE8: return "8th";
		case HOUSE9: return "9th";
		case HOUSE10: return "Mid";
		case HOUSE11: return "11h";
		case HOUSE12: return "12h";

		case PLACIDUS: return "Pla";
		case KOCH: return "Koc";
		case EQUAL: return "Equ";
		case CAMPANUS: return "Cam";
		case HORIZON: return "Hor";
		case PORPHYRY: return "Por";
		case REGIOMONTANUS: return "Reg";
		case VEHLOW: return "Veh";
		case MERIDIAN: return "Mer";
		case GAUQUELIN: return "Gau";
		case ALCABITIUS: return "Alc";
		case TOPOCENTRIC: return "Top";
		case MORINUS: return "Mor";
		case SUN: return "Sun";
		case MOON: return "Mon";
		case MERCURY: return "Mer";
		case VENUS: return "Ven";
		case MARS: return "Mar";
		case JUPITER: return "Jup";
		case SATURN: return "Sat";
		case URANUS: return "Ura";
		case NEPTUNE: return "Nep";
		case PLUTO: return "Plu";
		case CHIRON: return "Chi";
		case LILITH: return "Lil";
		case CERES: return "Cer";
		case PALLAS: return "Pal";
		case JUNO: return "Jun";
		case VESTA: return "Ves";
		case PHOLUS: return "Pho";
		case NNODE: return "Nod";
		case SNODE: return "SNd";
		case PFORTUNE: return "For";
		case VERTEX: return "Ver";
		case EPOINT: return "EPt";
		case EARTH: return "Ear";
		case ECLIPTIC_OBLIQUITY: return "EOb";
		case MEAN_NODE: return "MNd";
		case TRUE_NODE: return "TNd";
		case MEAN_APOG: return "Apo";
		case OSCU_APOG: return "OAp";
		case U_CUPIDO: return "Cup";
		case U_HADES: return "Had";
		case U_ZEUS: return "Zeu";
		case U_KRONOS: return "Kro";
		case U_APOLLON: return "Apo";
		case U_ADMETOS: return "Adm";
		case U_VULKANUS: return "Vul";
		case U_POSEIDON: return "Pos";
		case U_PROSERPINA: return "Pro";

		case CONJUNCTION: return "Con";
		case OPPOSITION: return "Opp";
		case SQUARE: return "Squ";
		case TRINE: return "Tri";
		case SEXTILE: return "Sxt";
		case INCONJUNCT: return "Inc";
		case SEMISEXTILE: return "SSx";
		case SEMISQUARE: return "SSq";
		case SESQUIQUADRATURE: return "Ses";
		case QUINTILE: return "Qui";
		case BIQUINTILE: return "BQn";
		case SEMIQUINTILE: return "SQn";
		case SEPTILE: return "Sep";
		case NOVILE: return "Nov";
		case BINOVILE: return "BNv";
		case BISEPTILE: return "BSp";
		case TRISEPTILE: return "TSp";
		case QUATRONOVILE: return "QNv";
		case PARALLEL: return "Par";
		case CONTRAPARALLEL: return "CPl";
		case DISPOSITOR_ASPECT: return "Dsp";

		case STELLIUM: return "Stl";
		case GRAND_TRINE: return "GTr";
		case T_SQUARE: return "TSq";
		case YOD_CONF: return "Yod";
		case KITE_CONF: return "Kit";
		case GRAND_CROSS: return "GCr";
		case MYSTIC_RECT: return "MyR";
		case BOOMERANG: return "Bom";
		case GR_QUINTILE: return "GQu";
		case GR_SEXTILE: return "GSx";

		case JANUARY: return "Jan";
		case FEBRUARY: return "Feb";
		case MARCH: return "Mar";
		case APRIL: return "Apr";
		case MAY: return "May";
		case JUNE: return "Jun";
		case JULY: return "Jul";
		case AUGUST: return "Aug";
		case SEPTEMBER: return "Sep";
		case OCTOBER: return "Oct";
		case NOVEMBER: return "Nov";
		case DECEMBER: return "Dec";
	}
	return "";
}

int trGetElement(int n)
{
	switch(n)
	{
		case ARIES: return colFIRE ; break;
		case TAURUS: return colEARTH ; break;
		case GEMINI: return colAIR ; break;
		case CANCER: return colWATER ; break;
		case LEO: return colFIRE ; break;
		case VIRGO: return colEARTH ; break;
		case LIBRA: return colAIR ; break;
		case SCORPIO: return colWATER ; break;
		case SAGITTARIUS: return colFIRE ; break;
		case CAPRICORN: return colEARTH ; break;
		case AQUARIUS: return colAIR ; break;
		case PISCES: return colWATER ; break;
		case HOUSE1: return colFIRE ; break;
		case HOUSE2: return colEARTH ; break;
		case HOUSE3: return colAIR ; break;
		case HOUSE4: return colWATER ; break;
		case HOUSE5: return colFIRE ; break;
		case HOUSE6: return colEARTH ; break;
		case HOUSE7: return colAIR ; break;
		case HOUSE8: return colWATER ; break;
		case HOUSE9: return colFIRE ; break;
		case HOUSE10: return colEARTH ; break;
		case HOUSE11: return colAIR ; break;
		case HOUSE12: return colWATER ; break;
		case SUN: return colFIRE ; break;
		case MOON: return colWATER ; break;
		case MERCURY: return colAIR ; break;
		case VENUS: return colAIR ; break;
		case MARS: return colFIRE ; break;
		case JUPITER: return colFIRE ; break;
		case SATURN: return colEARTH ; break;
		case URANUS: return colAIR ; break;
		case NEPTUNE: return colWATER ; break;
		case PLUTO: return colWATER ; break;
		case CHIRON: return colCHIRON ; break;
		case NNODE: return colOTHER ; break;
		case VERTEX: return colOTHER ; break;
		case CERES: return colCHIRON ; break;
		case PALLAS: return colCHIRON ; break;
		case JUNO: return colCHIRON ; break;
		case VESTA: return colCHIRON ; break;
		case PHOLUS: return colCHIRON ; break;
		case U_CUPIDO: return colURANIAN ; break;
		case U_HADES: return colURANIAN ; break;
		case U_ZEUS: return colURANIAN ; break;
		case U_KRONOS: return colURANIAN ; break;
		case U_APOLLON: return colURANIAN ; break;
		case U_ADMETOS: return colURANIAN ; break;
		case U_VULKANUS: return colURANIAN ; break;
		case U_POSEIDON: return colURANIAN ; break;
		case U_PROSERPINA: return colURANIAN ; break;
		case LILITH: return colOTHER ; break;
		case SNODE: return colOTHER ; break;
		case PFORTUNE: return colOTHER ; break;
		case EPOINT: return colOTHER ; break;
		case EARTH: return colOTHER ; break;
		case MEAN_NODE: return colOTHER ; break;
		case TRUE_NODE: return colOTHER ; break;
		case MEAN_APOG: return colOTHER ; break;
		case OSCU_APOG: return colOTHER ; break;
	}
	
	if(IS_PLANETARY_BODY(n)) return colOTHER;

	return colERROR;
}

int trGetMode(int n)
{
	switch(n)
	{
		case ARIES: return colCARDINAL ; break;
		case TAURUS: return colFIXED ; break;
		case GEMINI: return colMUTABLE ; break;
		case CANCER: return colCARDINAL ; break;
		case LEO: return colFIXED ; break;
		case VIRGO: return colMUTABLE ; break;
		case LIBRA: return colCARDINAL ; break;
		case SCORPIO: return colFIXED ; break;
		case SAGITTARIUS: return colMUTABLE ; break;
		case CAPRICORN: return colCARDINAL ; break;
		case AQUARIUS: return colFIXED ; break;
		case PISCES: return colMUTABLE ; break;
		case HOUSE1: return colCARDINAL ; break;
		case HOUSE2: return colFIXED ; break;
		case HOUSE3: return colMUTABLE ; break;
		case HOUSE4: return colCARDINAL ; break;
		case HOUSE5: return colFIXED ; break;
		case HOUSE6: return colMUTABLE ; break;
		case HOUSE7: return colCARDINAL ; break;
		case HOUSE8: return colFIXED ; break;
		case HOUSE9: return colMUTABLE ; break;
		case HOUSE10: return colCARDINAL ; break;
		case HOUSE11: return colFIXED ; break;
		case HOUSE12: return colMUTABLE ; break;
		case SUN: return colFIXED ; break;
		case MOON: return colCARDINAL ; break;
		case MERCURY: return colMUTABLE ; break;
		case VENUS: return colCARDINAL ; break;
		case MARS: return colCARDINAL ; break;
		case JUPITER: return colMUTABLE ; break;
		case SATURN: return colCARDINAL ; break;
		case URANUS: return colFIXED ; break;
		case NEPTUNE: return colMUTABLE ; break;
		case PLUTO: return colFIXED ; break;
		case CHIRON: return colCHIRON ; break;
		case NNODE: return colOTHER ; break;
		case VERTEX: return colOTHER ; break;
		case CERES: return colCHIRON ; break;
		case PALLAS: return colCHIRON ; break;
		case JUNO: return colCHIRON ; break;
		case VESTA: return colCHIRON ; break;
		case PHOLUS: return colCHIRON ; break;
		case U_CUPIDO: return colURANIAN ; break;
		case U_HADES: return colURANIAN ; break;
		case U_ZEUS: return colURANIAN ; break;
		case U_KRONOS: return colURANIAN ; break;
		case U_APOLLON: return colURANIAN ; break;
		case U_ADMETOS: return colURANIAN ; break;
		case U_VULKANUS: return colURANIAN ; break;
		case U_POSEIDON: return colURANIAN ; break;
		case U_PROSERPINA: return colURANIAN ; break;
		case LILITH: return colOTHER ; break;
		case SNODE: return colOTHER ; break;
		case PFORTUNE: return colOTHER ; break;
		case EPOINT: return colOTHER ; break;
		case EARTH: return colOTHER ; break;
		case MEAN_NODE: return colOTHER ; break;
		case TRUE_NODE: return colOTHER ; break;
		case MEAN_APOG: return colOTHER ; break;
		case OSCU_APOG: return colOTHER ; break;
	}

	if(IS_PLANETARY_BODY(n)) return colOTHER;

	return colERROR;
}

static const planet_rules rules_table[10] = {
	{ 1, LEO,         -1 }, /* Sun */
	{ 1, CANCER,      -1 }, /* Moon */
	{ 2, GEMINI,   VIRGO }, /* Mercury */
	{ 2, LIBRA,   TAURUS }, /* Venus */
	{ 1, ARIES,       -1 }, /* Mars */
	{ 1, SAGITTARIUS, -1 }, /* Jupiter */
	{ 1, CAPRICORN,   -1 }, /* Saturn */
	{ 1, AQUARIUS,    -1 }, /* Uranus */
	{ 1, PISCES,      -1 }, /* Neptune */
	{ 1, SCORPIO,     -1 }  /* Pluto */
};

static const int ruler_table[12] = {
	MARS,    /* Aries */
	VENUS,   /* Taurus */
	MERCURY, /* Gemini */
	MOON,    /* Cancer */
	SUN,     /* Leo */
	MERCURY, /* Virgo */
	VENUS,   /* Libra */
	PLUTO,   /* Scorpio */
	JUPITER, /* Sagittarius */
	SATURN,  /* Capricorn */
	URANUS,  /* Aquarius */
	NEPTUNE  /* Pisces */
};

int RulerOf(int sign)
{
	if(IS_SIGN(sign))
		return ruler_table[sign-ARIES];
	else
		return -1;
}

planet_rules Rules(int planet)
{
	planet_rules error_pr;

	if(IS_PLANET(planet))
	{
		return rules_table[planet-SUN];
	}
	else
	{
		error_pr.n = -1;
		error_pr.sign1 = -1;
		error_pr.sign2 = -1;
		return error_pr;
	}
}

double AspectDef(int n)
{
	switch(n)
	{
		case CONJUNCTION: return 0.0; break;
		case OPPOSITION: return 180.0; break;
		case SQUARE: return 90.0; break;
		case TRINE: return 120.0; break;
		case SEXTILE: return 60.0; break;
		case INCONJUNCT: return 150.0; break;
		case SEMISEXTILE: return 30.0; break;
		case SEMISQUARE: return 45.0; break;
		case SESQUIQUADRATURE: return 135.0; break;
		case QUINTILE: return 72.0; break;
		case BIQUINTILE: return 144.0; break;
		case SEMIQUINTILE: return 36.0; break;
		case SEPTILE: return (360.0/7.0); break;
		case NOVILE: return 40.0; break;
		case BINOVILE: return 80.0; break;
		case BISEPTILE: return (2.0*360.0/7.0); break;
		case TRISEPTILE: return (3.0*360.0/7.0); break;
		case QUATRONOVILE: return 160.0; break;
		case PARALLEL: return 0.0; break;
		case CONTRAPARALLEL: return 0.0; break;
	}
	return -1.0;
}

double AspectOrbDefaults(int n)
{
	// Just the defaults!  Programs should set the orbs themselves!
	switch(n)
	{
		case CONJUNCTION: return 8.0; break;
		case OPPOSITION: return 6.0; break;
		case SQUARE: return 6.0; break;
		case TRINE: return 6.0; break;
		case SEXTILE: return 6.0; break;
		case INCONJUNCT: return 3.0; break;
		case SEMISEXTILE: return 3.0; break;
		case SEMISQUARE: return 3.0; break;
		case SESQUIQUADRATURE: return 3.0; break;
		case QUINTILE: return 2.0; break;
		case BIQUINTILE: return 2.0; break;
		case SEMIQUINTILE: return 1.0; break;
		case SEPTILE: return 1.0; break;
		case NOVILE: return 1.0; break;
		case BINOVILE: return 1.0; break;
		case BISEPTILE: return 1.0; break;
		case TRISEPTILE: return 1.0; break;
		case QUATRONOVILE: return 1.0; break;
		case PARALLEL: return 1.0; break;
		case CONTRAPARALLEL: return 1.0; break;
	}
	return 1.0; // Unlike most other translation functions, exit gracefully
	            // on an unknown aspect; not having a default orb is not
	            // a fatal error for most charts
}

const int *GetAspConfigurationDef(int n)
{
	static const int stellium_a[] = STELLIUM_A;
	static const int grand_trine_a[] = GRAND_TRINE_A;
	static const int t_square_a[] = T_SQUARE_A;
	static const int kite_conf_a[] = KITE_CONF_A;
	static const int grand_cross_a[] = GRAND_CROSS_A;
	static const int yod_conf_a[] = YOD_CONF_A;
	static const int mystic_rect_a[] = MYSTIC_RECT_A;
	static const int boomerang_a[] = BOOMERANG_A;
	static const int gr_quintile_a[] = GR_QUINTILE_A;
	static const int gr_sextile_a[] = GR_SEXTILE_A;

	switch(n)
	{
		case STELLIUM: return stellium_a; break;
		case GRAND_TRINE: return grand_trine_a; break;
		case T_SQUARE: return t_square_a; break;
		case KITE_CONF: return kite_conf_a; break;
		case GRAND_CROSS: return grand_cross_a; break;
		case YOD_CONF: return yod_conf_a; break;
		case MYSTIC_RECT: return mystic_rect_a; break;
		case BOOMERANG: return boomerang_a; break;
		case GR_QUINTILE: return gr_quintile_a; break;
		case GR_SEXTILE: return gr_sextile_a; break;
	}
	return NULL;
}

const char *ChartTitleStr(int n)
{
	switch(n)
	{
		case CHART_NATAL1: return "Native"; break;
		case CHART_NATAL2: return "Partner"; break;
		case CHART_TRANSITS: return "Transit"; break;
		case CHART_DAVISON: return "Davison"; break;
		case CHART_COMPOSITE: return "Composite"; break;
		case CHART_PROGRESSED1: return "Native's Progressed"; break;
		case CHART_PROGRESSED2: return "Partner's Progressed"; break;
		case CHART_DEFAULT: return "Default"; break;
		default: return "Other";
	}
}
