//Translation.h
//
//One of the goals of the program is to be independent.  Independent of operating-
//system for one, which is why it uses the cross-platform wxWindows toolkit.  But
//it should also be independent of the back-end astrology library and the glyph-
//drawing routines.  Therefore, this module traslates among the main program, the
//glyph drawer, and the astrology library.  No other file should directly call
//either; if there is a need to add back-end functionality, it should go here.
//
//That being said, it was more convenient for the transformation of the glyphs to
//go in TurtleDraw at the moment; if a different glyph-drawing library or source
//file is substituted, a constant translation routine will need to be written, and
//probably should be put here.
//
//-------------------------------------------------------------------------------//
//
//Copyright (C) 2003 mikesag
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


#ifndef __TRANSITS__TRANSLATION_H__
#define __TRANSITS__TRANSLATION_H__

#define IS_FIRE(n)  (trGetElement(n) == colFIRE)
#define IS_EARTH(n) (trGetElement(n) == colEARTH)
#define IS_AIR(n)   (trGetElement(n) == colAIR)
#define IS_WATER(n) (trGetElement(n) == colWATER)

#define IS_CARDINAL(n) (trGetMode(n) == colCARDINAL)
#define IS_FIXED(n)    (trGetMode(n) == colFIXED)
#define IS_MUTABLE(n)  (trGetMode(n) == colMUTABLE)

#define IS_CHIRON_COL(n)  (trGetElement(n) == colCHIRON)
#define IS_OTHER_COL(n)   (trGetElement(n) == colOTHER)
#define IS_URANIAN_COL(n) (trGetElement(n) == colURANIAN)
#define IS_ERROR_COL(n)   (trGetElement(n) == colERROR)

struct planet_rules
{
	int n;
	int sign1;
	int sign2;
};

// Conversion Functions
int ToSwissEph(int n);   // Returns -1 on error
int FromSwissEph(int n);
int FromSwissEphHouse(char c);

const char *trGetName(int n);
const char *trGetAbbrevName(int n);
int trGetElement(int n);
int trGetMode(int n);

// IMPORTANT!!! Signs and houses are 1-based, not 0-based
int IntToSign(int n);
int SignToInt(int h);
int IntToHouse(int n);
int HouseToInt(int h);

int RulerOf(int sign);
planet_rules Rules(int planet);

double AspectDef(int n);
double AspectOrbDefaults(int n); // Default orbs only; program should set it itself

const int *GetAspConfigurationDef(int n);  // An array of integers; see Translation.h

const char *ChartTitleStr(int n);

#endif
