/*
 * TurtleDraw.cpp
 *
 * Code Copyright (C) 2003 mikesag
 * Data Copyright (C) 1991-1998 Walter D. Pullen (in specially marked section only)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * ------------------------------------------------------------------------------ *
 *
 * This file and the corresponding .h file implement turtle drawing as was
 * was used on Logo or BASIC back in the 80's.
 *
 * The turtle engine was written from scratch, but the glyphs themselves are
 * from Walter Pullen's Astrolog, relicensed by permission under the GNU GPL.
 * I'd like to thank him for his inspiration in creating such a high-quality
 * free astrology program!
 *
 * (I wrote a couple of extra glyphs myself; I give him permission to use
 * those glyphs himself under whatever license he wants, should he desire.)
 *
 */

#include "wx/wxprec.h"

#ifndef WX_PRECOMP
	#include "wx/wx.h"
#endif

#include "wx/settings.h"
#include "wx/dynarray.h"
#include "wx/image.h"

#include "AstroConst.h"
#include "TurtleDraw.h"

#define STAY_PUT  0x1
#define MOVE_ONLY 0x2
#define NO_GLYPH  0x4

#define TD_DELTAX 0
#define TD_DELTAY 1
#define TD_FLAGS  2

static int ScanNum(const wxString &s, size_t &pos);
static void CompileGlpyhs();

/********************* WALTER PULLEN'S GLYPHS ***********************/
#ifndef CONST
#define CONST const
#endif

#ifndef ARR
#define ARR
#endif

#ifndef STATIC
#define STATIC static
#endif

#define cSign    12
#define uranHi   40
#define oNorm    uranHi
#define cAspect  18

/* deltax, deltay, and flags */
typedef wxArrayInt wxArrayThreeInts[3];

STATIC wxArrayThreeInts ARR nDrawSign[cSign+2];
STATIC CONST char * ARR szDrawSign[cSign+2] = {"",
  "ND4HU2HLGDFBR6EUHLGD2G",                /* Aries       */
  "BL3D2F2R2E2U2H2NE2L2NH2G2",             /* Taurus      */
  "BLU3LHBR7GLNL3D6NL3RFBL7ERU3",          /* Gemini      */
  "BGNDHLGDFRNEFR2EREBU3NHDGLHUENRHL2GLG", /* Cancer      */
  "BF4H2UEU2H2L2G2D2FDGH",                 /* Leo         */
  "BF4BLHNGNHEU5G2ND3U2HGND6HGND6H",       /* Virgo       */
  "BGNL3HUER2FDGR3BD2L8",                  /* Libra       */
  "BH4FND6EFND6EFD6FREU",                  /* Scorpio     */
  "BG4E3NH2NF2E5NL2D2",                    /* Sagittarius */
  "BH3BLED4FND2EU2ENF2UFERFDGLF2D2G",      /* Capricorn #1 */
  "BG4EUEDFDEUEDFDEUEBU5GDGUHUGDGUHUGDG",  /* Aquarius    */
  "NL4NR4BH4F2D4G2BR8H2U4E2",              /* Pisces      */
  "BH4RFR2ER3G3D2GDFR2EU2HL3G2DG"};        /* Capricorn #2 */

STATIC wxArrayThreeInts ARR nDrawSign2[cSign+2];
STATIC CONST char * ARR szDrawSign2[cSign+2] = {"",
  "BD8U7HU3HU2H2L2G2D2F2BR12E2U2H2L2G2D2GD3G",     /* Aries  */
  "BH6BU2FDFRFNR4GLGDGD4FDFRFR4EREUEU4HUHLHEREUE", /* Taurus */
  "", /* Gemini */
  "BG5NRLH2U2E2R2F2D2G2F2R4ER2E3BH6NE2D2F2R2E2U2H2L2H2L4GL2G3", /* Cancer */
  "", /* Leo   */
  "", /* Virgo */
  "", /* Libra */
  "BH8F2ND12E2F2ND12E2F2D12F2RE2U3NGF", /* Scorpio */
  "", /* Sagittarius  */
  "", /* Capricorn #1 */
  "BG8EUE2UEDFD2FDEUE2UEDFD2FDEUE2UEBU10GDG2DGUHU2HUGDG2DGUHU2HUGDG2DG",
  "NL8NR8BH8F3DFD6GDG3BR16H3UHU6EUE3", /* Pisces */
  ""}; /* Capricorn #2 */

STATIC wxArrayThreeInts ARR nDrawObject[oNorm+7];
STATIC CONST char * ARR szDrawObject[oNorm+7] = {
  "ND4NL4NR4U4LGLDGD2FDRFR2ERUEU2HULHL",    /* Earth   */
  "U0BH3DGD2FDRFR2ERUEU2HULHL2GL",          /* Sun     */
  "BG3E2U2H2ER2FRDFD2GDLGL2H",              /* Moon    */
  "BD4UNL2NR2U2REU2HNEL2NHGD2FR",           /* Mercury */
  "LHU2ER2FD2GLD2NL2NR2D2",                 /* Venus   */
  "HLG2DF2RE2UHE4ND2L2",                    /* Mars    */
  "BH3RFDGDGDR5NDNR2U6E",                   /* Jupiter */
  "BH3R2NUNR2D3ND3RERFDGDF",                /* Saturn  */
  "BD4NEHURBFULU3NUNR2L2NU2DGBU5NFBR6GD3F", /* Uranus #1 */
  "BD4U2NL2NR2U5NUNRLBL2NUNLDF2R2E2UNRU",   /* Neptune   */
  "D2NL2NR2D2BU8GFEHBL3D2F2R2E2U2",         /* Pluto  #1 */
  "BG2LDFEULU3NURFRFBU5GLGLU2",             /* Chiron          */
  "BD4UNL3NR3U2RE2UH2L2G",                  /* Ceres           */
  "BD4UNL3NR3UE2HUHNUGDGF2",                /* Pallas Athena   */
  "BD4UNL2NR2U4NL4NR4NE3NF3NG3NH3U3",       /* Juno            */
  "BU4DBG3NLFDF2E2UERBH2GDGHUH",            /* Vesta           */
  "BG2LGFEU2HU2E2R2F2D2GD2FEHL",            /* North Node      */
  "BG4E8BG2FD2G2L2H2U2E2R2F",               /* Lilith #1       */
  "NE2NF2NG2H2GD2F2R2E2U2H2L2G",            /* Part of Fortune */
  "U2NHNEBD4NGNFU2L2NHNGR4NEF",             /* Vertex          */
  "BH4NR3D4NR2D4R3BR2U8R2FD2GL2",           /* East Point      */
  "BG4U4NR2U3EFD7BR2NURU2HU2RDBR3ULD5RU",   /* Ascendant  */
  "BH3ER4FD2GLGLG2DR6",                     /* 2nd Cusp   */
  "BH3ER4FD2GNL3FD2GL4H",                   /* 3rd Cusp   */
  "BH4R2NR2D8NL2R2BR4NUL2U8R2D",            /* Nadir      */
  "BG3FR4EU2HL5U4R6",                       /* 5th Cusp   */
  "BE3HL4GD6FR4EU2HL4G",                    /* 6th Cusp   */
  "BH4D8REU6HLBF7DRU2HU2RDBG4NRU3NRU2R",    /* Descendant */
  "BL2GD2FR4EU2HNL4EU2HL4GD2F",             /* 8th Cusp   */
  "BG3FR4EU6HL4GD2FR4E",                    /* 9th Cusp   */
  "BG4U8F2ND6E2D8BR4NUL2U8R2D",             /* Midheaven  */
  "BH3ED8NLRBR2RNRU8G",                     /* 11th Cusp  */
  "BG4RNRU8GBR4ER2FD2GLG2D2R4",             /* 12th Cusp  */
  "BH4BRFDG2DR8BG3UNL2NR2U5LUEFDL",         /* Cupido    */
  "BENUNL2NR2D3ND2NR2L2H2U2E2R4",           /* Hades     */
  "BU4NG2NF2D7NDBLHLBR6LGL2GLBR6LHL",       /* Zeus      */
  "BU2D3ND3NR2L2BH2UE2R4F2D",               /* Kronos    */
  "U3NLR2NRD3NL2NR2D4NRL2NLU4L4UEUH",       /* Apollon   */
  "BUNU2NL2NR2D2ND3LHU2ENHR2NEFD2GL",       /* Admetos   */
  "G2DGR6HUH2U4NG2F2",                      /* Vulcanus  */
  "ND4U4BL3DF2R2E2UBD8UH2L2G2D",            /* Poseidon  */
  "BD2D0BU6NG2NF2D4LGD2FR2EU2HL",           /* Uranus #2 */
  "BL3R5EU2HL5D8R5",                        /* Pluto  #2 */
  "UERHL2G2D2F2R2ELHU",                     /* Lilith #2 */
  "BH2LHEFD2GD2F2R2E2U2HU2EFGL",            /* Lilith #3 */
  "BUR2EUHL2D6L3DFR3EUL3",                  /* Pholus */
  "BU4BL4R3D8L3BR8L3U8R3BL4BD4NU0"          /* Proserpina,
                                               or the Apple ][ Memorial Glyph */
  };

STATIC wxArrayThreeInts ARR nDrawObject2[oNorm+7];
STATIC CONST char * ARR szDrawObject2[oNorm+7] = {
  "ND8NL8NR8U8L2GLG3DGD4FDF3RFR4ERE3UEU4HUH3LHL2", /* Earth */
  "U0BU8L2GLG3DGD4FDF3RFR4ERE3UEU4HUH3LHL2",       /* Sun   */
  "BG6E3UEU2HUH3E2R4FRF3DFD4GDG3LGL4H2",           /* Moon  */
  "", /* Mercury */
  "", /* Venus   */
  "BELHL4G3D4F3R4E3U4HUE7ND5L5",      /* Mars    */
  "BH6BRRF2D2GDGDGDGDR10ND2NR4U12E2", /* Jupiter */
  "", /* Saturn  */
  "BD4LGD2FR2EU2HLU6NU2NR4L4NU4D2G2BU10NF2BR12G2D6F2",     /* Uranus #1 */
  "BD8U4NL4NR4U10NU2NR2L2BL3LNU2NLD2FDFRFR4EREUEU2NLNRU2", /* Neptune   */
  "D4NL4NR4D4BU16LGD2FR2EU2HLBL6D4FDFRFR4EREUEU4",         /* Pluto  #1 */
  "BG4LGD2FR2EU2HLU7RF2RF2RFBU10GLG2LG2BLU5",     /* Chiron        */
  "BD8U2NL6NR6U4R3E3U4H3L4G2",                    /* Ceres         */
  "BD8U2NL6NR6U2E4HUHUHUHNUGDGDGDGF4",            /* Pallas Athena */
  "BD8U2NL4NR4U8NL7NR7NE5NF5NG5NH5U6",            /* Juno          */
  "BU8D3BG5NL3DF2DF2DFEUE2UE2UR3BH4GDG2DGHUH2UH", /* Vesta         */
  "", /* North Node      */
  "", /* Lilith #1       */
  "", /* Part of Fortune */
  "", /* Vertex          */
  "", /* East Point      */
  "BG8U8NR4U6E2F2D14BR4NHREU3HLHU3ERFBR6HLGD8FRE", /* Ascendant */
  "", /* 2nd Cusp   */
  "", /* 3rd Cusp   */
  "BH8R4NR4D16NL4R4BR8BUNUGL3HU14ER3FD", /* Nadir */
  "", /* 5th Cusp   */
  "", /* 6th Cusp   */
  "BH8D16R2E2U12H2L2BF14BGFREU3HLHU3ERFBG9NR3U5NR3U5R3", /* Descendant */
  "", /* 8th Cusp   */
  "", /* 9th Cusp   */
  "BG8U16F4ND12E4D16BR8BUNUGL3HU14ER3FD", /* Midheaven */
  "", /* 11th Cusp  */
  "", /* 12th Cusp  */
  "", /* Cupido    */
  "", /* Hades     */
  "", /* Zeus      */
  "", /* Kronos    */
  "", /* Apollon   */
  "", /* Admetos   */
  "", /* Vulcanus  */
  "", /* Poseidon  */
  "", /* Uranus #2 */
  "", /* Pluto  #2 */
  "", /* Lilith #2 */
  "", /* Lilith #3 */
  "BU2R4E2U2H2L4D12L4G1D2F1R8E1U2H1L4",       /* Pholus */
  ""  /* Proserpina */
  };

STATIC wxArrayThreeInts ARR nDrawHouse[cSign+1];
STATIC CONST char * ARR szDrawHouse[cSign+1] = {"",
  "BD2NLNRU4L", "BHBUR2D2L2D2R2", "BHBUR2D2NL2D2L2",
  "BHBUD2R2NU2D2", "BEBUL2D2R2D2L2", "NLRD2L2U4R2",
  "BHBUR2DG2D", "NRLU2R2D4L2U2", "NRLU2R2D4L2",
  "BH2NLD4NLRBR2U4R2D4L2", "BH2NLD4NLRBR2RNRU4L", "BH2NLD4NLRBR2NR2U2R2U2L2"};

STATIC wxArrayThreeInts ARR nDrawHouse2[cSign+1];
STATIC CONST char * ARR szDrawHouse2[cSign+1] = {"",
  "BD4NL2NR2U8G2", "BH2BUER2FD2G4DR4", "BH2BUER2FD2GNLFD2GL2H",
  "BH2BU2D4R3NU4NRD4", "BE2BU2L4D4R3FD2GL2H", "NL2RFD2GL2HU6ER2F",
  "", "NRLHU2ER2FD2GFD2GL2HU2E", "NR2LHU2ER2FD6GL2H",
  "BH4NG2D8NL2R2BR5HU6ER2FD6GL2", "BH4NG2D8NL2R2BR4R2NR2U8G2",
  "BH4NG2D8NL2R2BR4NR4UE4U2HL2G"};

STATIC wxArrayThreeInts ARR nDrawAspect[cAspect+3];
STATIC CONST char * ARR szDrawAspect[cAspect+3] = {"",
  "HLG2DF2RE2UHE4",                        /* Conjunction      */
  "BGL2GDFREU2E2U2ERFDGL2",                /* Opposition       */
  "BH4R8D8L8U8",                           /* Square           */
  "BU4GDGDGDGDR8UHUHUHUH",                 /* Trine            */
  "BLNH3NG3RNU4ND4RNE3F3",                 /* Sextile          */
  "BG4EUEUEUEUNL4NR4BDFDFDFDF",            /* Inconjunct       */
  "BH4FDFDFDFDNL4NR4BUEUEUEUE",            /* Semisextile      */
  "BE4G8R8",                               /* Semisquare       */
  "BD2L3U6R6D6L3D2R2",                     /* Sesquiquadrature */
  "F4BU3U2HULHL2GLDGD2FDRFR2E3",           /* Quintile         */
  "BD2U3NR3NU3L3BD5R6",                    /* Biquintile       */
  "BU2D3NR3ND3L3BU5R6",                    /* Semiquintile     */
  "BH3R6G6",                               /* Septile      */
  "BR3L5HUER4FD4GL4H",                     /* Novile       */
  "BF2UHL2GFR3DGL3BE6LNLU2NRLBL4LNLD2NLR", /* Binovile     */
  "BL2R4G4BE6LNLU2NRLBL4LNLD2NLR",         /* Biseptile    */
  "BL2R4G4BE6L7NLU2NLR3ND2R3ND2R",         /* Triseptile   */
  "BF2UHL2GFR3DGL3BU6LNLU2NLRBR2F2E2",     /* Quatronovile */
  "BU4BLD8BR2U8",                          /* Parallel       */
  "BU4BLD8BR2U8BF3BLL6BD2R6"};             /* Contraparallel */

STATIC wxArrayThreeInts ARR nDrawAspect2[cAspect+3];
STATIC CONST char * ARR szDrawAspect2[cAspect+3] = {"",
  "BELHL4G3D4F3R4E3U4HUE7",                /* Conjunction */
  "BG3HL2G2D2F2R2E2U2HE6HU2E2R2F2D2G2L2H", /* Opposition  */
  "", /* Square           */
  "BU8GDGDGDGDGDGDGDGDR16UHUHUHUHUHUHUHUH",     /* Trine       */
  "BU8D16BL8BU2E3RE3R2E3RE3BL16F3RF3R2F3RF3",   /* Sextile     */
  "BG8EUEUEUEUEUEUEUEUNL8NR8BDFDFDFDFDFDFDFDF", /* Inconjunct  */
  "BH8FDFDFDFDFDFDFDFDNL8NR8BUEUEUEUEUEUEUEUE", /* Semisextile */
  "", /* Semisquare       */
  "", /* Sesquiquadrature */
  "BFF7BU6U4HUH3LHL4GLG3DGD4FDF3RFR4E6", /* Quintile */
  "", /* Biquintile   */
  "", /* Semiquintile */
  "", /* Septile      */
  "", /* Novile       */
  "", /* Binovile     */
  "", /* Biseptile    */
  "", /* Triseptile   */
  "", /* Quatronovile */
  "", /* Parallel        */
  ""}; /* Contraparallel */

STATIC wxArrayThreeInts ARR nDrawCh[128-32+2];
STATIC CONST char * ARR szDrawCh[128-32+2] = {"",
  "BR2D4BD2D0", "BRD2BR2U2", "BD2R4BD2L4BFU4BR2D4", "BR2D6BENL3EHL2HER3",
  "RDLNUBR4G4BR4DLUR", "BD2NF4UEFDG2DFRE2", "BR2DG", "BR3G2D2F2", "BRF2D2G2",
  "BD2FNGRNU2ND2RNEF", "BD3R2NU2ND2R2", "BD5BR2DG", "BD3R4", "BD6BRRULD",
  "BD5E4", /* Special Characters */

  "BDD4NE4FR2EU4HL2G", "BFED6NLR", "BDER2FDG4R4", "BDER2FDGNLFDGL2H",
  "D3R3NU3ND3R", "NR4D3R3FDGL2H", "BR3NFL2GD4FR2EUHL3", "R4DG4D",
  "BDDFNR2GDFR2EUHEUHL2G", "BD5FR2EU4HL2GDFR3", /* Numbers */

  "BR2BD2D0BD2D0", "BR2BD2D0BD2G", "BR3G3F3", "BD2R4BD2L4", "BRF3G3",
  "BDER2FDGLDBD2D0", "BF2DFEU2HL2GD4FR2", /* Special Characters */

  "BD6U4E2F2D2NL4D2", "D6R3EUHNL3EUHL3", "BR3NFL2GD4FR2E", "D6R2E2U2H2L2",
  "NR4D3NR3D3R4", "NR4D3NR3D3", "BR3NFL2GD4FR2EU2L2", "D3ND3R4NU3D3",
  "BRRNRD6NLR", "BD4DFR2EU5", "D3ND3RNE3F3", "D6R4", "ND6F2NDE2D6",
  "ND6F4ND2U4", "BDD4FR2EU4HL2G", "R3FDGL3NU3D3", "BDD4FRENHNFEU3HL2G",
  "ND6R3FDGL2NLF3", "BR3NFL2GDFR2FDGL2H", "R2NR2D6", "D5FR2EU5",
  "D2FDFNDEUEU2", "D6E2NUF2U6", "DF4DBL4UE4U", "D2FRND3REU2",
  "R4DG4DR4", /* Upper Case Letters */

  "BR3L2D6R2", "BDF4", "BRR2D6L2", "BD2E2F2", "BD6R4", "BR2DF", /* Symbols */

  "BF4G2LHU2ER2FD3", "D5NDFR2EU2HL2G", "BF4BUHL2GD2FR2E", "BR4D5NDGL2HU2ER2F",
  "BD4R4UHL2GD2FR3", "BD3RNR3ND3U2ERF", "BD8R3EU4HL2GD2FR2E", "D3ND3ER2FD3",
  "BR2D0BD2D4", "BR2D0BD2D5GLH", "D4ND2REREBD4HLH", "BR2D6",
  "BD2DND3EFNDEFD3", "BD2DND3ER2FD3", "BD3D2FR2EU2HL2G", "BD2DND5ER2FD2GL2H",
  "BR4BD8U5HL2GD2FR2E", "BD2DND3ER2F", "BD6R3EHL2HER3", "BR2D2NL2NR2D4",
  "BD2D3FRE2NU2D2", "BD2DFDFEUEU", "BD2D3FENUFEU3", "BD2F2NG2NE2F2",
  "BD2D3FR2ENU3D2GL3", "BD2R4G4R4", /* Lower Case Letters */

  "BR3GDGFDF", "BR2D2BD2D2", "BRFDFGDG", "BFEFE", "BD6R4",
  "BR2FGHE", "BL3R6G2BE2H2"}; /* Symbols */
/****************** END OF WALTER PULLEN'S GLYPHS ********************/


void InitializeTurtle()
{
	CompileGlpyhs();
}

static int ScanNum(const wxString &s, size_t &pos)
{
	int n, j;
	char c;

	if(pos >= s.length())
		return 1;

	n = j = 0;
	c = s[pos];
	while ((c >= '0') && (c <= '9'))
	{
		pos++;
		j++;

		n = 10*n + (int)(c - '0');

		if(pos < s.length())
			c = s[pos];
		else
			break;
	}
	
	return ((n > 0) ? n : ((j < 1) ? 1 : 0));
}

static void CompileGlpyhs()
{
	/**************************************************/
	/* Compile to a list of x and y changes and flags */
	/* instead of actually drawing.  Then we can make */
	/* TurtleDraw() use dx and dy directly to         */
	/* draw the glyphs instead of the strings, thus   */
	/* vastly improving its efficiency (given that    */
	/* ScanNum() was the most called routine in the   */
	/* routine in the entire program before!)         */
	/**************************************************/
	
	const int size_array[] = {cSign + 2, cSign + 2, oNorm + 7, oNorm + 7,
		cSign + 1, cSign + 1, cAspect + 3, cAspect + 3, 128 - 32 + 2};
	const int N_ARRAYS = sizeof(size_array)/sizeof(int);
	const char ARR **string_array[] = {szDrawSign, szDrawSign2,
		szDrawObject, szDrawObject2, szDrawHouse, szDrawHouse2,
		szDrawAspect, szDrawAspect2, szDrawCh};
	wxArrayThreeInts ARR *int_array[] = {nDrawSign, nDrawSign2,
		nDrawObject, nDrawObject2, nDrawHouse, nDrawHouse2,
		nDrawAspect, nDrawAspect2, nDrawCh};
	
	static int shown_error = 0;
	int linelen, deltax, deltay, flags;
	wxString cmds;
	size_t i;
	int iarr, j;

	for(iarr = 0; iarr < N_ARRAYS; iarr++)
	{
		for(j = 0; j < size_array[iarr]; j++)
		{
			cmds = wxString(string_array[iarr][j]);
			cmds.MakeUpper();
			flags = 0;
		
			/* pretend like we're drawing it, but store the dx and dy instead
			   of actually doing the drawing. */
			if(cmds.length() == 0)
			{
				int_array[iarr][j][TD_DELTAX].Add(0);
				int_array[iarr][j][TD_DELTAY].Add(0);
				int_array[iarr][j][TD_FLAGS ].Add(NO_GLYPH);
			}
			else for(i = 0; i < cmds.length(); i++)
			{
				deltax = deltay = 0;

				if(isspace(cmds[i])) /* ignore whitespace */
					continue;
				
				if(cmds[i] == 'B')
				{
					flags = flags | MOVE_ONLY;
					continue;
				}
				
				if(cmds[i] == 'N')
				{
					flags = flags | STAY_PUT;
					continue;
				}
				
				switch(cmds[i])
				{
				case 'U': deltax =  0; deltay = -1; break;      /* Up    */
				case 'D': deltax =  0; deltay =  1; break;      /* Down  */
				case 'L': deltax = -1; deltay =  0; break;      /* Left  */
				case 'R': deltax =  1; deltay =  0; break;      /* Right */
				case 'E': deltax =  1; deltay = -1; break;      /* NorthEast */
				case 'F': deltax =  1; deltay =  1; break;      /* SouthEast */
				case 'G': deltax = -1; deltay =  1; break;      /* SouthWest */
				case 'H': deltax = -1; deltay = -1; break;      /* NorthWest */
				default:
					if(shown_error < 5)
					{
						wxString str;
						str.Printf("ERROR compiling glyph \"%s\" (position %d: %c)",
							cmds.c_str(), i, (char)(cmds.GetChar(i)));
						wxMessageBox(str);
						shown_error++;
					}
				}
				
				i++;
				linelen = ScanNum(cmds, i);
				i--;

				int_array[iarr][j][TD_DELTAX].Add(deltax*linelen);
				int_array[iarr][j][TD_DELTAY].Add(deltay*linelen);
				int_array[iarr][j][TD_FLAGS ].Add(flags);
				flags = 0;
			} /* end for(i) [loop within string] */
		} /* end for(j) [loop over glyphs] */
	} /* end for(iarr) [loop over arrays] */

	/* pack compiled glyphs */
	//for(iarr = 0; iarr < N_ARRAYS; iarr++)
	//	for(i = 0; i < 3; i++)
	//		int_array[iarr][i]->Shrink();
}

static void TurtleDrawCompiled(const wxArrayThreeInts &step,
							   wxDC *dc, int x0, int y0, int scale)
{
	static int shown_error = 0;
	int oldx, oldy, deltax, deltay, flags, x, y;
	size_t i;

	x = x0;
	y = y0;

	for(i = 0; i < step[0].GetCount(); i++)
	{
		deltax = step[TD_DELTAX][i]*scale;
		deltay = step[TD_DELTAY][i]*scale;
		flags  = step[TD_FLAGS ][i];		
		oldx = x;
		oldy = y;
		if(flags & MOVE_ONLY)
		{
			x += deltax;
			y += deltay;
		}
		else
		{
			x += deltax;
			y += deltay;
			dc->DrawLine(oldx, oldy, x, y);
			dc->DrawPoint(x, y); /*because wxWindows doesn't draw that point*/
			if(flags & STAY_PUT)
			{
				x = oldx;
				y = oldy;
			}
		}
	}
}

void TurtleDraw(const wxString &commands, wxDC *dc, int x0, int y0, int scale)
{
	static int shown_error = 0;
	int oldx, oldy, x, y, linelen, deltax, deltay;
	bool move_only, stay_put;
	size_t i;
	wxString cmds(commands);

	x = x0;
	y = y0;
	move_only = stay_put = false;
	deltax = deltay = 0;
	cmds.MakeUpper();

	for(i = 0; i < cmds.length(); i++)
	{
		if(isspace(cmds[i])) /* ignore whitespace */
			continue;

		if(cmds[i] == 'B')
		{
			move_only = true;
			continue;
		}

		if(cmds[i] == 'N')
		{
			stay_put = true;
			continue;
		}

		switch(cmds[i])
		{
			case 'U': deltax =  0; deltay = -1; break;      /* Up    */
			case 'D': deltax =  0; deltay =  1; break;      /* Down  */
			case 'L': deltax = -1; deltay =  0; break;      /* Left  */
			case 'R': deltax =  1; deltay =  0; break;      /* Right */
			case 'E': deltax =  1; deltay = -1; break;      /* NorthEast */
			case 'F': deltax =  1; deltay =  1; break;      /* SouthEast */
			case 'G': deltax = -1; deltay =  1; break;      /* SouthWest */
			case 'H': deltax = -1; deltay = -1; break;      /* NorthWest */
			default:
				if(shown_error < 5)
				{
					wxString str;
					str.Printf("ERROR drawing \"%s\" (position %d: %c)",
						cmds.c_str(), i, (char)(cmds.GetChar(i)));
					wxMessageBox(str);
					shown_error++;
				}
		}

		i++;
		linelen = ScanNum(cmds, i)*scale;
		i--;

		oldx = x;
		oldy = y;
		if(move_only)
		{
			x += deltax*linelen;
			y += deltay*linelen;
		}
		else
		{
			x += deltax*linelen;
			y += deltay*linelen;
			dc->DrawLine(oldx, oldy, x, y);
			dc->DrawPoint(x, y); /*because wxWindows doesn't draw that point*/
			if(stay_put)
			{
				x = oldx;
				y = oldy;
			}
		}

		move_only = stay_put = false;
	}
}

static void DrawAbstractObject(int n, wxDC *dc, int x, int y, int scale,
							   bool scaleflag,
							   CONST wxArrayThreeInts ARR nGlyphs[],
							   CONST wxArrayThreeInts ARR nGlyphs2[],int maxgl)
{
	if(n >= maxgl) return;
	if((!scaleflag) || (scale & 1) || (nGlyphs2[n][TD_FLAGS][0] & NO_GLYPH))
		TurtleDrawCompiled(nGlyphs[n], dc, x, y, scale);
	else
		TurtleDrawCompiled(nGlyphs2[n], dc, x, y, scale >> 1);
#ifdef DEBUG_GLYPHS
	wxBrush tmpbrush = dc->GetBrush();
	dc->SetBrush(wxBrush(wxColour(*wxWHITE), wxTRANSPARENT));
	dc->DrawRectangle(x-5*scale,y-5*scale,10*scale,10*scale);
	dc->SetBrush(tmpbrush);
#endif
}

void DrawSign(int n, wxDC *dc, int x, int y, int scale)
{
	DrawAbstractObject(n, dc, x, y, scale, true,
		nDrawSign, nDrawSign2, cSign+2);
}

void DrawHouse(int n, wxDC *dc, int x, int y, int scale)
{
	DrawAbstractObject(n, dc, x, y, scale, true,
		nDrawHouse, nDrawHouse2, cSign+1);
}

void DrawAspect(int n, wxDC *dc, int x, int y, int scale)
{
	if(n == (DISPOSITOR_ASPECT - FIRST_ASPECT + 1))
	{
		DrawChar(RIGHT_ARROW_CHAR, dc, x, y, scale);
		return;
	}
	
	DrawAbstractObject(n, dc, x, y, scale, true,
		nDrawAspect, nDrawAspect2, cAspect+3);
}

void DrawObject(int n, wxDC *dc, int x, int y, int scale)
{
	DrawAbstractObject(n, dc, x, y, scale, true,
		nDrawObject, nDrawObject2, oNorm+7);
}

void DrawChar(int n, wxDC *dc, int x, int y, int scale)
{
	if(n == 0) return;
	DrawAbstractObject(n, dc, x, y, scale, false,
		nDrawCh, nDrawCh, 128-32+2);
}

/* Translates from the universal number to the Astrolog number and type */
void DrawGlyph(int n, wxDC *dc, int x, int y, int scale)
{
	int a;

	if(IS_SIGN(n))
	{
		DrawSign(n - FIRST_SIGN + 1, dc, x, y, scale);
	}
	else if(IS_HOUSE(n))
	{
		DrawObject(n - FIRST_HOUSE + 21, dc, x, y, scale);
	}
	else if(IS_ASPECT(n) || (n == DISPOSITOR_ASPECT))
	{
		DrawAspect(n - FIRST_ASPECT + 1, dc, x, y, scale);
	}
	else if(IS_PLANET(n))
	{
		DrawObject(n - FIRST_PLANET + 1, dc, x, y, scale);
	}
	else if((n >= TURTLE_DRAW_ASCII) && (n < (TURTLE_DRAW_ASCII+128)))
	{
		DrawChar(n - TURTLE_DRAW_ASCII, dc, x, y, scale);
	}
	else if(IS_OTHER_BODY(n))
	{
		a = -1;
		switch(n)
		{
				case EARTH:     a = 0;  break;
                case CHIRON:    a = 11; break;
                case LILITH:    a = 17; break;
                case CERES:     a = 12; break;
                case PALLAS:    a = 13; break;
                case JUNO:      a = 14; break;
                case VESTA:     a = 15; break;
                case NNODE:     a = 16; break;
                case SNODE:     a = 17; break;
                case PFORTUNE:  a = 18; break;
                case VERTEX:    a = 19; break;
                case EPOINT:    a = 20; break;
                case MEAN_NODE: a = 16; break;
                case TRUE_NODE: a = 16; break;
                case MEAN_APOG: a = 16; break;
                case OSCU_APOG: a = 16; break;
                case U_CUPIDO:     a = 33; break;
                case U_HADES:      a = 34; break;
                case U_ZEUS:       a = 35; break;
                case U_KRONOS:     a = 36; break;
                case U_APOLLON:    a = 37; break;
                case U_ADMETOS:    a = 38; break;
                case U_VULKANUS:   a = 39; break;
                case U_POSEIDON:   a = 40; break;
                case U_PROSERPINA: a = 46; break;
				case PHOLUS:       a = 45; break;
		}
		if(a >= 0)
			DrawObject(a, dc, x, y, scale);
	}
}

wxColour DefaultObjectColour(int colConst, unsigned char intensity)
{
	const unsigned char intens = intensity;
	switch(colConst)
	{
	case colFIRE:     return wxColour(intens, 0, 0);
	case colEARTH:    return wxColour(3*intens/4, 3*intens/4, 0);
	case colAIR:      return wxColour(0, intens, intens/8);
	case colWATER:    return wxColour(intens/4, intens/4, intens);
	case colCARDINAL: return DefaultObjectColour(colFIRE, intensity);
	case colMUTABLE:  return DefaultObjectColour(colWATER, intensity);
	case colFIXED:    return DefaultObjectColour(colEARTH, intensity);
	case colCHIRON:   return wxColour(intens, 0, intens);
	case colOTHER:    return wxColour(0, intens/2, intens/2);
	case colURANIAN:  return wxColour(intens/2, 0, intens/2);
	case colERROR:    return wxColour(intens/2, intens/4, intens/4);
	}

	return(wxColour(  0,  0,  0)); /* return black on unknown color type */
}

wxColour DefaultAspectColour(int n, unsigned char intensity)
{
	const unsigned char intens = intensity;
	switch(n)
	{
	case CONJUNCTION:       return wxColour(intens, intens, 0);
	case OPPOSITION:        return wxColour(intens/3, intens/3, intens);
	case SQUARE:            return wxColour(intens, 0, 0);
	case SEXTILE:           return wxColour(0, 3*intens/4, 3*intens/4);
	case TRINE:             return wxColour(0, intens, 0);
	case INCONJUNCT:        return wxColour(intens, 0, intens);
	case SEMISEXTILE:       return wxColour(intens/2, intens/2, 0);
	case SEMISQUARE:        return wxColour(intens, intens/2, 0);
	case SESQUIQUADRATURE:  return wxColour(intens, intens/2, 0);
	case QUINTILE:          return wxColour(0, intens/2, intens/2);
	case BIQUINTILE:        return wxColour(0, intens/2, intens/2);
	case SEMIQUINTILE:      return wxColour(0, intens/2, intens/2);
	case SEPTILE:           return wxColour(intens/2, intens/2, 0);
	case NOVILE:            return wxColour(intens/2, 0, intens/2);
	case BINOVILE:          return wxColour(intens/2, 0, intens/2);
	case BISEPTILE:         return wxColour(intens/2, intens/2, 0);
	case TRISEPTILE:        return wxColour(intens/2, intens/2, 0);
	case QUATRONOVILE:      return wxColour(intens/2, 0, intens/2);
	case DISPOSITOR_ASPECT: return wxColour(3*intens/4, 3*intens/4, 3*intens/4);
	}

	return(wxColour(  0,  0,  0)); /* return black on unknown aspect */
}
