//Typewriter.h
//
//Copyright (C) 2003 mikesag
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//In general, the drawing routines here function like a typewriter;
//in other words, draw a line, then move the "cursor" down by
//however much the previous line took plus some whitespace
//Hence, the analogy of a typewriter is used throughout.
//This is quite useful in the information panel and the tool
//windows, as there a lot of data needs to be displayed
//sequentially and doesn't need to be precisely placable as
//in the chart.  Hence a lot of the "grunge-work" needed to
//display the data and keep track of the position is done
//automatically in the class.

#ifndef Typewriter_H
#define Typewriter_H

#ifndef wxNO_ALIGN
#define wxNO_ALIGN 0x80
#endif

class CTypewriterDraw
{
public:
	CTypewriterDraw();

	void Reset(int xmargain, int ymargain, int totw, int toth);
	void SetMargains(int xmargain, int ymargain);
	void NewLine(int size);
	void Resposition(int newx, int newy);
	void TypeText(wxDC &dc, const wxString &s, const wxColour &col, int ptsize,
		bool bold = false, int just = wxLEFT, int family = wxMODERN);
	void TypeGlyph(wxDC &dc, int n, const wxColour &col, int scale, int thickness,
		bool centered = FALSE, bool newline_at_end = FALSE);
	int TextExtentX(wxDC &dc, const wxString &s, int ptsize,
		bool bold = false, int family = wxMODERN);
	void TypeSignPos(wxDC &dc, int n, double longtitude, double speed,
		int glscale, int textscale, int align,
		bool newline_at_end = FALSE);
	void TypeLine(wxDC &dc, const wxColour &textcol, int initscale,
		int align, bool newline_at_end, const wxString &fmt,
		int toth = -1, int totscale = -1);
	int GetGlyphLineHeight(int scale);
	int GetX() { return x; }
	int GetY() { return y; }
	void SetX(int xx) { x = xx; }
	void SetY(int yy) { y = yy; }

private:
	int x, y, x0, y0, w, h, glsize;
};

#endif
